@@ -109,6 +109,10 @@ public:
 		: pool(_pool), level(0), root(NULL), defaultAccessor(this)
 	{ }
 
+	explicit BePlusTree(Allocator& _pool)
+		: pool(&_pool), level(0), root(NULL), defaultAccessor(this)
+	{ }
+
 	BePlusTree(Allocator *_pool, const BePlusTree& from)
 		: pool(_pool), level(0), root(NULL), defaultAccessor(this)
 	{
@@ -542,14 +546,8 @@ public:
 
 	    Value& current() const { return (*curr)[curPos]; }
 
-		// CVC: Here making this public is hack for MSVC6 that didn't acknowledge the
-		// friend declaration.
-#if defined(_MSC_VER) && _MSC_VER <= 1200
-	public:
-#else
 	private:
-#endif
-	
+
 		// Returns true if current position is valid and already points to the given key.
 		// Note that we can't guarantie validity of current position if tree is accessed 
 		// by different Accessor's. Therefore this method is private and can be used only 
@@ -666,13 +664,14 @@ bool BePlusTree<Value, Key, Allocator, KeyOfValue, Cmp, LeafCount, NodeCount>::a
 			
 	// Start building recovery map.
 	// This array contains index of the element we try to add on page of each level
-	// -1 means that element is on new page
+	// MAP_NEW_PAGE means that element is on new page
 	// In case of low memory condition we use this data to recover to innocent state
 	size_t recovery_map[MAX_TREE_LEVEL];
+	const size_t MAP_NEW_PAGE = ~((size_t) 0);
 			
 	if (pos == LeafCount) {
 		newLeaf->insert(0, item);
-		recovery_map[0] = -1;
+		recovery_map[0] = MAP_NEW_PAGE;
 	}
 	else {
 		newLeaf->insert(0, (*leaf)[LeafCount - 1]);
@@ -741,7 +740,7 @@ bool BePlusTree<Value, Key, Allocator, KeyOfValue, Cmp, LeafCount, NodeCount>::a
 			if (pos == NodeCount) {
 				NodeList::setNodeParentAndLevel(newNode, curLevel, newList);
 				newList->insert(0, newNode);
-				recovery_map[curLevel + 1] = -1;
+				recovery_map[curLevel + 1] = MAP_NEW_PAGE;
 			}
 			else {
 				void *t = (*nodeList)[NodeCount - 1];
@@ -773,7 +772,7 @@ bool BePlusTree<Value, Key, Allocator, KeyOfValue, Cmp, LeafCount, NodeCount>::a
 		while (curLevel) {
 			NodeList *itemL = reinterpret_cast<NodeList*>(newNode);
 			void *lower;
-		    if (recovery_map[curLevel] < 0) {
+		    if (recovery_map[curLevel] == MAP_NEW_PAGE) {
 				lower = (*itemL)[0];
 			} 
 			else {
@@ -788,7 +787,7 @@ bool BePlusTree<Value, Key, Allocator, KeyOfValue, Cmp, LeafCount, NodeCount>::a
 			curLevel--;
 		}
 		ItemList *itemL2 = reinterpret_cast<ItemList*>(newNode);
-		if (recovery_map[0] >= 0) {
+		if (recovery_map[0] != MAP_NEW_PAGE) {
 			itemL2->prev->remove(recovery_map[0]);
 			itemL2->prev->insert(itemL2->prev->getCount(), (*itemL2)[0]);
 		}
