@@ -25,18 +25,20 @@
  * MOD 29-Jun-2002
  */
 
+#include "../jrd/common.h"
 struct tok {
     USHORT tok_ident;
     const char* tok_string;
     USHORT tok_version;
-    bool specialHandling;
+    bool nonReserved;
 };
 
 typedef tok TOK;
+typedef const TOK* Tokens;
 
 // These symbols are exported
 extern "C" {
-int KEYWORD_stringIsAToken(const char*);
-const TOK* KEYWORD_getTokens();
+int API_ROUTINE KEYWORD_stringIsAToken(const char*);
+Tokens API_ROUTINE KEYWORD_getTokens();
 }
 
