@@ -34,6 +34,9 @@
 #include "../jrd/gds_proto.h"
 #include "../jrd/gdsassert.h"
 
+using MsgFormat::SafeArg;
+
+
 static void add_dimensions(STR, DUDLEY_FLD);
 static void add_field(STR, DUDLEY_FLD, DUDLEY_REL);
 static void add_files(STR, FIL, DBB);
@@ -85,7 +88,7 @@ static inline void check_dyn(str* dyn, const int l)
 	if (!(dyn->str_current - dyn->str_start + l <=  dyn->str_length)
 		&& !TRN_get_buffer (dyn, l) )
 	{
-		DDL_error_abort( NULL, 320, NULL, NULL, NULL, NULL, NULL );
+		DDL_error_abort( NULL, 320);
 	}
 }
 
@@ -124,7 +127,7 @@ void TRN_translate(void)
 	dyn->str_current = dyn->str_start =
 		reinterpret_cast<UCHAR*>(gds__alloc(8192));
 	if (!dyn->str_current)
-		DDL_error_abort(NULL, 14, 0, 0, 0, 0, 0);	/* msg 14: memory exhausted */
+		DDL_error_abort(NULL, 14);	/* msg 14: memory exhausted */
 	dyn->str_length = 8192;
 
 #ifdef DEBUG_GDS_ALLOC
@@ -135,7 +138,7 @@ void TRN_translate(void)
 	if (!dudleyGlob.DYN_file_name[0])
 		output_file = stdout;
 	else if (!(output_file = fopen(dudleyGlob.DYN_file_name, FOPEN_WRITE_TYPE))) {
-		DDL_msg_put(281, dudleyGlob.DYN_file_name, NULL, NULL, NULL, NULL);
+		DDL_msg_put(281, SafeArg() << dudleyGlob.DYN_file_name);
 		/* msg 281: gdef: can't open DYN output file: %s */
 		DDL_exit(FINI_ERROR);
 	}
@@ -265,7 +268,7 @@ void TRN_translate(void)
 				break;
 
 			default:
-				DDL_err(282, NULL, NULL, NULL, NULL, NULL);	/* msg 282: action not implemented yet */
+				DDL_err(282);	/* msg 282: action not implemented yet */
 			}
 
 	check_dyn(dyn, 2);
@@ -278,12 +281,12 @@ void TRN_translate(void)
 	case lan_undef:
 	case lan_c:
 		if (PRETTY_print_dyn(dyn->str_start, gen_dyn_c, 0, 0))
-			DDL_err(283, NULL, NULL, NULL, NULL, NULL);	/*msg 283: internal error during DYN pretty print */
+			DDL_err(283);	/*msg 283: internal error during DYN pretty print */
 		break;
 
 	case lan_cxx:
 		if (PRETTY_print_dyn(dyn->str_start, gen_dyn_cxx, 0, 0))
-			DDL_err(283, NULL, NULL, NULL, NULL, NULL);	/*msg 283: internal error during DYN pretty print */
+			DDL_err(283);	/*msg 283: internal error during DYN pretty print */
 		break;
 
 	case lan_pascal:
@@ -294,7 +297,7 @@ void TRN_translate(void)
 				   "   isc_dyn	: packed array [1..%d] of char := (\n",
 				   length);
 		if (PRETTY_print_dyn(dyn->str_start, gen_dyn_pas, 0, 1))
-			DDL_err(285, NULL, NULL, NULL, NULL, NULL);	/*msg 285: internal error during DYN pretty print */
+			DDL_err(285);	/*msg 285: internal error during DYN pretty print */
 		fprintf(output_file, "   );	(* end of DYN string *)\n");
 		break;
 
@@ -314,7 +317,7 @@ void TRN_translate(void)
 		fprintf(output_file,
 				   "isc_dyn_length: short_integer := %d;\n", length);
 		fprintf(output_file,
-				   "isc_dyn	: CONSTANT interbase.blr (1..%d) := (\n", length);
+				   "isc_dyn	: CONSTANT firebird.blr (1..%d) := (\n", length);
 		raw_ada(dyn);
 		break;
 
