@@ -53,6 +53,9 @@ const double REDUCE_SELECTIVITY_FACTOR_LESS = 0.05;
 const double REDUCE_SELECTIVITY_FACTOR_GREATER = 0.05;
 const double REDUCE_SELECTIVITY_FACTOR_STARTING = 0.01;
 
+const double REDUCE_SELECTIVITY_FACTOR_EQUALITY = 0.1;
+const double REDUCE_SELECTIVITY_FACTOR_INEQUALITY = 0.5;
+
 const double MAXIMUM_SELECTIVITY = 1.0;
 
 // Default (Minimum) cost (nr. of pages) for an index.
@@ -169,9 +172,13 @@ public:
 		bool outer, bool inner, jrd_nod** sortNode);
 	~OptimizerRetrieval();
 
-	InversionCandidate* getCost();
 	InversionCandidate* getInversion(RecordSource** rsb);
 
+	InversionCandidate* getCost()
+	{
+		return getInversion(NULL);
+	}
+
 protected:
 	jrd_nod* composeInversion(jrd_nod* node1, jrd_nod* node2, NOD_T node_type) const;
 	void findDependentFromStreams(jrd_nod* node, SortedStreamList* streamList) const;
