@@ -35,6 +35,7 @@
 #include "../jrd/err_proto.h"    /* Index error types */
 #include "../jrd/RecordNumber.h"
 #include "../jrd/sbm.h"
+#include "../jrd/LocksCache.h"
 
 /* 64 turns out not to be enough indexes */
 /* #define MAX_IDX		 64	*/	/* that should be plenty of indexes */
@@ -46,11 +47,11 @@ struct dsc;
 namespace Jrd {
 
 class jrd_rel;
-class jrd_tra;
 template <typename T> class vec;
 class jrd_req;
 struct temporary_key;
 class jrd_tra;
+class BtrPageGCLock;
 
 enum idx_null_state {
   idx_nulls_none,
@@ -94,7 +95,7 @@ struct IndexDescAlloc : public pool_alloc_rpt<index_desc> {
 
 const int idx_numeric		= 0;
 const int idx_string		= 1;
-// AB: idx_timestamp1 removed
+const int idx_timestamp1	= 2;
 const int idx_byte_array	= 3;
 const int idx_metadata		= 4;
 const int idx_sql_date		= 5;
@@ -137,6 +138,7 @@ struct index_insertion {
 	temporary_key*	iib_key;		/* varying string for insertion */
 	RecordBitmap* iib_duplicates;	/* spare bit map of duplicates */
 	jrd_tra*	iib_transaction;	/* insertion transaction */
+	BtrPageGCLock*	iib_dont_gc_lock;	// lock to prevent removal of splitted page
 };
 
 
@@ -224,6 +226,28 @@ const int irb_exclude_upper	= 64;			// exclude upper bound keys while scanning i
 
 typedef Firebird::HalfStaticArray<float, 4> SelectivityList;
 
+class GlobalRWLock;
+typedef LocksCache<CachedLock> BtrPageLocks;
+
+class BtrPageGCLock
+{
+public:
+	explicit BtrPageGCLock(Jrd::thread_db* tdbb)
+	{
+		m_lock = NULL;
+	}
+
+	~BtrPageGCLock();
+
+	static BtrPageLocks* getLocksCache(thread_db* tdbb);
+	void disablePageGC(thread_db* tdbb, const PageNumber &page);
+	void enablePageGC(thread_db* tdbb);
+	static bool isPageGCAllowed(thread_db* tdbb, const PageNumber &page);
+
+private:
+	GlobalRWLock *m_lock;
+};
+
 } //namespace Jrd
 
 #endif /* JRD_BTR_H */
