@@ -162,7 +162,6 @@ static void float_to_text(const dsc*, dsc*, FPTR_ERROR);
 static void integer_to_text(const dsc*, dsc*, FPTR_ERROR);
 static void string_to_datetime(const dsc*, GDS_TIMESTAMP*, EXPECT_DATETIME,
 							   FPTR_ERROR);
-static double power_of_ten(const int);
 
 #ifndef NATIVE_QUAD
 #ifndef WORDS_BIGENDIAN
@@ -405,7 +404,7 @@ double CVT_get_double(const dsc* desc, FPTR_ERROR err)
 						   later in this routine. */
 
 						if (exp >= SHORT_LIMIT)
-							(*err) (isc_arith_except, 0);
+							(*err) (isc_arith_except, isc_arg_end);
 					}
 					else if (*p == '-' && !digit_seen && !sign)
 						sign = -1;
@@ -438,7 +437,7 @@ double CVT_get_double(const dsc* desc, FPTR_ERROR err)
 			   the user know...  */
 
 			if (ABSOLUT(scale) > DBL_MAX_10_EXP)
-				(*err)(isc_arith_except, 0);
+				(*err)(isc_arith_except, isc_arg_end);
 
 /*
   Repeated division is a good way to mung the least significant bits
@@ -450,9 +449,9 @@ double CVT_get_double(const dsc* desc, FPTR_ERROR err)
  	    do value *= 10.; while (++scale);
 */
 			if (scale > 0)
-				value /= power_of_ten(scale);
+				value /= CVT_power_of_ten(scale);
 			else if (scale < 0)
-				value *= power_of_ten(-scale);
+				value *= CVT_power_of_ten(-scale);
 		}
 		return value;
 
@@ -465,7 +464,7 @@ double CVT_get_double(const dsc* desc, FPTR_ERROR err)
 		break;
 
 	default:
-		(*err) (isc_badblk, 0);	/* internal error */
+		(*err) (isc_badblk, isc_arg_end);	/* internal error */
 		break;
 	}
 
@@ -480,12 +479,12 @@ double CVT_get_double(const dsc* desc, FPTR_ERROR err)
    the user know... */
 
 	if (ABSOLUT(dscale) > DBL_MAX_10_EXP)
-		(*err) (isc_arith_except, 0);
+		(*err) (isc_arith_except, isc_arg_end);
 
 	if (dscale > 0)
-		value *= power_of_ten(dscale);
+		value *= CVT_power_of_ten(dscale);
 	else if (dscale < 0)
-		value /= power_of_ten(-dscale);
+		value /= CVT_power_of_ten(-dscale);
 
 	return value;
 }
@@ -551,12 +550,12 @@ SLONG CVT_get_long(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		else if (scale < 0)
 			do {
 				if ((val64 > INT64_LIMIT) || (val64 < -INT64_LIMIT))
-					(*err) (isc_arith_except, 0);
+					(*err) (isc_arith_except, isc_arg_end);
 				val64 *= 10;
 			} while (++scale);
 
 		if ((val64 > LONG_MAX_int64) || (val64 < LONG_MIN_int64))
-			(*err) (isc_arith_except, 0);
+			(*err) (isc_arith_except, isc_arg_end);
 		return (SLONG) val64;
 
 	case dtype_quad:
@@ -564,7 +563,7 @@ SLONG CVT_get_long(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		high = ((SLONG *) p)[HIGH_WORD];
 		if ((value >= 0 && !high) || (value < 0 && high == -1))
 			break;
-		(*err) (isc_arith_except, 0);
+		(*err) (isc_arith_except, isc_arg_end);
 		break;
 
 	case dtype_real:
@@ -589,9 +588,9 @@ SLONG CVT_get_long(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		}
 #endif
 		if (scale > 0)
-			d /= power_of_ten(scale);
+			d /= CVT_power_of_ten(scale);
 		else if (scale < 0)
-			d *= power_of_ten(-scale);
+			d *= CVT_power_of_ten(-scale);
 
 		if (d > 0)
 			d += 0.5 + eps;
@@ -606,12 +605,12 @@ SLONG CVT_get_long(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		if (d < (double) LONG_MIN_real) {
 			if (d > (double) LONG_MIN_real - 1.)
 				return LONG_MIN;
-			(*err) (isc_arith_except, 0);
+			(*err) (isc_arith_except, isc_arg_end);
 		}
 		if (d > (double) LONG_MAX_real) {
 			if (d < (double) LONG_MAX_real + 1.)
 				return LONG_MAX_int;
-			(*err) (isc_arith_except, 0);
+			(*err) (isc_arith_except, isc_arg_end);
 		}
 		return (SLONG) d;
 
@@ -635,7 +634,7 @@ SLONG CVT_get_long(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		break;
 
 	default:
-		(*err)(isc_badblk, 0);	/* internal error */
+		(*err)(isc_badblk, isc_arg_end);	/* internal error */
 		break;
 	}
 
@@ -662,7 +661,7 @@ SLONG CVT_get_long(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 	else if (scale < 0) {
 		do {
 			if (value > LONG_LIMIT || value < -LONG_LIMIT)
-				(*err) (isc_arith_except, 0);
+				(*err) (isc_arith_except, isc_arg_end);
 			value *= 10;
 		} while (++scale);
 	}
@@ -868,7 +867,7 @@ SQUAD CVT_get_quad(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 				return QUAD_MIN_int;
 			else if (d < (double) QUAD_MAX_real + 1.)
 				return QUAD_MAX_int;
-			(*err)(isc_arith_except, 0);
+			(*err)(isc_arith_except, isc_arg_end);
 		}
 		return QUAD_FROM_DOUBLE(d, err);
 
@@ -892,7 +891,7 @@ SQUAD CVT_get_quad(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		break;
 
 	default:
-		(*err)(isc_badblk, 0);	/* internal error */
+		(*err)(isc_badblk, isc_arg_end);	/* internal error */
 		break;
 	}
 
@@ -902,7 +901,7 @@ SQUAD CVT_get_quad(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		return value;
 
 #ifndef NATIVE_QUAD
-	(*err)(isc_badblk, 0);	/* internal error */
+	(*err)(isc_badblk, isc_arg_end);	/* internal error */
 #else
 	if (scale > 0) {
 		SLONG fraction = 0;
@@ -925,7 +924,7 @@ SQUAD CVT_get_quad(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 	else {
 		do {
 			if (value > QUAD_LIMIT || value < -QUAD_LIMIT)
-				(*err) (isc_arith_except, 0);
+				(*err) (isc_arith_except, isc_arg_end);
 			value *= 10;
 		} while (++scale);
 	}
@@ -999,9 +998,9 @@ SINT64 CVT_get_int64(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		}
 #endif
 		if (scale > 0)
-			d /= power_of_ten(scale);
+			d /= CVT_power_of_ten(scale);
 		else if (scale < 0)
-			d *= power_of_ten(-scale);
+			d *= CVT_power_of_ten(-scale);
 
 		if (d > 0)
 			d += 0.5 + eps;
@@ -1020,7 +1019,7 @@ SINT64 CVT_get_int64(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		   double, and thus will have no effect on the sum. */
 
 		if (d < (double) QUAD_MIN_real || (double) QUAD_MAX_real < d)
-			(*err) (isc_arith_except, 0);
+			(*err) (isc_arith_except, isc_arg_end);
 
 		return (SINT64) d;
 
@@ -1044,7 +1043,7 @@ SINT64 CVT_get_int64(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 		break;
 
 	default:
-		(*err) (isc_badblk, 0);	/* internal error */
+		(*err) (isc_badblk, isc_arg_end);	/* internal error */
 		break;
 	}
 
@@ -1071,7 +1070,7 @@ SINT64 CVT_get_int64(const dsc* desc, SSHORT scale, FPTR_ERROR err)
 	else if (scale < 0) {
 		do {
 			if (value > INT64_LIMIT || value < -INT64_LIMIT)
-				(*err) (isc_arith_except, 0);
+				(*err) (isc_arith_except, isc_arg_end);
 			value *= 10;
 		} while (++scale);
 	}
@@ -1281,7 +1280,57 @@ USHORT CVT_make_string(const dsc*          desc,
 }
 
 
-void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
+double CVT_power_of_ten(const int scale)
+{
+/*************************************
+ *
+ *      p o w e r _ o f _ t e n
+ *
+ *************************************
+ *
+ * Functional description
+ *      return 10.0 raised to the scale power for 0 <= scale < 320.
+ *
+ *************************************/
+
+	/* Note that we could speed things up slightly by making the auxiliary
+	 * arrays global to this source module and replacing this function with
+	 * a macro, but the old code did up to 308 multiplies to our 1, and
+	 * that seems enough of a speed-up for now.
+	 */
+
+	static const double upper_part[] =
+		{ 1.e000, 1.e032, 1.e064, 1.e096, 1.e128,
+		1.e160, 1.e192, 1.e224, 1.e256, 1.e288
+	};
+
+	static const double lower_part[] =
+		{ 1.e00, 1.e01, 1.e02, 1.e03, 1.e04, 1.e05,
+		1.e06, 1.e07, 1.e08, 1.e09, 1.e10, 1.e11,
+		1.e12, 1.e13, 1.e14, 1.e15, 1.e16, 1.e17,
+		1.e18, 1.e19, 1.e20, 1.e21, 1.e22, 1.e23,
+		1.e24, 1.e25, 1.e26, 1.e27, 1.e28, 1.e29,
+		1.e30, 1.e31
+	};
+
+	/* The sole caller of this function checks for scale <= 308 before calling,
+	 * but we just fb_assert the weakest precondition which lets the code work.
+	 * If the size of the exponent field, and thus the scaling, of doubles
+	 * gets bigger, increase the size of the upper_part array.
+	 */
+	fb_assert((scale >= 0) && (scale < 320));
+
+	/* Note that "scale >> 5" is another way of writing "scale / 32",
+	 * while "scale & 0x1f" is another way of writing "scale % 32".
+	 * We split the scale into the lower 5 bits and everything else,
+	 * then use the "everything else" to index into the upper_part array,
+	 * whose contents increase in steps of 1e32.
+	 */
+	return upper_part[scale >> 5] * lower_part[scale & 0x1f];
+}
+
+
+void FB_EXPORTED CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 {
 /**************************************
  *
@@ -1510,7 +1559,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 				else
 #endif
 #endif
-					(*err) (isc_arith_except, 0);
+					(*err) (isc_arith_except, isc_arg_end);
 #ifndef REQUESTER
 #ifndef SUPERCLIENT
 			}
@@ -1543,7 +1592,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 				length = MIN(length, to->dsc_length);
 #if !defined(REQUESTER) && !defined(SUPERCLIENT)
 				if (toCharSet && !toCharSet->wellFormed(length, q))
-					(*err)(isc_malformed_string, 0);
+					(*err)(isc_malformed_string, isc_arg_end);
 				toLength = length;
 #endif
 
@@ -1571,7 +1620,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 				length = MIN(length, to->dsc_length - 1);
 #if !defined(REQUESTER) && !defined(SUPERCLIENT)
 				if (toCharSet && !toCharSet->wellFormed(length, q))
-					(*err)(isc_malformed_string, 0);
+					(*err)(isc_malformed_string, isc_arg_end);
 				toLength = length;
 #endif
 
@@ -1585,7 +1634,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 					MIN(length, (SLONG) (to->dsc_length - sizeof(USHORT)));
 #if !defined(REQUESTER) && !defined(SUPERCLIENT)
 				if (toCharSet && !toCharSet->wellFormed(length, q))
-					(*err)(isc_malformed_string, 0);
+					(*err)(isc_malformed_string, isc_arg_end);
 				toLength = length;
 #endif
 
@@ -1609,7 +1658,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 					toLength != 31 &&	// allow non CHARSET_LEGACY_SEMANTICS to be used as connection charset
 					toCharSet->length(toLength, start, false) > to_size / toCharSet->maxBytesPerChar())
 				{
-					(*err)(isc_arith_except, 0);
+					(*err)(isc_arith_except, isc_arg_end);
 				}
 			}
 #endif
@@ -1621,7 +1670,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 
 				do {
 					if (*q++ != fill_char)
-						(*err) (isc_arith_except, 0);
+						(*err) (isc_arith_except, isc_arg_end);
 				} while (--l);
 			}
 			return;
@@ -1666,7 +1715,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 
 		if (to->dsc_dtype != from->dsc_dtype)
 			(*err) (isc_wish_list, isc_arg_gds, isc_blobnotsup,
-					isc_arg_string, "move", 0);
+					isc_arg_string, "move", isc_arg_end);
 
 		/* Note: DSC_EQUIV failed above as the blob sub_types were different,
 		 * or their character sets were different.  In V4 we aren't trying
@@ -1683,7 +1732,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 		/* fb_assert(l <= MAX_SSHORT); */
 		*(SSHORT *) p = (SSHORT) l;
 		if (*(SSHORT *) p != l)
-			(*err) (isc_arith_except, 0);
+			(*err) (isc_arith_except, isc_arg_end);
 		return;
 
 	case dtype_long:
@@ -1707,7 +1756,7 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 		{
 			double d_value = CVT_get_double(from, err);
 			if (ABSOLUT(d_value) > FLOAT_MAX)
-				(*err) (isc_arith_except, 0);
+				(*err) (isc_arith_except, isc_arg_end);
 			*(float*) p = (float) d_value;
 		}
 		return;
@@ -1732,10 +1781,10 @@ void CVT_move(const dsc* from, dsc* to, FPTR_ERROR err)
 	if (from->dsc_dtype == dtype_array || from->dsc_dtype == dtype_blob)
 	{
 		(*err) (isc_wish_list, isc_arg_gds, isc_blobnotsup,
-		        isc_arg_string, "move", 0);
+		        isc_arg_string, "move", isc_arg_end);
 	}
 
-	(*err) (isc_badblk, 0);	/* internal error */
+	(*err) (isc_badblk, isc_arg_end);	/* internal error */
 }
 
 
@@ -1770,7 +1819,7 @@ static void conversion_error(const dsc* desc, FPTR_ERROR err)
 #endif
 	}
 
-	(*err)(isc_convert_error, isc_arg_string, p, 0);
+	(*err)(isc_convert_error, isc_arg_string, p, isc_arg_end);
 }
 
 
@@ -1813,7 +1862,7 @@ static void datetime_to_text(const dsc* from, dsc* to, FPTR_ERROR err)
 		break;
 	default:
 		fb_assert(false);
-		(*err) (isc_badblk, 0);	/* internal error */
+		(*err) (isc_badblk, isc_arg_end);	/* internal error */
 		break;
 	}
 
@@ -1917,7 +1966,7 @@ static SSHORT decompose(const char* string,
    supported by the platform as a native datatype. */
 
 	if (dtype == dtype_quad)
-		(*err) (isc_badblk, 0);	/* internal error */
+		(*err) (isc_badblk, isc_arg_end);	/* internal error */
 #endif
 
 	dsc errd;
@@ -1956,11 +2005,11 @@ static SSHORT decompose(const char* string,
 			if (value >= limit_by_10) {
 				/* possibility of an overflow */
 				if (value > limit_by_10)
-					(*err) (isc_arith_except, 0);
+					(*err) (isc_arith_except, isc_arg_end);
 				else if (((*p > '8') && (sign == -1))
 						 || ((*p > '7') && (sign != -1)))
 				{
-					(*err) (isc_arith_except, 0);
+					(*err) (isc_arith_except, isc_arg_end);
 				}
 			}
 
@@ -2016,7 +2065,7 @@ static SSHORT decompose(const char* string,
 				   applied to the value. */
 
 				if (exp >= SHORT_LIMIT)
-					(*err) (isc_arith_except, 0);
+					(*err) (isc_arith_except, isc_arg_end);
 			}
 			else if (*p == '-' && !digit_seen && !sign)
 				sign = -1;
@@ -2162,7 +2211,7 @@ static void float_to_text(const dsc* from, dsc* to, FPTR_ERROR err)
 			/* If we cannot print at least two digits, one on each side of the
 			   ".", report an overflow exception. */
 			if (precision < 2)
-				(*err) (isc_arith_except, 0);
+				(*err) (isc_arith_except, isc_arg_end);
 
 			chars_printed = sprintf(temp, num_format, width, precision, d);
 
@@ -2174,7 +2223,7 @@ static void float_to_text(const dsc* from, dsc* to, FPTR_ERROR err)
 			if (chars_printed > width) {
 				precision -= (chars_printed - width);
 				if (precision < 2)
-					(*err) (isc_arith_except, 0);
+					(*err) (isc_arith_except, isc_arg_end);
 			    chars_printed = sprintf(temp, num_format, width, precision, d);
 			}
 		}
@@ -2230,7 +2279,7 @@ static void integer_to_text(const dsc* from, dsc* to, FPTR_ERROR err)
    supported by the platform as a native datatype. */
 
 	if (from->dsc_dtype == dtype_quad)
-		(*err) (isc_badblk, 0);	/* internal error */
+		(*err) (isc_badblk, isc_arg_end);	/* internal error */
 #endif
 
 	SSHORT pad_count = 0, decimal = 0, neg = 0;
@@ -2719,7 +2768,7 @@ static void string_to_datetime(
 	ts.encode(&times);
 
 	if (!ts.isRangeValid()) {
-		(*err) (isc_date_range_exceeded, 0);
+		(*err) (isc_date_range_exceeded, isc_arg_end);
 	}
 
 	if (expect_type != expect_sql_time) {
@@ -2745,54 +2794,3 @@ static void string_to_datetime(
 
 	date->timestamp_time += components[6];
 }
-
-
-double power_of_ten(const int scale)
-{
-/*************************************
- *
- *      p o w e r _ o f _ t e n
- *
- *************************************
- *
- * Functional description
- *      return 10.0 raised to the scale power for 0 <= scale < 320.
- *
- *************************************/
-
-	/* Note that we could speed things up slightly by making the auxiliary
-	 * arrays global to this source module and replacing this function with
-	 * a macro, but the old code did up to 308 multiplies to our 1, and
-	 * that seems enough of a speed-up for now.
-	 */
-
-	static const double upper_part[] =
-		{ 1.e000, 1.e032, 1.e064, 1.e096, 1.e128,
-		1.e160, 1.e192, 1.e224, 1.e256, 1.e288
-	};
-
-	static const double lower_part[] =
-		{ 1.e00, 1.e01, 1.e02, 1.e03, 1.e04, 1.e05,
-		1.e06, 1.e07, 1.e08, 1.e09, 1.e10, 1.e11,
-		1.e12, 1.e13, 1.e14, 1.e15, 1.e16, 1.e17,
-		1.e18, 1.e19, 1.e20, 1.e21, 1.e22, 1.e23,
-		1.e24, 1.e25, 1.e26, 1.e27, 1.e28, 1.e29,
-		1.e30, 1.e31
-	};
-
-	/* The sole caller of this function checks for scale <= 308 before calling,
-	 * but we just fb_assert the weakest precondition which lets the code work.
-	 * If the size of the exponent field, and thus the scaling, of doubles
-	 * gets bigger, increase the size of the upper_part array.
-	 */
-	fb_assert((scale >= 0) && (scale < 320));
-
-	/* Note that "scale >> 5" is another way of writing "scale / 32",
-	 * while "scale & 0x1f" is another way of writing "scale % 32".
-	 * We split the scale into the lower 5 bits and everything else,
-	 * then use the "everything else" to index into the upper_part array,
-	 * whose contents increase in steps of 1e32.
-	 */
-	return upper_part[scale >> 5] * lower_part[scale & 0x1f];
-}
-
