@@ -36,7 +36,6 @@
 #include "gen/iberror.h"
 #include "../jrd/intl.h"
 #include "../jrd/gdsassert.h"
-#include "../jrd/all_proto.h"
 #include "../jrd/cvt_proto.h"
 #include "../jrd/cvt2_proto.h"
 #include "../jrd/err_proto.h"
@@ -191,82 +190,6 @@ SSHORT CVT2_compare(const dsc* arg1, const dsc* arg2, FPTR_ERROR err)
 #endif
 
 		case dtype_text:
-			{
-				/*
-				 * For the sake of optimization, we call INTL_compare
-				 * only when we cannot just do byte-by-byte compare, as is
-				 * done in the foll. code.
-				 * We can do a local compare here, if
-				 *    (a) one of the arguments is charset ttype_binary
-				 * OR (b) both of the arguments are char set ttype_none
-				 * OR (c) both of the arguments are char set ttype_ascii
-				 * If any argument is ttype_dynamic, we must see the
-				 * charset of the attachment.
-				 */
-				SET_TDBB(tdbb);
-				CHARSET_ID charset1 = INTL_TTYPE(arg1);
-				if (charset1 == ttype_dynamic)
-					charset1 = INTL_charset(tdbb, charset1);
-
-				CHARSET_ID charset2 = INTL_TTYPE(arg2);
-				if (charset2 == ttype_dynamic)
-					charset2 = INTL_charset(tdbb, charset2);
-
-				if ((IS_INTL_DATA(arg1) || IS_INTL_DATA(arg2)) &&
-					(charset1 != ttype_binary) &&
-					(charset2 != ttype_binary) &&
-					((charset1 != ttype_ascii) ||
-					 (charset2 != ttype_ascii)) &&
-					((charset1 != ttype_none) || (charset2 != ttype_none))
-					)
-				{
-					return INTL_compare(tdbb, arg1, arg2, err);
-				}
-
-				USHORT length;
-				const UCHAR pad = charset1 == ttype_binary || charset2 == ttype_binary ? '\0' : ' ';
-				if (arg1->dsc_length >= arg2->dsc_length)
-				{
-					length = arg2->dsc_length;
-					if (length)
-						do
-							if (*p1++ != *p2++)
-								if (p1[-1] > p2[-1])
-									return 1;
-								else
-									return -1;
-						while (--length);
-					length = arg1->dsc_length - arg2->dsc_length;
-					if (length)
-						do
-							if (*p1++ != pad)
-								if (p1[-1] > pad)
-									return 1;
-								else
-									return -1;
-						while (--length);
-					return 0;
-				}
-				length = arg1->dsc_length;
-				if (length)
-					do
-						if (*p1++ != *p2++)
-							if (p1[-1] > p2[-1])
-								return 1;
-							else
-								return -1;
-					while (--length);
-				length = arg2->dsc_length - arg1->dsc_length;
-				do
-					if (*p2++ != pad)
-						if (pad > p2[-1])
-							return 1;
-						else
-							return -1;
-				while (--length);
-				return 0;
-			}
-
 		case dtype_varying:
 		case dtype_cstring:
 		case dtype_array:
@@ -584,8 +507,8 @@ SSHORT CVT2_blob_compare(const dsc* arg1, const dsc* arg2, FPTR_ERROR err)
 			bpbLength = sizeof(bpb);
 		}
 
-	    blb* blob1 = BLB_open(tdbb, tdbb->tdbb_request->req_transaction, (bid*) arg1->dsc_address);
-		blb* blob2 = BLB_open2(tdbb, tdbb->tdbb_request->req_transaction, (bid*) arg2->dsc_address, bpbLength, bpb);
+	    blb* blob1 = BLB_open(tdbb, tdbb->getRequest()->req_transaction, (bid*) arg1->dsc_address);
+		blb* blob2 = BLB_open2(tdbb, tdbb->getRequest()->req_transaction, (bid*) arg2->dsc_address, bpbLength, bpb);
 
 		if (charSet1->isMultiByte())
 		{
@@ -656,7 +579,7 @@ SSHORT CVT2_blob_compare(const dsc* arg1, const dsc* arg2, FPTR_ERROR err)
 
 		l2 = CVT2_make_string2(arg2, ttype1, &p, temp_str, err);
 
-		blb* blob1 = BLB_open(tdbb, tdbb->tdbb_request->req_transaction, (bid*) arg1->dsc_address);
+		blb* blob1 = BLB_open(tdbb, tdbb->getRequest()->req_transaction, (bid*) arg1->dsc_address);
 
 		if (charSet1->isMultiByte())
 			buffer1.getBuffer(blob1->blb_length);
