@@ -107,7 +107,7 @@ void ERR_bugcheck_msg(const TEXT* msg)
 	DEBUG;
 	CCH_shutdown_database(dbb);
 
-	ERR_post(isc_bug_check, isc_arg_string, ERR_cstring(msg), 0);
+	ERR_post(isc_bug_check, isc_arg_string, ERR_cstring(msg), isc_arg_end);
 }
 #endif
 
@@ -173,6 +173,9 @@ void ERR_duplicate_error(IDX_E	code,
 
 	thread_db* tdbb = JRD_get_thread_data();
 
+	ISC_STATUS_ARRAY org_status;
+	memcpy(org_status, tdbb->tdbb_status_vector, sizeof(ISC_STATUS_ARRAY));
+
 	MET_lookup_index(tdbb, index, relation->rel_name, index_number + 1);
 	if (index.length()) {
 		index_name = ERR_cstring(index);
@@ -187,10 +190,12 @@ void ERR_duplicate_error(IDX_E	code,
 		constraint_name = "***unknown***";
 	}
 
+	memcpy(tdbb->tdbb_status_vector, org_status, sizeof(ISC_STATUS_ARRAY));
+
 	switch (code) {
 	case idx_e_keytoobig:
 		ERR_post(isc_imp_exc, isc_arg_gds, isc_keytoobig,
-				 isc_arg_string, index_name, 0);
+				 isc_arg_string, index_name, isc_arg_end);
 		break;
 
 	case idx_e_conversion:
@@ -200,22 +205,26 @@ void ERR_duplicate_error(IDX_E	code,
 	case idx_e_foreign_target_doesnt_exist:
 		ERR_post(isc_foreign_key, isc_arg_string, constraint_name,
 			 	 isc_arg_string, ERR_cstring(relation->rel_name), 
-			 	 isc_arg_gds, isc_foreign_key_target_doesnt_exist, 0);
+			 	 isc_arg_gds, isc_foreign_key_target_doesnt_exist, isc_arg_end);
 		break;
 
 	case idx_e_foreign_references_present:
 		ERR_post(isc_foreign_key, isc_arg_string, constraint_name,
 			 	 isc_arg_string, ERR_cstring(relation->rel_name),
-			 	 isc_arg_gds, isc_foreign_key_references_present, 0);
+			 	 isc_arg_gds, isc_foreign_key_references_present, isc_arg_end);
 		break;
 
-	default:
+	case idx_e_duplicate:
 		if (constraint.length() > 0)
 			ERR_post(isc_unique_key_violation,
 					 isc_arg_string, constraint_name,
-					 isc_arg_string, ERR_cstring(relation->rel_name), 0);
+					 isc_arg_string, ERR_cstring(relation->rel_name), isc_arg_end);
 		else
-			ERR_post(isc_no_dup, isc_arg_string, index_name, 0);
+			ERR_post(isc_no_dup, isc_arg_string, index_name, isc_arg_end);
+		break;
+
+	default:
+		fb_assert(false);
 	}
 }
 #endif
@@ -241,7 +250,7 @@ void ERR_error(int number)
 	if (gds__msg_lookup(0, JRD_BUGCHK, number, sizeof(errmsg), errmsg, NULL) < 1)
 		sprintf(errmsg, "error code %d", number);
 
-	ERR_post(isc_random, isc_arg_string, ERR_cstring(errmsg), 0);
+	ERR_post(isc_random, isc_arg_string, ERR_cstring(errmsg), isc_arg_end);
 }
 #endif
 
@@ -262,7 +271,7 @@ void ERR_error_msg(const TEXT* msg)
  **************************************/
 
 	DEBUG;
-	ERR_post(isc_random, isc_arg_string, ERR_cstring(msg), 0);
+	ERR_post(isc_random, isc_arg_string, ERR_cstring(msg), isc_arg_end);
 }
 #endif
 
@@ -296,8 +305,7 @@ void ERR_log(int facility, int number, const TEXT* message)
 	fb_utils::snprintf(errmsg + len, sizeof(errmsg) - len, " (%d)", number);
 
 	gds__log("Database: %s\n\t%s", (tdbb && tdbb->getAttachment()) ?
-		tdbb->getAttachment()->att_filename.c_str() : "",
-		errmsg, 0);
+		tdbb->getAttachment()->att_filename.c_str() : "", errmsg);
 }
 #endif
 
@@ -651,7 +659,7 @@ static void internal_error(ISC_STATUS status, int number,
 		fb_utils::snprintf(errmsg + len, sizeof(errmsg) - len, " (%d)", number);
 	}
 
-	ERR_post(status, isc_arg_string, ERR_cstring(errmsg), 0);
+	ERR_post(status, isc_arg_string, ERR_cstring(errmsg), isc_arg_end);
 }
 #endif
 
