@@ -108,7 +108,10 @@ static bool request_completed(EVT_REQ);
 static ISC_STATUS return_ok(ISC_STATUS *);
 #ifdef MULTI_THREAD
 static THREAD_ENTRY_DECLARE watcher_thread(THREAD_ENTRY_PARAM);
-#endif
+#ifdef WIN_NT
+HANDLE watcher_thread_handle;
+#endif // WIN_NT
+#endif // MULTI_THREAD
 
 static SSHORT acquire_count;
 static EVH EVENT_header = NULL;
@@ -122,7 +125,6 @@ static SH_MEM_T EVENT_data;
 static MTX_T event_mutex[1];
 #endif
 
-
 void EVENT_cancel(SLONG request_id)
 {
 /**************************************
@@ -386,6 +388,12 @@ SLONG EVENT_que(ISC_STATUS* status_vector,
  * Functional description
  *
  **************************************/
+// Sanity check
+
+	if (events[0] != EPB_version1) {
+		ERR_post(isc_random, isc_arg_string, "Invalid EPB form", isc_arg_end);
+	}
+
 // Allocate request block
 
 	acquire();
@@ -423,6 +431,13 @@ SLONG EVENT_que(ISC_STATUS* status_vector,
 
 		const USHORT count = *p++;
 
+		/* Sanity check */
+
+		if (count > end - events) {
+			release();
+			ERR_post(isc_random, isc_arg_string, "Invalid EPB form", isc_arg_end);
+		}
+
 		/* The data in the event block may have trailing blanks.  Strip them off. */
 
         const UCHAR* find_end = p + count;
@@ -514,34 +529,16 @@ static EVH acquire(void)
 	if (EVENT_header->evh_length > EVENT_data.sh_mem_length_mapped) {
 		const SLONG length = EVENT_header->evh_length;
 
-#ifdef WIN_NT
-		/* Before remapping the memory, wakeup the watcher thread.
-		 * Then remap the shared memory and allow the watcher thread
-		 * to remap.
-		 */
-		/* Need to make the following code generic to all SUPERSERVER
-		 * platforms. Postponed for now. B.Sriram, 10-Jul-1997
-		 */
-
+		EVH header = NULL;
+#if (defined HAVE_MMAP || defined WIN_NT)
 		PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		process->prb_flags |= PRB_remap;
-		event_t* event = process->prb_event;
-
-		post_process(process);
-
-		while (true) {
-			release();
-			Sleep(3);
-			acquire();
-
-			process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-			if (!(process->prb_flags & PRB_remap))
-				break;
+		if (process->prb_flags & PRB_dont_remap)
+		{
+			ISC_event_post(process->prb_event);
+			while (process->prb_flags & PRB_dont_remap)
+				THREAD_SLEEP(10);
 		}
-#endif /* WIN_NT */
 
-	EVH header = NULL;
-#if (!(defined SUPERSERVER) && (defined HAVE_MMAP))
 		ISC_STATUS_ARRAY status_vector;
 		header = (evh*) ISC_remap_file(status_vector, &EVENT_data, length, false);
 #endif
@@ -551,11 +548,6 @@ static EVH acquire(void)
 			exit(FINI_ERROR);
 		}
 		EVENT_header = header;
-
-#ifdef WIN_NT
-		process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		process->prb_flags &= ~PRB_remap_over;
-#endif /* WIN_NT */
 	}
 
 	return EVENT_header;
@@ -595,33 +587,16 @@ static FRB alloc_global(UCHAR type, ULONG length, bool recurse)
 		const SLONG old_length = EVENT_data.sh_mem_length_mapped;
 		const SLONG ev_length = old_length + EVENT_EXTEND_SIZE;
 
-#ifdef WIN_NT
-
-		/* Before remapping the memory, wakeup the watcher thread.
-		 * Then remap the shared memory and allow the watcher thread
-		 * to remap.
-		 */
-
-		PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		process->prb_flags |= PRB_remap;
-		event_t* event = process->prb_event;
-		post_process(process);
-
-		while (true) {
-			release();
-			THREAD_EXIT();
-			Sleep(3);
-			THREAD_ENTER();
-			acquire();
-
-			process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-			if (!(process->prb_flags & PRB_remap))
-				break;
-		}
-#endif /* WIN_NT */
-
 		EVH header = 0;
-#if !((defined SUPERSERVER) && (defined HAVE_MMAP))
+#if (defined HAVE_MMAP || defined WIN_NT)
+		PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
+		if (process->prb_flags & PRB_dont_remap)
+		{
+			ISC_event_post(process->prb_event);
+			while (process->prb_flags & PRB_dont_remap)
+				THREAD_SLEEP(10);
+		}
+
 		ISC_STATUS_ARRAY status_vector;
 		header =
 			reinterpret_cast<EVH>
@@ -642,11 +617,6 @@ static FRB alloc_global(UCHAR type, ULONG length, bool recurse)
 
 			free_global(free);
 
-#ifdef WIN_NT
-			process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-			process->prb_flags &= ~PRB_remap_over;
-#endif /* WIN_NT */
-
 			return alloc_global(type, length, true);
 		}
 	}
@@ -714,7 +684,11 @@ static SLONG create_process(void)
 	release();
 
 #ifdef MULTI_THREAD
+#ifdef WIN_NT
+	if (gds__thread_start(watcher_thread, NULL, THREAD_medium, THREAD_blast, &watcher_thread_handle))
+#else
 	if (gds__thread_start(watcher_thread, NULL, THREAD_medium, THREAD_blast, 0))
+#endif
 		ERR_bugcheck_msg("cannot start thread");
 #endif
 
@@ -760,46 +734,61 @@ static void delete_process(SLONG process_offset)
  **************************************/
 	PRB process = (PRB) SRQ_ABS_PTR(process_offset);
 
+#ifdef WIN_NT
+	// If embedded library is unloaded because of host process exits then all 
+	// threads are already terminated and we can't syncronize correctly with 
+	// (dead) watcher thread. We can just detect if watcher thread still alive 
+	// and act correspondingly.
+
+	DWORD exit_code = 0;
+	const bool watcher_alive = 
+		(GetExitCodeThread(watcher_thread_handle, &exit_code) && 
+		(exit_code == STILL_ACTIVE));
+#else
+	const bool watcher_alive = true;
+#endif
+
 /* Delete any open sessions */
 
 	while (!SRQ_EMPTY(process->prb_sessions)) {
 		SES session =
 			(SES) ((UCHAR *) SRQ_NEXT(process->prb_sessions) -
 				   OFFSET(SES, ses_sessions));
+
+#ifdef MULTI_THREAD
+		if (!watcher_alive) {
+			session->ses_flags &= ~SES_delivering;
+		}
+#endif
+
 		delete_session(SRQ_REL_PTR(session));
 	}
 
+	if (EVENT_process_offset == process_offset) {
+#ifdef MULTI_THREAD
+		/* Terminate the event watcher thread */
+
+		process->prb_flags |= PRB_exiting;
+		while (process->prb_flags & PRB_exiting) {
+			if (!watcher_alive) 
+				break;
+
+			ISC_event_post(process->prb_event);
+
+			release();
+			THREAD_SLEEP(10);
+			acquire();
+
+			process = (PRB) SRQ_ABS_PTR(process_offset);
+		}
+		EVENT_process_offset = 0;
+#endif
+	}
+
 /* Untangle and release process block */
 
 	remove_que(&process->prb_processes);
 	free_global((FRB) process);
-
-	if (EVENT_process_offset == process_offset) {
-#ifdef MULTI_THREAD
-		/* Terminate the event watcher thread */
-		/* When we come through the exit handler, the event semaphore might
-		   have already been released by another exit handler.  So we cannot
-		   use that semaphore to post the event.  Besides, the watcher thread 
-		   would be terminated anyway because the whole NLM is being unloaded. */
-		// CVC: NLM??? is this Novell Netware specific code???
-
-		process->prb_flags |= PRB_exiting;
-		BOOLEAN timeout = FALSE;
-		while (process->prb_flags & PRB_exiting && !timeout) {
-			ISC_event_post(process->prb_event);
-			SLONG value = ISC_event_clear(process->prb_event);
-			release();
-#ifdef SOLARIS_MT
-			event_t* events = EVENT_process->prb_event;
-#else
-			event_t* events = process->prb_event;
-#endif
-			timeout = ISC_event_wait(1, &events, &value, 5 * 1000000, 0, 0);
-			acquire();
-		}
-		EVENT_process_offset = 0;
-#endif
-	}
 }
 
 
@@ -859,9 +848,7 @@ static void delete_session(SLONG session_id)
 
 		// give a chance for delivering thread to detect SES_purge flag we just set
 		release();
-		THREAD_EXIT();
 		THREAD_SLEEP(100);
-		THREAD_ENTER();
 		acquire();
 
 		return;
@@ -1314,7 +1301,7 @@ static void post_process(PRB process)
 
 	process->prb_flags &= ~PRB_wakeup;
 	process->prb_flags |= PRB_pending;
-	release();
+//	release();
 
 #ifdef MULTI_THREAD
 	ISC_event_post(process->prb_event);
@@ -1326,7 +1313,7 @@ static void post_process(PRB process)
 #endif
 
 #endif
-	acquire();
+//	acquire();
 }
 
 
@@ -1396,12 +1383,12 @@ static void release(void)
 #ifdef MULTI_THREAD
 	EVENT_header->evh_current_process = 0;
 	if (mutex_state = ISC_mutex_unlock(MUTEX))
-		mutex_bugcheck("mutex lock", mutex_state);
+		mutex_bugcheck("mutex unlock", mutex_state);
 #else
 	if (!--acquire_count) {
 		EVENT_header->evh_current_process = 0;
 		if (mutex_state = ISC_mutex_unlock(MUTEX))
-			mutex_bugcheck("mutex lock", mutex_state);
+			mutex_bugcheck("mutex unlock", mutex_state);
 #ifdef UNIX
 		if (EVENT_process_offset) {
 			PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
@@ -1546,40 +1533,28 @@ static THREAD_ENTRY_DECLARE watcher_thread(THREAD_ENTRY_PARAM)
 
 		if (process->prb_flags & PRB_exiting) {
 			process->prb_flags &= ~PRB_exiting;
-			ISC_event_post(process->prb_event);
 			release();
 			break;
 		}
-#ifdef WIN_NT
-		if (process->prb_flags & PRB_remap) {
-			process->prb_flags |= PRB_remap_over;
-			process->prb_flags &= ~PRB_remap;
-			release();
-			while (true) {
-				Sleep(3);
-				acquire();
-				process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-				release();
-				if (!(process->prb_flags & PRB_remap_over))
-					break;
-			}
-			acquire();
-			process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		}
-#endif
 
 		SLONG value = ISC_event_clear(process->prb_event);
 		release();
 		deliver(NULL);
 		acquire();
 		process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		release();
+		
+		// we going to wait on event, allocated in shared memory
+		// forbid shared memory remapping until waiting
+		process->prb_flags |= PRB_dont_remap;
 #ifdef SOLARIS_MT
 		event_t* events = EVENT_process->prb_event;
 #else
 		event_t* events = process->prb_event;
 #endif
+		release();
+
 		ISC_event_wait(1, &events, &value, 0, 0, 0);
+		process->prb_flags &= ~PRB_dont_remap;
 	}
 	return 0;
 }
