@@ -555,7 +555,7 @@ struct AccessItem
 {
 	Firebird::MetaName		acc_security_name;
 	SLONG					acc_view_id;
-	Firebird::MetaName		acc_name;
+	Firebird::MetaName		acc_name, acc_r_name;
 	const TEXT*				acc_type;
 	SecurityClass::flags_t	acc_mask;
 
@@ -579,13 +579,17 @@ struct AccessItem
 		if ((v = i1.acc_name.compare(i2.acc_name)) != 0)
 			return v > 0;
 
+		if ((v = i1.acc_r_name.compare(i2.acc_r_name)) != 0)
+			return v > 0;
+
 		return false; // Equal
 	}
 
 	AccessItem(const Firebird::MetaName& security_name, SLONG view_id, 
-		const Firebird::MetaName& name, const TEXT* type, SecurityClass::flags_t mask)
+		const Firebird::MetaName& name, const TEXT* type,
+		SecurityClass::flags_t mask, const Firebird::MetaName& relName)
 	: acc_security_name(security_name), acc_view_id(view_id), acc_name(name),
-		acc_type(type), acc_mask(mask)
+		acc_r_name(relName), acc_type(type), acc_mask(mask)
 	{}
 };
 
@@ -730,8 +734,7 @@ typedef Firebird::GenericMap<Firebird::Pair<Firebird::Right<Item, ItemInfo> > >
 
 class CompilerScratch : public pool_alloc<type_csb>
 {
-public:
-	CompilerScratch(MemoryPool& p, size_t len, const Firebird::MetaName& domain_validation = Firebird::MetaName())
+	CompilerScratch(MemoryPool& p, size_t len, const Firebird::MetaName& domain_validation)
 	:	/*csb_blr(0),
 		csb_running(0),
 		csb_node(0),
@@ -762,6 +765,7 @@ public:
 		csb_rpt(p, len)
 	{}
 
+public:
 	static CompilerScratch* newCsb(MemoryPool& p, size_t len, const Firebird::MetaName& domain_validation = Firebird::MetaName())
 		{ return FB_NEW(p) CompilerScratch(p, len, domain_validation); }
 
@@ -769,7 +773,7 @@ public:
 	{
 		if (csb_n_stream >= MAX_STREAMS && check)
 		{
-			ERR_post(isc_too_many_contexts, 0);
+			ERR_post(isc_too_many_contexts, isc_arg_end);
 		}
 		return csb_n_stream++;
 	}
