@@ -242,7 +242,7 @@ static CLEAN	cleanup_handlers = NULL;
 static gds_msg* global_default_msg = NULL;
 static bool initialized = false;
 
-void* API_ROUTINE gds__alloc_debug(SLONG size_request,
+VoidPtr API_ROUTINE gds__alloc_debug(SLONG size_request,
                                    const TEXT* filename,
                                    ULONG lineno)
 {
@@ -2411,7 +2411,7 @@ void API_ROUTINE gds__sqlcode_s(const ISC_STATUS* status_vector, ULONG* sqlcode)
 }
 
 
-void* API_ROUTINE gds__temp_file(
+VoidPtr API_ROUTINE gds__temp_file(
 					 BOOLEAN stdio_flag, const TEXT* string,
 					 TEXT* expanded_string, TEXT* dir, BOOLEAN unlink_flag)
 {
@@ -2992,6 +2992,10 @@ static void blr_print_cond(gds_ctl* control)
 		blr_format(control, "blr_default_code, ");
 		break;
 
+	case blr_raise:
+		blr_format(control, "blr_raise, ");
+		break;
+
 	default:
 		blr_error(control, "*** invalid condition type ***");
 		break;
@@ -3617,7 +3621,7 @@ static void safe_concat_path(TEXT *resultString, const TEXT *appendString)
 }
 
 
-void gds__default_printer(void* arg, SSHORT offset, const TEXT* line)
+void FB_EXPORTED gds__default_printer(void* arg, SSHORT offset, const TEXT* line)
 {
 	printf("%4d %s\n", offset, line);
 }
@@ -3640,7 +3644,7 @@ void gds__trace_printer(void* arg, SSHORT offset, const TEXT* line)
 
 #undef gds__alloc
 
-void* API_ROUTINE gds__alloc(SLONG size_request)
+VoidPtr API_ROUTINE gds__alloc(SLONG size_request)
 {
 	return getDefaultMemoryPool()->allocate_nothrow(size_request, 0
 #ifdef DEBUG_GDS_ALLOC
