@@ -233,10 +233,10 @@ void IDX_create_index(
 
 	if (relation->rel_file) {
 		ERR_post(isc_no_meta_update, isc_arg_gds, isc_extfile_uns_op,
-				 isc_arg_string, ERR_cstring(relation->rel_name), 0);
+				 isc_arg_string, ERR_cstring(relation->rel_name), isc_arg_end);
 	}
 	else if (relation->isVirtual()) {
-		ERR_post(isc_no_meta_update, isc_arg_gds, isc_wish_list, 0);
+		ERR_post(isc_no_meta_update, isc_arg_gds, isc_wish_list, isc_arg_end);
 	}
 
 	get_root_page(tdbb, relation);
@@ -277,7 +277,7 @@ void IDX_create_index(
 				 isc_arg_gds,
 				 isc_keytoobig,
 				 isc_arg_string,
-				 ERR_cstring(index_name), 0);
+				 ERR_cstring(index_name), isc_arg_end);
 	}
 
 	RecordStack stack;
@@ -347,7 +347,12 @@ void IDX_create_index(
 /* Loop thru the relation computing index keys.  If there are old versions,
    find them, too. */
 	temporary_key key;
-	while (DPM_next(tdbb, &primary, LCK_read, false, false)) {
+	while (DPM_next(tdbb, &primary, LCK_read, 
+#ifdef SCROLLABLE_CURSORS
+					false, 
+#endif
+					false)) 
+	{
 		if (transaction && !VIO_garbage_collect(tdbb, &primary, transaction))
 			continue;
 		if (primary.rpb_flags & rpb_deleted)
@@ -392,36 +397,41 @@ void IDX_create_index(
 				else {
 					result = BTR_key(tdbb, relation, record, idx, &key, &null_state, false);
 				}
-				if (null_state != idx_nulls_none) {
-					result = idx_e_ok;
-				}
-				else {
-					result =
-						check_partner_index(tdbb, relation, record,
-											transaction, idx,
-											partner_relation,
-											partner_index_id);
-				}
-			}
 
-			if (result == idx_e_ok) {
-				idx_null_state null_state;
-				BTR_key(tdbb, relation, record, idx, &key, &null_state, false);
-				key_is_null = (null_state == idx_nulls_all);
-
-				if (isPrimary && null_state != idx_nulls_none) 
+				if (result == idx_e_ok && null_state == idx_nulls_none)
 				{
-					fb_assert(key.key_null_segment < idx->idx_count);
-
-					const USHORT bad_id = idx->idx_rpt[key.key_null_segment].idx_field;
-					const jrd_fld *bad_fld = MET_get_field(relation, bad_id);
-
-					ERR_post(isc_not_valid,
-						isc_arg_string, bad_fld->fld_name.c_str(),
-						isc_arg_string, NULL_STRING_MARK, 0);
+					result = check_partner_index(tdbb, relation, record,
+												 transaction, idx,
+												 partner_relation,
+												 partner_index_id);
 				}
 			}
-			else {
+
+			if (result == idx_e_ok)
+			{
+				idx_null_state null_state;
+				result = BTR_key(tdbb, relation, record, idx, &key, &null_state, false);
+
+				if (result == idx_e_ok)
+				{
+					if (isPrimary && null_state != idx_nulls_none) 
+					{
+						fb_assert(key.key_null_segment < idx->idx_count);
+
+						const USHORT bad_id = idx->idx_rpt[key.key_null_segment].idx_field;
+						const jrd_fld *bad_fld = MET_get_field(relation, bad_id);
+
+						ERR_post(isc_not_valid,
+							isc_arg_string, bad_fld->fld_name.c_str(),
+							isc_arg_string, NULL_STRING_MARK, isc_arg_end);
+					}
+
+					key_is_null = (null_state == idx_nulls_all);
+				}
+			}
+
+			if (result != idx_e_ok)
+			{
 				do {
 					if (record != gc_record)
 						delete record;
@@ -429,8 +439,7 @@ void IDX_create_index(
 				gc_record->rec_flags &= ~REC_gc_active;
 				if (primary.getWindow(tdbb).win_flags & WIN_large_scan)
 					--relation->rel_scan_count;
-				ERR_duplicate_error(result, partner_relation,
-									partner_index_id);
+				ERR_duplicate_error(result, relation, idx->idx_id);
 			}
 
 			if (key.key_length > key_length) {
@@ -441,7 +450,7 @@ void IDX_create_index(
 				gc_record->rec_flags &= ~REC_gc_active;
 				if (primary.getWindow(tdbb).win_flags & WIN_large_scan)
 					--relation->rel_scan_count;
-				ERR_post(isc_key_too_big, 0);
+				ERR_post(isc_key_too_big, isc_arg_end);
 			}
 
 			UCHAR* p;
@@ -458,7 +467,7 @@ void IDX_create_index(
 				if (primary.getWindow(tdbb).win_flags & WIN_large_scan)
 					--relation->rel_scan_count;
 				ERR_post(isc_no_dup, isc_arg_string,
-						 ERR_cstring(index_name), 0);
+						 ERR_cstring(index_name), isc_arg_end);
 			}
 
 			USHORT l = key.key_length;
@@ -494,7 +503,7 @@ void IDX_create_index(
 
 	if (ifl_data.ifl_duplicates > 0) {
 		ERR_post(isc_no_dup, isc_arg_string,
-				 ERR_cstring(index_name), 0);
+				 ERR_cstring(index_name), isc_arg_end);
 	}
 
 	}
@@ -509,7 +518,7 @@ void IDX_create_index(
 	if (ifl_data.ifl_duplicates > 0) {
 		// we don't need SORT_fini() here, as it's called inside BTR_create()
 		ERR_post(isc_no_dup, isc_arg_string,
-				 ERR_cstring(index_name), 0);
+				 ERR_cstring(index_name), isc_arg_end);
 	}
 
 	if ((relation->rel_flags & REL_temp_conn) &&
@@ -587,8 +596,7 @@ void IDX_delete_index(thread_db* tdbb, jrd_rel* relation, USHORT id)
 
 	signal_index_deletion(tdbb, relation, id);
 
-	RelationPages* relPages = relation->getPages(tdbb);
-	WIN window(relPages->rel_pg_space_id, relPages->rel_index_root);
+	WIN window(get_root_page(tdbb, relation));
 	CCH_FETCH(tdbb, &window, LCK_write, pag_root);
 
 	const bool tree_exists = BTR_delete_index(tdbb, &window, id);
@@ -625,6 +633,8 @@ void IDX_delete_indices(thread_db* tdbb, jrd_rel* relation, RelationPages* relPa
 
 	SET_TDBB(tdbb);
 
+	fb_assert(relPages->rel_index_root);
+
 	WIN window(relPages->rel_pg_space_id, relPages->rel_index_root);
 	index_root_page* root = (index_root_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_root);
 
@@ -720,8 +730,7 @@ void IDX_garbage_collect(thread_db*			tdbb,
 	insertion.iib_relation = rpb->rpb_relation;
 	insertion.iib_key = &key1;
 
-	RelationPages* relPages = rpb->rpb_relation->getPages(tdbb);
-	WIN window(relPages->rel_pg_space_id, relPages->rel_index_root);
+	WIN window(get_root_page(tdbb, rpb->rpb_relation));
 
 	index_root_page* root = (index_root_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_root);
 
