@@ -201,6 +201,17 @@ enum  info_db_implementations
 	isc_info_db_impl_linux_mipsel = 71,
 	isc_info_db_impl_linux_mips = 72,
 	isc_info_db_impl_darwin_x64 = 73,
+	isc_info_db_impl_sun_amd64 = 74,
+
+	isc_info_db_impl_linux_arm = 75,
+	isc_info_db_impl_linux_ia64 = 76,
+
+	isc_info_db_impl_darwin_ppc64 = 77,
+        isc_info_db_impl_linux_s390x = 78,
+        isc_info_db_impl_linux_s390 = 79,
+
+	isc_info_db_impl_linux_sh = 80,
+	isc_info_db_impl_linux_sheb = 81,
 
 	isc_info_db_impl_last_value   // Leave this LAST!
 };
