@@ -92,6 +92,7 @@ void NBackupState::invalidate(thread_db* tdbb, bool ast_handler)
 {
 	backup_manager->set_state(nbak_state_unknown);
 	NBAK_TRACE_AST( ("set state nbak_state_unknown") );
+	backup_manager->closeDelta();
 }
 
 void NBackupState::blockingAstHandler(thread_db* tdbb)
@@ -233,6 +234,28 @@ void BackupManager::generate_filename()
 	explicit_diff_name = false;
 }
 
+void BackupManager::openDelta()
+{
+    fb_assert(!diff_file);
+    diff_file = PIO_open(database, diff_name, false, diff_name, false);
+
+    if (database->dbb_flags & (DBB_force_write | DBB_no_fs_cache))
+    {
+        PIO_force_write(diff_file,
+            database->dbb_flags & DBB_force_write,
+            database->dbb_flags & DBB_no_fs_cache);
+    }
+}
+
+void BackupManager::closeDelta()
+{
+    if (diff_file)
+    {
+        PIO_close(diff_file);
+        diff_file = NULL;
+    }
+}
+
 // Initialize and open difference file for writing
 void BackupManager::begin_backup(thread_db* tdbb)
 {
@@ -240,7 +263,7 @@ void BackupManager::begin_backup(thread_db* tdbb)
 
 	// Check for raw device
 	if ((!explicit_diff_name) && database->onRawDevice()) {
-		ERR_post(isc_need_difference, 0);
+		ERR_post(isc_need_difference, isc_arg_end);
 	}
 
 	WIN window(HEADER_PAGE_NUMBER);
@@ -267,6 +290,12 @@ void BackupManager::begin_backup(thread_db* tdbb)
 		// Create file	
 		NBAK_TRACE(("Creating difference file %s", diff_name.c_str()));
 		diff_file = PIO_create(database, diff_name, true, false, false);
+		if (database->dbb_flags & (DBB_force_write | DBB_no_fs_cache))
+		{
+			PIO_force_write(diff_file, 
+				database->dbb_flags & DBB_force_write, 
+				database->dbb_flags & DBB_no_fs_cache);
+		}
 #ifdef UNIX 
 		// adjust difference file access rights to make it match main DB ones
 		if (diff_file && geteuid() == 0) {
@@ -529,11 +558,8 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 		last_allocated_page = 0;
 		if (!alloc_lock->tryReleaseLock(tdbb))
 			ERR_bugcheck_msg("There are holders of alloc_lock after end_backup finish");
-		
-		if (diff_file) {
-			PIO_close(diff_file);
-			diff_file = NULL;
-		}
+
+		closeDelta();
 		unlink(diff_name.c_str());
 		
 		if (database_locked)
@@ -697,6 +723,8 @@ bool BackupManager::write_difference(ISC_STATUS* status, ULONG diff_page, Ods::p
 	temp_bdb.bdb_page = diff_page;
 	temp_bdb.bdb_dbb = database;
 	temp_bdb.bdb_buffer = page;
+	// Check that diff page is not allocation page
+	fb_assert(diff_page % (database->dbb_page_size / sizeof(ULONG)));
 	if (!PIO_write(diff_file, &temp_bdb, page, status))
 		return false;
 	return true;
@@ -741,7 +769,9 @@ BackupManager::BackupManager(thread_db* tdbb, Database* _database, int ini_state
 void BackupManager::shutdown_locks(thread_db* tdbb)
 {
 	delete alloc_lock;
+	alloc_lock = NULL;
 	delete database_lock;
+	database_lock = NULL;
 }
 
 
@@ -837,11 +867,6 @@ bool BackupManager::actualize_state(thread_db* tdbb)
 		generate_filename();
 		
 	if (new_backup_state == nbak_state_normal || missed_cycle) {
-		if (diff_file) {
-			NBAK_TRACE(("Close difference file"));
-			PIO_close(diff_file);
-			diff_file = NULL;
-		}
 		// Page allocation table cache is no longer valid.
 		if (alloc_table) {
 			NBAK_TRACE(("Dropping alloc table"));
@@ -856,7 +881,7 @@ bool BackupManager::actualize_state(thread_db* tdbb)
 	if (new_backup_state != nbak_state_normal && !diff_file) {
 		try {
 			NBAK_TRACE(("Open difference file"));
-			diff_file = PIO_open(database, diff_name, false, diff_name, false);
+			openDelta();
 		}
 		catch (const Firebird::Exception& ex) {
 			Firebird::stuff_exception(status, ex);
@@ -871,8 +896,7 @@ bool BackupManager::actualize_state(thread_db* tdbb)
 
 void BackupManager::shutdown(thread_db* tdbb) 
 {
-	if (diff_file)
-		PIO_close(diff_file);
-	shutdown_locks(tdbb);
+    closeDelta();
+    shutdown_locks(tdbb);
 	delete this;
 }
