@@ -127,7 +127,7 @@ using namespace Jrd;
 #ifdef SUPERSERVER
 #define MASK		0600
 #else
-#define MASK		0666
+#define MASK		0660
 #endif
 
 #define FCNTL_BROKEN
@@ -237,13 +237,29 @@ jrd_file* PIO_create(Database* dbb, const Firebird::PathName& file_name, bool ov
 #endif
 #endif
 
-	const int desc = open(file_name.c_str(), flag, MASK);
+	const int desc = open(file_name.c_str(), flag, 0666);
 	if (desc == -1) 
 	{
 		ERR_post(isc_io_error,
 				 isc_arg_string, "open O_CREAT",
 				 isc_arg_string, ERR_string(file_name),
-				 isc_arg_gds, isc_io_create_err, isc_arg_unix, errno, 0);
+				 isc_arg_gds, isc_io_create_err, isc_arg_unix, errno, isc_arg_end);
+	}
+#ifdef HAVE_FCHMOD
+	if (fchmod(desc, MASK) < 0)
+#else
+	if (chmod(file_name.c_str(), MASK) < 0)
+#endif
+	{
+		int chmodError = errno;
+		// ignore possible errors in this calls - even if they have failed
+		// we cannot help much with former recovery
+		close(desc);
+		unlink(file_name.c_str());
+		ERR_post(isc_io_error,
+				 isc_arg_string, "chmod",
+				 isc_arg_string, ERR_string(file_name),
+				 isc_arg_gds, isc_io_create_err, isc_arg_unix, chmodError, isc_arg_end);
 	}
 
 	if (temporary
@@ -260,7 +276,7 @@ jrd_file* PIO_create(Database* dbb, const Firebird::PathName& file_name, bool ov
 			ERR_post(isc_io_error,
 					 isc_arg_string, "unlink",
 					 isc_arg_string, ERR_string(file_name),
-					 isc_arg_gds, isc_io_create_err, isc_arg_unix, errno, 0);
+					 isc_arg_gds, isc_io_create_err, isc_arg_unix, errno, isc_arg_end);
 		}
 #endif
 	}
@@ -348,6 +364,9 @@ void PIO_flush(jrd_file* main_file)
 #endif
 }
 
+#ifdef SOLARIS
+#define O_DIRECT 0
+#endif
 
 void PIO_force_write(jrd_file* file, bool forcedWrites, bool notUseFSCache)
 {
@@ -381,7 +400,7 @@ void PIO_force_write(jrd_file* file, bool forcedWrites, bool notUseFSCache)
 					 isc_arg_string, "fcntl() SYNC/DIRECT",
 					 isc_arg_cstring, file->fil_length,
 					 ERR_string(file->fil_string, file->fil_length), isc_arg_gds,
-					 isc_io_access_err, isc_arg_unix, errno, 0);
+					 isc_io_access_err, isc_arg_unix, errno, isc_arg_end);
 		}
 #else //FCNTL_BROKEN
 		maybeCloseFile(file->fil_desc);
@@ -391,8 +410,11 @@ void PIO_force_write(jrd_file* file, bool forcedWrites, bool notUseFSCache)
 		{
 			ERR_post(isc_io_error, isc_arg_string, "re open() for SYNC/DIRECT",
 					 isc_arg_cstring, file->fil_length, ERR_string(file->fil_string, file->fil_length),
-					 isc_arg_gds, isc_io_open_err, isc_arg_unix, errno, 0);
+					 isc_arg_gds, isc_io_open_err, isc_arg_unix, errno, isc_arg_end);
 		}
+#ifdef SOLARIS
+		directio(file->fil_desc, notUseFSCache ? DIRECTIO_ON : DIRECTIO_OFF);
+#endif
 #endif //FCNTL_BROKEN
 
 		file->fil_flags &= ~(FIL_force_write | FIL_no_fs_cache);
@@ -588,6 +610,9 @@ USHORT PIO_init_data(Database* dbb, jrd_file* main_file, ISC_STATUS* status_vect
 
 	USHORT leftPages = initPages;
 	const ULONG initBy = MIN(file->fil_max_page - startPage, leftPages);
+	if (initBy < leftPages)
+		leftPages = initBy;
+
 	for (ULONG i = startPage; i < startPage + initBy; )
 	{
 		bdb.bdb_page = PageNumber(0, i);
@@ -655,7 +680,7 @@ jrd_file* PIO_open(Database* dbb,
 			ERR_post(isc_io_error,
 					 isc_arg_string, "open",
 					 isc_arg_cstring, file_name.length(), ERR_cstring(file_name),
-					 isc_arg_gds, isc_io_open_err, isc_arg_unix, errno, 0);
+					 isc_arg_gds, isc_io_open_err, isc_arg_unix, errno, isc_arg_end);
 		}
 		else {
 			/* If this is the primary file, set Database flag to indicate that it is
@@ -685,7 +710,7 @@ jrd_file* PIO_open(Database* dbb,
 					isc_arg_cstring, file_name.length(),
 						ERR_cstring (file_name),
 					isc_arg_gds, isc_io_open_err,
-					isc_arg_unix, ENOENT, 0);
+					isc_arg_unix, ENOENT, isc_arg_end);
 	}
 #endif /* SUPPORT_RAW_DEVICES */
 
@@ -1069,9 +1094,9 @@ static jrd_file* setup_file(Database* dbb, const Firebird::PathName& file_name,
 						isc_arg_string, "lseek",
 						isc_arg_cstring, file_name.length(), ERR_cstring (file_name),
 						isc_arg_gds, isc_io_read_err,
-						isc_arg_unix, errno, 0);
+						isc_arg_unix, errno, isc_arg_end);
 				if ((reinterpret_cast<Ods::header_page*>(header_page_buffer)->hdr_flags & Ods::hdr_shutdown_mask) == Ods::hdr_shutdown_single)
-					ERR_post(isc_shutdown, isc_arg_cstring, file_name.length(), ERR_cstring(file_name), 0);
+					ERR_post(isc_shutdown, isc_arg_cstring, file_name.length(), ERR_cstring(file_name), isc_arg_end);
 				pageSpace->file = NULL; // Will be set again later by the caller				
 			}
 			catch (const Firebird::Exception&) {
@@ -1126,7 +1151,7 @@ static bool unix_error(
 				 isc_arg_string, ERR_string(file->fil_string,
 											file->fil_length),
 				 isc_arg_gds,
-				 operation, isc_arg_unix, errno, 0);
+				 operation, isc_arg_unix, errno, isc_arg_end);
 
 
     // Added a false for final return - which seems to be the answer,
@@ -1285,7 +1310,7 @@ static bool raw_devices_validate_database (
 					isc_arg_string, "raw_devices_validate_database",
 					isc_arg_string, file_name.length(), ERR_cstring (file_name),
 					isc_arg_gds, isc_io_read_err,
-					isc_arg_unix, errno, 0);
+					isc_arg_unix, errno, isc_arg_end);
 
 	for (int i = 0; i < IO_RETRY; i++)
 	{
@@ -1294,7 +1319,7 @@ static bool raw_devices_validate_database (
 						isc_arg_string, "lseek",
 						isc_arg_string, file_name.length(), ERR_cstring (file_name),
 						isc_arg_gds, isc_io_read_err,
-						isc_arg_unix, errno, 0);
+						isc_arg_unix, errno, isc_arg_end);
 		const ssize_t bytes = read (desc, header, sizeof(header));
 		if (bytes == sizeof(header))
 			goto read_finished;
@@ -1303,14 +1328,14 @@ static bool raw_devices_validate_database (
 						isc_arg_string, "read",
 						isc_arg_string, file_name.length(), ERR_cstring (file_name),
 						isc_arg_gds, isc_io_read_err,
-						isc_arg_unix, errno, 0);
+						isc_arg_unix, errno, isc_arg_end);
 	}
 
 	ERR_post (isc_io_error,
 				isc_arg_string, "read_retry",
 				isc_arg_string, file_name.length(), ERR_cstring (file_name),
 				isc_arg_gds, isc_io_read_err,
-				isc_arg_unix, errno, 0);
+				isc_arg_unix, errno, isc_arg_end);
 
   read_finished:
 	/* Rewind file pointer */
@@ -1319,7 +1344,7 @@ static bool raw_devices_validate_database (
 					isc_arg_string, "lseek",
 					isc_arg_string, file_name.length(), ERR_cstring (file_name),
 					isc_arg_gds, isc_io_read_err,
-					isc_arg_unix, errno, 0);
+					isc_arg_unix, errno, isc_arg_end);
 
 	/* Validate database header. Code lifted from PAG_header. */
 	if (hp->hdr_header.pag_type != pag_header /*|| hp->hdr_sequence*/)
@@ -1362,7 +1387,7 @@ static int raw_devices_unlink_database (
 						isc_arg_string, "open",
 						isc_arg_string, file_name.length(), ERR_cstring (file_name),
 						isc_arg_gds, isc_io_open_err,
-						isc_arg_unix, errno, 0);
+						isc_arg_unix, errno, isc_arg_end);
 	}
 
 	memset(header, 0xa5, sizeof(header));
@@ -1378,7 +1403,7 @@ static int raw_devices_unlink_database (
 			isc_arg_string, "write",
 			isc_arg_string, file_name.length(), ERR_cstring (file_name),
 			isc_arg_gds, isc_io_write_err,
-			isc_arg_unix, errno, 0);
+			isc_arg_unix, errno, isc_arg_end);
 	}
 
 	//if (desc != -1) perhaps it's better to check this???
