@@ -289,10 +289,6 @@ void PIO_extend(jrd_file* main_file, const ULONG extPages, const USHORT pageSize
  *
  **************************************/
  
-#if (defined(_MSC_VER) && (_MSC_VER <= 1200)) // || defined(MINGW)
-	const DWORD INVALID_SET_FILE_POINTER = 0xFFFFFFFF;
-#endif
-
 	// hvlad: prevent other reading\writing threads from changing file pointer.
 	// As we open file without FILE_FLAG_OVERLAPPED, ReadFile\WriteFile calls
 	// will change file pointer we set here and file truncation instead of file
@@ -599,6 +595,9 @@ USHORT PIO_init_data(Database* dbb, jrd_file* main_file, ISC_STATUS* status_vect
 
 	USHORT leftPages = initPages;
 	const ULONG initBy = MIN(file->fil_max_page - startPage, leftPages);
+	if (initBy < leftPages)
+		leftPages = initBy;
+
 	for (ULONG i = startPage; i < startPage + initBy; )
 	{
 		bdb.bdb_page = PageNumber(0, i);
