@@ -69,6 +69,7 @@
 #endif
 
 #include "../common/config/config.h"
+#include "../common/utils_proto.h"
 #include "../jrd/fil.h"
 #include "../jrd/jrd.h"
 #include "../jrd/pag.h"
@@ -81,6 +82,7 @@
 #include "../jrd/lck.h"
 #include "../jrd/sdw.h"
 #include "../jrd/cch.h"
+#include "../jrd/nbak.h"
 #include "../jrd/tra.h"
 #ifdef VIO_DEBUG
 #include "../jrd/vio_debug.h"
@@ -112,7 +114,7 @@ static bool find_type(SLONG, WIN*, pag**, USHORT, USHORT, UCHAR**,
 inline void err_post_if_database_is_readonly(const Database* dbb)
 {
 	if (dbb->dbb_flags & DBB_read_only)
-		ERR_post(isc_read_only_database, 0);
+		ERR_post(isc_read_only_database, isc_arg_end);
 }
 
 // Class definitions (obsolete platforms are commented out)
@@ -155,8 +157,17 @@ static const int CLASS_DARWIN_I386 = 28;	//Darwin/Intel
 static const int CLASS_LINUX_MIPSEL = 29;    // LINUX/MIPSEL
 static const int CLASS_LINUX_MIPS = 30;    // LINUX/MIPS
 static const int CLASS_DARWIN_X64 = 31;   // Darwin/x64
+static const int CLASS_SOLARIS_AMD64 = 32;	//Solaris/amd64
+static const int CLASS_LINUX_ARM = 33;    // LINUX/ARM
+static const int CLASS_LINUX_IA64 = 34;    // LINUX/IA64
+static const int CLASS_DARWIN_PPC64 = 35; // Darwin/PowerPC64
+static const int CLASS_LINUX_S390X = 36;    // LINUX/s390x
+static const int CLASS_LINUX_S390 = 37;     // LINUX/s390
+static const int CLASS_LINUX_SH = 38;		// LINUX/SH (little-endian)
+static const int CLASS_LINUX_SHEB = 39;		// LINUX/SH (big-endian)
+
 static const int CLASS_MAX10 = CLASS_LINUX_AMD64;	// This should not be changed, no new ports with ODS10
-static const int CLASS_MAX = CLASS_DARWIN_X64;
+static const int CLASS_MAX = CLASS_LINUX_SHEB;
 
 // ARCHITECTURE COMPATIBILITY CLASSES
 
@@ -242,16 +253,28 @@ static ArchitectureType archMatrix[CLASS_MAX + 1] = {
 	archBigEndian,    // CLASS_LINUX_PPC
 	archLittleEndian, // CLASS_DARWIN_I386
 	archLittleEndian, // CLASS_LINUX_MIPSEL
-	archBigEndian,  // CLASS_LINUX_MIPS
-	archLittleEndian  // CLASS_DARWIN_X64
+	archBigEndian,    // CLASS_LINUX_MIPS
+	archLittleEndian, // CLASS_DARWIN_X64
+	archLittleEndian, // CLASS_SOLARIS_AMD64
+	archLittleEndian, // CLASS_LINUX_ARM
+	archLittleEndian, // CLASS_LINUX_IA64
+	archBigEndian,    // CLASS_DARWIN_PPC64
+	archBigEndian,    // CLASS_LINUX_S390X
+	archBigEndian,    // CLASS_LINUX_S390
+	archLittleEndian, // CLASS_LINUX_SH
+	archBigEndian     // CLASS_LINUX_SHEB
 };
 
 #ifdef sun
 #ifdef i386
 const SSHORT CLASS		= CLASS_SOLARIS_I386;
-#else
+#endif
+#ifdef sparc
 const SSHORT CLASS		= CLASS_SOLARIS_SPARC;
 #endif
+#ifdef AMD64
+const SSHORT CLASS		= CLASS_SOLARIS_AMD64;
+#endif
 #endif
 
 #ifdef HPUX
@@ -287,12 +310,26 @@ const SSHORT CLASS		= CLASS_LINUX_I386;
 const SSHORT CLASS		= CLASS_LINUX_SPARC;
 #elif defined(AMD64)
 const SSHORT CLASS		= CLASS_LINUX_AMD64;
+#elif defined(ARM)
+const SSHORT CLASS		= CLASS_LINUX_ARM;
 #elif defined(PPC)
 const SSHORT CLASS		= CLASS_LINUX_PPC;
 #elif defined(MIPSEL)
 const SSHORT CLASS		= CLASS_LINUX_MIPSEL;
 #elif defined(MIPS)
 const SSHORT CLASS		= CLASS_LINUX_MIPS;
+#elif defined(IA64)
+const SSHORT CLASS		= CLASS_LINUX_IA64;
+#elif defined(__s390__)
+# if defined(__s390x__)
+const SSHORT CLASS              = CLASS_LINUX_S390X;
+# else
+const SSHORT CLASS              = CLASS_LINUX_S390;
+# endif            // defined(__s390x__)
+#elif defined(SH)
+const SSHORT CLASS		= CLASS_LINUX_SH;
+#elif defined(SHEB)
+const SSHORT CLASS		= CLASS_LINUX_SHEB;
 #else
 #error no support on other hardware for Linux
 #endif
@@ -319,6 +356,8 @@ const SSHORT CLASS		= CLASS_DARWIN_I386;
 const SSHORT CLASS		= CLASS_DARWIN_X64;
 #elif defined(powerpc)
 const SSHORT CLASS		= CLASS_DARWIN_PPC;
+#elif defined(DARWINPPC64)
+const SSHORT CLASS		= CLASS_DARWIN_PPC64;
 #endif
 #endif  // DARWIN
 
@@ -752,6 +791,7 @@ PAG PAG_allocate(WIN * window)
 	// from this pointer page
 	const bool isODS11_1 = (dbb->dbb_ods_version == ODS_VERSION11 && 
 							dbb->dbb_minor_version == 1);
+	bool pipMarked = false;
 
 /* Find an allocation page with something on it */
 
@@ -784,35 +824,45 @@ PAG PAG_allocate(WIN * window)
 						{
 							if (isODS11_1)
 							{
+								BackupManager::SharedDatabaseHolder sdbHolder(tdbb, dbb->dbb_backup_manager);
+								const bool nbak_stalled = (dbb->dbb_backup_manager->get_state() == nbak_state_stalled);
+
 								USHORT next_init_pages = 1;
 								// ensure there are space on disk for faked page
 								if (relative_bit + 1 > pip_page->pip_header.reserved)
 								{
 									fb_assert(relative_bit == pip_page->pip_header.reserved);
 
-									USHORT init_pages = 1;
-									if (!(dbb->dbb_flags & DBB_no_reserve)) 
+									USHORT init_pages = 0;
+									if (!nbak_stalled)
 									{
-										const int minExtendPages = MIN_EXTEND_BYTES / dbb->dbb_page_size;
-										init_pages = sequence ? 64 : MIN(pip_page->pip_header.reserved / 16, 64);
+										init_pages = 1;
+										if (!(dbb->dbb_flags & DBB_no_reserve)) 
+										{
+											const int minExtendPages = MIN_EXTEND_BYTES / dbb->dbb_page_size;
+											init_pages = sequence ? 64 : MIN(pip_page->pip_header.reserved / 16, 64);
 
-										// don't touch pages belongs to the next PIP
-										init_pages = 
-											MIN(init_pages, pageMgr.pagesPerPIP - pip_page->pip_header.reserved);
+											// don't touch pages belongs to the next PIP
+											init_pages = 
+												MIN(init_pages, pageMgr.pagesPerPIP - pip_page->pip_header.reserved);
 
-										if (init_pages < minExtendPages)
-											init_pages = 1;
+											if (init_pages < minExtendPages)
+												init_pages = 1;
 
-										next_init_pages = init_pages;
+											next_init_pages = init_pages;
+										}
+
+										ISC_STATUS_ARRAY status;
+										const ULONG start = 
+											sequence * pageMgr.pagesPerPIP + pip_page->pip_header.reserved;
+
+										init_pages = PIO_init_data(dbb, pageSpace->file, status, start, init_pages);
 									}
 
-									ISC_STATUS_ARRAY status;
-									const ULONG start = 
-										sequence * pageMgr.pagesPerPIP + pip_page->pip_header.reserved;
-
-									init_pages = PIO_init_data(dbb, pageSpace->file, status, start, init_pages);
-
-									if (init_pages) {
+									if (init_pages) 
+									{
+										CCH_MARK(tdbb, &pip_window);
+										pipMarked = true;
 										pip_page->pip_header.reserved += init_pages;
 									}
 									else 
@@ -823,7 +873,6 @@ PAG PAG_allocate(WIN * window)
 										CCH_must_write(window);
 										try {
 											CCH_RELEASE(tdbb, window);
-											pip_page->pip_header.reserved = relative_bit + 1;
 										}
 										catch (Firebird::status_exception) 
 										{
@@ -838,12 +887,16 @@ PAG PAG_allocate(WIN * window)
 											throw;
 										}
 	
+										CCH_MARK(tdbb, &pip_window);
+										pipMarked = true;
+										pip_page->pip_header.reserved = relative_bit + 1;
+
 										new_page = CCH_fake(tdbb, window, 1);
 									}
 									fb_assert(new_page);
 								}
 
-								if (!(dbb->dbb_flags & DBB_no_reserve)) 
+								if (!(dbb->dbb_flags & DBB_no_reserve) && !nbak_stalled) 
 								{
 									const ULONG initialized = 
 										sequence * pageMgr.pagesPerPIP + pip_page->pip_header.reserved;
@@ -869,7 +922,9 @@ PAG PAG_allocate(WIN * window)
 
 	pageSpace->pipHighWater = sequence;
 
-	CCH_MARK(tdbb, &pip_window);
+	if (!pipMarked) {
+		CCH_MARK(tdbb, &pip_window);
+	}
 	*bytes &= ~bit;
 	page_inv_page* pip_page = (page_inv_page*) pip_window.win_buffer;
 
@@ -1239,7 +1294,9 @@ void PAG_header(bool info)
 		(*vector)[0] = header->hdr_PAGES;
 	}
 
-	dbb->dbb_next_transaction = header->hdr_next_transaction;
+	if (!info || dbb->dbb_next_transaction < header->hdr_next_transaction) {
+		dbb->dbb_next_transaction = header->hdr_next_transaction;
+	}
 
 	if (!info || dbb->dbb_oldest_transaction < header->hdr_oldest_transaction) {
 		dbb->dbb_oldest_transaction = header->hdr_oldest_transaction;
@@ -1251,7 +1308,9 @@ void PAG_header(bool info)
 		dbb->dbb_oldest_snapshot = header->hdr_oldest_snapshot;
 	}
 
-	dbb->dbb_attachment_id = header->hdr_attachment_id;
+	if (!info || dbb->dbb_attachment_id < header->hdr_attachment_id) {
+		dbb->dbb_attachment_id = header->hdr_attachment_id;
+	}
 	dbb->dbb_creation_date = *(ISC_TIMESTAMP*) header->hdr_creation_date;
 
 	if (header->hdr_flags & hdr_read_only) {
@@ -1272,7 +1331,7 @@ void PAG_header(bool info)
 				 isc_arg_string, "read-write",
 				 isc_arg_string, "database",
 				 isc_arg_string, ERR_string(attachment->att_filename),
-				 0);
+				 isc_arg_end);
 	}
 
 	const bool useFSCache = dbb->dbb_bcb->bcb_count < Config::getMaxFileSystemCache();
@@ -1352,7 +1411,7 @@ void PAG_header_init()
 	if (header->hdr_header.pag_type != pag_header || header->hdr_sequence) {
 		ERR_post(isc_bad_db_format,
 				 isc_arg_string, ERR_string(attachment->att_filename),
-				 0);
+				 isc_arg_end);
 	}
 
 	const USHORT ods_version = header->hdr_ods_version & ~ODS_FIREBIRD_FLAG;
@@ -1365,7 +1424,7 @@ void PAG_header_init()
 				 isc_arg_number, (SLONG) header->hdr_ods_minor,
 				 isc_arg_number, (SLONG) ODS_VERSION,
 				 isc_arg_number, (SLONG) ODS_CURRENT,
-				 0);
+				 isc_arg_end);
 	}
 
 	// Note that if this check is turned on, it should be recoded in order that
@@ -1395,7 +1454,7 @@ void PAG_header_init()
 	{
 	    ERR_post(isc_bad_db_format,
 				 isc_arg_string, ERR_string(attachment->att_filename),
-				 0);
+				 isc_arg_end);
 	}
 
 	if (header->hdr_page_size < MIN_PAGE_SIZE ||
@@ -1403,7 +1462,7 @@ void PAG_header_init()
 	{
 		ERR_post(isc_bad_db_format,
 				 isc_arg_string, ERR_string(attachment->att_filename),
-				 0);
+				 isc_arg_end);
 	}
 
 	dbb->dbb_ods_version = ods_version;
@@ -1902,7 +1961,7 @@ void PAG_set_db_SQL_dialect(Database* dbb, SSHORT flag)
 				header->hdr_flags & hdr_SQL_dialect_3)
 			{
 				// Check the returned value here!
-				ERR_post_warning(isc_dialect_reset_warning, 0);
+				ERR_post_warning(isc_dialect_reset_warning, isc_arg_end);
 			}
 
 			dbb->dbb_flags &= ~DBB_DB_SQL_dialect_3;	/* set to 0 */
@@ -1918,7 +1977,7 @@ void PAG_set_db_SQL_dialect(Database* dbb, SSHORT flag)
 			CCH_RELEASE(tdbb, &window);
 			ERR_post(isc_inv_dialect_specified, isc_arg_number, flag,
 					 isc_arg_gds, isc_valid_db_dialects, isc_arg_string,
-					 "1 and 3", isc_arg_gds, isc_dialect_not_changed, 0);
+					 "1 and 3", isc_arg_gds, isc_dialect_not_changed, isc_arg_end);
 			break;
 		}
 	}
@@ -2407,19 +2466,20 @@ void PageManager::releaseLocks()
 
 USHORT PageManager::getTempPageSpaceID(thread_db* tdbb)
 {
+	USHORT result;
 #ifdef SUPERSERVER
-	return TEMP_PAGE_SPACE;
+	result = TEMP_PAGE_SPACE;
 #else
 	SET_TDBB(tdbb);
 	Database* dbb = tdbb->getDatabase();
 	Attachment* att = tdbb->getAttachment();
 	if (!att->att_temp_pg_lock)
 	{
-		Lock* lock = FB_NEW_RPT(*dbb->dbb_permanent, sizeof(USHORT)) Lock();
+		Lock* lock = FB_NEW_RPT(*dbb->dbb_permanent, sizeof(lock->lck_key.lck_long)) Lock();
 		lock->lck_type = LCK_page_space;
 		lock->lck_owner_handle = LCK_get_owner_handle(tdbb, lock->lck_type);
 		lock->lck_parent = dbb->dbb_lock;
-		lock->lck_length = sizeof(USHORT);
+		lock->lck_length = sizeof(lock->lck_key.lck_long);
 		lock->lck_dbb = dbb;
 
 		PAG_attachment_id(tdbb);
@@ -2430,13 +2490,20 @@ USHORT PageManager::getTempPageSpaceID(thread_db* tdbb)
 			lock->lck_key.lck_long = static_cast<SLONG>(tmp) + TEMP_PAGE_SPACE + 1;
 			if (LCK_lock(tdbb, lock, LCK_write, LCK_NO_WAIT))
 				break;
+			fb_utils::init_status(tdbb->tdbb_status_vector);
 		}
 
 		att->att_temp_pg_lock = lock;
 	}
 	
-	return (USHORT) att->att_temp_pg_lock->lck_key.lck_long;
+	result = (USHORT) att->att_temp_pg_lock->lck_key.lck_long;
 #endif
+
+	if (!this->findPageSpace(result)) {
+		PAG_attach_temp_pages(tdbb, result);
+	}
+
+	return result;
 }
 
 ULONG PAG_page_count(Database* database, PageCountCallback* cb)
