@@ -214,7 +214,7 @@ jrd_nod* PAR_blr(thread_db*	tdbb,
 		error(csb, isc_metadata_corrupt,
 			  isc_arg_gds, isc_wroblrver,
 			  isc_arg_number, (SLONG) blr_version4,
-			  isc_arg_number, (SLONG) version, 0);
+			  isc_arg_number, (SLONG) version, isc_arg_end);
 	}
 
 	if (version == blr_version4)
@@ -428,7 +428,7 @@ USHORT PAR_desc(thread_db* tdbb, CompilerScratch* csb, DSC* desc, ItemInfo* item
 						break;
 
 					default:
-						error(csb, isc_collation_requires_text, 0);
+						error(csb, isc_collation_requires_text, isc_arg_end);
 						break;
 				}
 			}
@@ -443,7 +443,7 @@ USHORT PAR_desc(thread_db* tdbb, CompilerScratch* csb, DSC* desc, ItemInfo* item
 		}
 
 	default:
-		error(csb, isc_datnotsup, 0);
+		error(csb, isc_datnotsup, isc_arg_end);
 	}
 
 	if (itemInfo)
@@ -514,7 +514,7 @@ jrd_nod* PAR_make_field(thread_db* tdbb, CompilerScratch* csb,
 	jrd_prc* procedure = csb->csb_rpt[stream].csb_procedure;
 
 	const SSHORT id = procedure ? find_proc_field(procedure, base_field) :
-		MET_lookup_field (tdbb, csb->csb_rpt[stream].csb_relation, base_field, 0);
+		MET_lookup_field (tdbb, csb->csb_rpt[stream].csb_relation, base_field);
 
 	if (id < 0)
 		return NULL;
@@ -552,7 +552,7 @@ jrd_nod* PAR_make_field(thread_db* tdbb, CompilerScratch* csb,
 	if (relation)
 	{
 		if (!relation->rel_fields) {
-			ERR_post(isc_depend_on_uncommitted_rel, 0);
+			ERR_post(isc_depend_on_uncommitted_rel, isc_arg_end);
 		}
 		field = (*relation->rel_fields)[id];
 	}
@@ -657,7 +657,7 @@ CompilerScratch* PAR_parse(thread_db* tdbb, const UCHAR* blr, USHORT internal_fl
 	{
 		error(csb, isc_wroblrver,
 			  isc_arg_number, (SLONG) blr_version4,
-			  isc_arg_number, (SLONG) version, 0);
+			  isc_arg_number, (SLONG) version, isc_arg_end);
 	}
 
 	if (version == blr_version4)
@@ -796,6 +796,9 @@ static SSHORT find_proc_field(const jrd_prc* procedure, const Firebird::MetaName
  *
  **************************************/
 	vec<Parameter*>* list = procedure->prc_output_fields;
+	if (!list)
+		return -1;
+
 	vec<Parameter*>::const_iterator ptr = list->begin();
 	for (const vec<Parameter*>::const_iterator end = list->end(); ptr < end; ++ptr)
 	{
@@ -930,7 +933,7 @@ static PsqlException* par_condition(thread_db* tdbb, CompilerScratch* csb)
 		if (code_number)
 			item.xcp_code = code_number;
 		else
-			error(csb, isc_codnotdef, isc_arg_string, ERR_cstring(name), 0);
+			error(csb, isc_codnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 		break;
 
 	case blr_exception:
@@ -938,7 +941,7 @@ static PsqlException* par_condition(thread_db* tdbb, CompilerScratch* csb)
 		item.xcp_type = xcp_xcp_code;
 		par_name(csb, name);
 		if (!(item.xcp_code = MET_lookup_exception_number(tdbb, name)))
-			error(csb, isc_xcpnotdef, isc_arg_string, ERR_cstring(name), 0);
+			error(csb, isc_xcpnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 		dep_node = PAR_make_node(tdbb, e_dep_length);
 		dep_node->nod_type = nod_dependency;
 		dep_node->nod_arg[e_dep_object] = (jrd_nod*)(IPTR) item.xcp_code;
@@ -996,14 +999,14 @@ static PsqlException* par_conditions(thread_db* tdbb, CompilerScratch* csb)
 			if (code_number)
 				item.xcp_code = code_number;
 			else
-				error(csb, isc_codnotdef, isc_arg_string, ERR_cstring(name), 0);
+				error(csb, isc_codnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 			break;
 
 		case blr_exception:
 			item.xcp_type = xcp_xcp_code;
 			par_name(csb, name);
 			if (!(item.xcp_code = MET_lookup_exception_number(tdbb, name)))
-				error(csb, isc_xcpnotdef, isc_arg_string, ERR_cstring(name), 0);
+				error(csb, isc_xcpnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 			dep_node = PAR_make_node(tdbb, e_dep_length);
 			dep_node->nod_type = nod_dependency;
 			dep_node->nod_arg[e_dep_object] = (jrd_nod*)(IPTR) item.xcp_code;
@@ -1053,14 +1056,14 @@ static SSHORT par_context(CompilerScratch* csb, SSHORT* context_ptr)
 			return tail->csb_stream;
 		}
 		else {
-			error(csb, isc_ctxinuse, 0);
+			error(csb, isc_ctxinuse, isc_arg_end);
 		}
 	}
 
 	const SSHORT stream = csb->nextStream(false);
 	if (stream >= MAX_STREAMS)
 	{
-		error(csb, isc_too_many_contexts, 0);
+		error(csb, isc_too_many_contexts, isc_arg_end);
 	}
 
 	tail->csb_flags |= csb_used;
@@ -1155,7 +1158,7 @@ static jrd_nod* par_exec_proc(thread_db* tdbb, CompilerScratch* csb, SSHORT blr_
 			procedure = MET_lookup_procedure(tdbb, name, false);
 		}
 		if (!procedure)
-			error(csb, isc_prcnotdef, isc_arg_string, ERR_cstring(name), 0);
+			error(csb, isc_prcnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 	}
 
 	jrd_nod* node = PAR_make_node(tdbb, e_esp_length);
@@ -1274,7 +1277,7 @@ static jrd_nod* par_field(thread_db* tdbb, CompilerScratch* csb, SSHORT blr_oper
 				but we must support legacy BLR.
 		*/
 	{
-		error(csb, isc_ctxnotdef, 0);
+		error(csb, isc_ctxnotdef, isc_arg_end);
 	}
 
 	Firebird::MetaName name;
@@ -1311,12 +1314,12 @@ static jrd_nod* par_field(thread_db* tdbb, CompilerScratch* csb, SSHORT blr_oper
 				error(csb,
 					  isc_fldnotdef2,
 					  isc_arg_string, ERR_cstring(name),
-					  isc_arg_string, procedure->prc_name.c_str(), 0);
+					  isc_arg_string, procedure->prc_name.c_str(), isc_arg_end);
 		}
 		else {
 			jrd_rel* relation = tail->csb_relation;
 			if (!relation)
-				error(csb, isc_ctxnotdef, 0);
+				error(csb, isc_ctxnotdef, isc_arg_end);
 
 			/* make sure relation has been scanned before using it */
 
@@ -1327,27 +1330,34 @@ static jrd_nod* par_field(thread_db* tdbb, CompilerScratch* csb, SSHORT blr_oper
 			}
 
 			par_name(csb, name);
-			if ((id = MET_lookup_field(tdbb, relation, name.c_str(), 0)) < 0) {
+			if ((id = MET_lookup_field(tdbb, relation, name)) < 0) {
 				if (csb->csb_g_flags & csb_validation) {
 					id = 0;
 					flags |= nod_id;
 					is_column = true;
 				}
 				else {
+					if (relation->rel_flags & REL_system)
+					{
+						jrd_nod* node = PAR_make_node(tdbb, 0);
+						node->nod_type = nod_null;
+						return node;
+ 					}
+
 					if (tdbb->getAttachment()->att_flags & ATT_gbak_attachment)
 					{
 						warning(csb, isc_fldnotdef, isc_arg_string,
 								ERR_cstring(name), isc_arg_string,
-								relation->rel_name.c_str(), 0);
+								relation->rel_name.c_str(), isc_arg_end);
 					}
 					else if (relation->rel_name.length() > 0)
 					{
 						error(csb, isc_fldnotdef, isc_arg_string,
 							  ERR_cstring(name), isc_arg_string,
-							  relation->rel_name.c_str(), 0);
+							  relation->rel_name.c_str(), isc_arg_end);
 					}
 					else
-						error(csb, isc_ctxnotdef, 0);
+						error(csb, isc_ctxnotdef, isc_arg_end);
 				}
 			}
 		}
@@ -1424,7 +1434,7 @@ static jrd_nod* par_function(thread_db* tdbb, CompilerScratch* csb)
 		}
 		else {
 			csb->csb_running -= count;
-			error(csb, isc_funnotdef, isc_arg_string, ERR_cstring(name), 0);
+			error(csb, isc_funnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 		}
 	}
 
@@ -1440,14 +1450,14 @@ static jrd_nod* par_function(thread_db* tdbb, CompilerScratch* csb)
 			warning(csb, isc_funnotdef,
 					isc_arg_string, ERR_cstring(name),
 					isc_arg_interpreted,
-					"module name or entrypoint could not be found", 0);
+					"module name or entrypoint could not be found", isc_arg_end);
 		}
 		else {
 			csb->csb_running -= count;
 			error(csb, isc_funnotdef,
 				  isc_arg_string, ERR_cstring(name),
 				  isc_arg_interpreted,
-				  "module name or entrypoint could not be found", 0);
+				  "module name or entrypoint could not be found", isc_arg_end);
 		}
 
 	jrd_nod* node = PAR_make_node(tdbb, e_fun_length);
@@ -1657,7 +1667,7 @@ static jrd_nod* par_message(thread_db* tdbb, CompilerScratch* csb)
 	}
 
 	if (offset > MAX_FORMAT_SIZE)
-		error(csb, isc_imp_exc, isc_arg_gds, isc_blktoobig, 0);
+		error(csb, isc_imp_exc, isc_arg_gds, isc_blktoobig, isc_arg_end);
 
 	format->fmt_length = (USHORT) offset;
 
@@ -1685,13 +1695,13 @@ static jrd_nod* par_modify(thread_db* tdbb, CompilerScratch* csb, SSHORT blr_ope
 	if (context >= csb->csb_rpt.getCount() || 
 		!(csb->csb_rpt[context].csb_flags & csb_used))
 	{
-		error(csb, isc_ctxnotdef, 0);
+		error(csb, isc_ctxnotdef, isc_arg_end);
 	}
 	const SSHORT org_stream = csb->csb_rpt[context].csb_stream;
 	const SSHORT new_stream = csb->nextStream(false);
 	if (new_stream >= MAX_STREAMS)
 	{
-		error(csb, isc_too_many_contexts, 0);
+		error(csb, isc_too_many_contexts, isc_arg_end);
 	}
 	context = (unsigned int) BLR_BYTE;
 
@@ -1747,7 +1757,7 @@ static USHORT par_name(CompilerScratch* csb, Firebird::MetaName& name)
 			*s++ = BLR_BYTE;
 		}
 		*s = 0;
-		ERR_post(isc_identifier_too_long, isc_arg_string, ERR_cstring(st), 0);
+		ERR_post(isc_identifier_too_long, isc_arg_string, ERR_cstring(st), isc_arg_end);
 	}
 
 	char* s = name.getBuffer(l);
@@ -1844,7 +1854,7 @@ static jrd_nod* par_plan(thread_db* tdbb, CompilerScratch* csb)
 
 		n = BLR_BYTE;
 		if (n >= csb->csb_rpt.getCount() || !(csb->csb_rpt[n].csb_flags & csb_used))
-			error(csb, isc_ctxnotdef, 0);
+			error(csb, isc_ctxnotdef, isc_arg_end);
 		const SSHORT stream = csb->csb_rpt[n].csb_stream;
 
 		relation_node->nod_arg[e_rel_stream] = (jrd_nod*) (IPTR) stream;
@@ -1884,13 +1894,13 @@ static jrd_nod* par_plan(thread_db* tdbb, CompilerScratch* csb)
 					{
 						warning(csb, isc_indexname, isc_arg_string,
 								ERR_cstring(name), isc_arg_string,
-								relation->rel_name.c_str(), 0);
+								relation->rel_name.c_str(), isc_arg_end);
 					}
 					else
 					{
 						error(csb, isc_indexname, isc_arg_string,
 							  ERR_cstring(name), isc_arg_string,
-							  relation->rel_name.c_str(), 0);
+							  relation->rel_name.c_str(), isc_arg_end);
 					}
 				}
 
@@ -1953,13 +1963,13 @@ static jrd_nod* par_plan(thread_db* tdbb, CompilerScratch* csb)
 						{
 							warning(csb, isc_indexname, isc_arg_string,
 									ERR_cstring(name), isc_arg_string,
-									relation->rel_name.c_str(), 0);
+									relation->rel_name.c_str(), isc_arg_end);
 						}
 						else
 						{
 							error(csb, isc_indexname, isc_arg_string,
 								  ERR_cstring(name), isc_arg_string,
-								  relation->rel_name.c_str(), 0);
+								  relation->rel_name.c_str(), isc_arg_end);
 						}
 					}
 
@@ -2013,26 +2023,33 @@ static jrd_nod* par_procedure(thread_db* tdbb, CompilerScratch* csb, SSHORT blr_
 
 	SET_TDBB(tdbb);
 
-	{
-		Firebird::MetaName name;
+	Firebird::MetaName name;
 
-		if (blr_operator == blr_procedure) {
-			par_name(csb, name);
-			procedure = MET_lookup_procedure(tdbb, name, false);
-		}
-		else {
-			const SSHORT pid = BLR_WORD;
-			if (!(procedure = MET_lookup_procedure_id(tdbb, pid, false, false, 0)))
-				name.printf("id %d", pid);
-		}
-		if (!procedure)
-			error(csb, isc_prcnotdef, isc_arg_string, ERR_cstring(name), 0);
+	if (blr_operator == blr_procedure) {
+		par_name(csb, name);
+		procedure = MET_lookup_procedure(tdbb, name, false);
 	}
+	else {
+		const SSHORT pid = BLR_WORD;
+		if (!(procedure = MET_lookup_procedure_id(tdbb, pid, false, false, 0)))
+			name.printf("id %d", pid);
+	}
+
+	if (!procedure)
+		error(csb, isc_prcnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 
 	if (procedure->prc_type == prc_executable)
 	{
-		error(csb, isc_illegal_prc_type,
-			  isc_arg_string, ERR_string(procedure->prc_name), 0);
+		if (tdbb->getAttachment()->att_flags & ATT_gbak_attachment)
+		{
+			warning(csb, isc_illegal_prc_type,
+					isc_arg_string, ERR_cstring(name), isc_arg_end);
+		}
+		else
+		{ 
+			error(csb, isc_illegal_prc_type,
+				  isc_arg_string, ERR_cstring(name), isc_arg_end);
+		}
 	}
 
 	jrd_nod* node = PAR_make_node(tdbb, e_prc_length);
@@ -2087,7 +2104,7 @@ static void par_procedure_parms(
 				  input_flag ? isc_prcmismat : isc_prc_out_param_mismatch,
 				  isc_arg_string,
 				  ERR_cstring(procedure->prc_name.c_str()),
-				  0);
+				  isc_arg_end);
 		}
 		else
 			mismatch = true;
@@ -2158,9 +2175,8 @@ static void par_procedure_parms(
 
 			// default value for parameter 
 			if ((count <= 0) && input_flag) {
-				Parameter* parameter =
-					(*procedure->prc_input_fields)[procedure->prc_inputs - n];
-				asgn->nod_arg[asgn_arg1] = parameter->prm_default_value;
+				Parameter* parameter = (*procedure->prc_input_fields)[procedure->prc_inputs - n];
+				asgn->nod_arg[asgn_arg1] = CMP_clone_node(tdbb, csb, parameter->prm_default_value);
 			}
 			else {
 				asgn->nod_arg[asgn_arg1] = parse(tdbb, csb, VALUE);
@@ -2186,7 +2202,7 @@ static void par_procedure_parms(
 			  input_flag ? isc_prcmismat : isc_prc_out_param_mismatch,
 			  isc_arg_string,
 			  ERR_cstring(procedure->prc_name.c_str()),
-			  0);
+			  isc_arg_end);
 	}
 }
 
@@ -2225,7 +2241,7 @@ static jrd_nod* par_relation(
 		}
 		if (!(relation = MET_lookup_relation_id(tdbb, id, false))) {
 			name.printf("id %d", id);
-			error(csb, isc_relnotdef, isc_arg_string, ERR_cstring(name), 0);
+			error(csb, isc_relnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 		}
 	}
 	else if (blr_operator == blr_relation || blr_operator == blr_relation2) {
@@ -2235,7 +2251,7 @@ static jrd_nod* par_relation(
 			par_name(csb, *alias_string);
 		}
 		if (!(relation = MET_lookup_relation(tdbb, name)))
-			error(csb, isc_relnotdef, isc_arg_string, ERR_cstring(name), 0);
+			error(csb, isc_relnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 	}
 
 /* if an alias was passed, store with the relation */
@@ -2516,7 +2532,7 @@ static jrd_nod* par_sys_function(thread_db* tdbb, CompilerScratch* csb)
 	if (!function)
 	{
 		csb->csb_running -= count;
-		error(csb, isc_funnotdef, isc_arg_string, ERR_cstring(name), 0);
+		error(csb, isc_funnotdef, isc_arg_string, ERR_cstring(name), isc_arg_end);
 	}
 
 	jrd_nod* node = PAR_make_node(tdbb, e_sysfun_length);
@@ -2782,7 +2798,7 @@ static jrd_nod* parse(thread_db* tdbb, CompilerScratch* csb, USHORT expected,
 		n = BLR_BYTE;
 		if (n > MAX_TIME_PRECISION) {
 			ERR_post(isc_invalid_time_precision,
-					 isc_arg_number, MAX_TIME_PRECISION, 0);
+					 isc_arg_number, MAX_TIME_PRECISION, isc_arg_end);
 		}
 		node->nod_arg[0] = (jrd_nod*) (IPTR) n;
 		break;
@@ -2824,7 +2840,7 @@ static jrd_nod* parse(thread_db* tdbb, CompilerScratch* csb, USHORT expected,
 	case blr_erase:
 		n = BLR_BYTE;
 		if (n >= csb->csb_rpt.getCount() || !(csb->csb_rpt[n].csb_flags & csb_used))
-			error(csb, isc_ctxnotdef, 0);
+			error(csb, isc_ctxnotdef, isc_arg_end);
 		node->nod_arg[e_erase_stream] =
 			(jrd_nod*) (IPTR) csb->csb_rpt[n].csb_stream;
 		break;
@@ -2955,7 +2971,7 @@ static jrd_nod* parse(thread_db* tdbb, CompilerScratch* csb, USHORT expected,
 			const SLONG tmp = MET_lookup_generator(tdbb, name.c_str());
 			if (tmp < 0) {
 				error(csb, isc_gennotdef,
-					  isc_arg_string, ERR_cstring(name), 0);
+					  isc_arg_string, ERR_cstring(name), isc_arg_end);
 			}
 			node->nod_arg[e_gen_relation] = (jrd_nod*) (IPTR) tmp;
 			node->nod_arg[e_gen_value] = parse(tdbb, csb, VALUE);
@@ -2980,7 +2996,7 @@ static jrd_nod* parse(thread_db* tdbb, CompilerScratch* csb, USHORT expected,
 	case blr_dbkey:
 		n = BLR_BYTE;
 		if (n >= csb->csb_rpt.getCount() || !(csb->csb_rpt[n].csb_flags & csb_used))
-			error(csb, isc_ctxnotdef, 0);
+			error(csb, isc_ctxnotdef, isc_arg_end);
 		node->nod_arg[0] = (jrd_nod*) (IPTR) csb->csb_rpt[n].csb_stream;
 		break;
 
@@ -3073,14 +3089,14 @@ static jrd_nod* parse(thread_db* tdbb, CompilerScratch* csb, USHORT expected,
 			if (n >= csb->csb_rpt.getCount() ||
 				!(message = csb->csb_rpt[n].csb_message))
 			{
-				error(csb, isc_badmsgnum, 0);
+				error(csb, isc_badmsgnum, isc_arg_end);
 			}
 			node->nod_arg[e_arg_message] = message;
 			n = BLR_WORD;
 			node->nod_arg[e_arg_number] = (jrd_nod*) (IPTR) n;
 			const Format* format = (Format*) message->nod_arg[e_msg_format];
 			if (n >= format->fmt_count)
-				error(csb, isc_badparnum, 0);
+				error(csb, isc_badparnum, isc_arg_end);
 			if (blr_operator != blr_parameter) {
 				jrd_nod* temp = PAR_make_node(tdbb, e_arg_length);
 				node->nod_arg[e_arg_flag] = temp;
@@ -3091,7 +3107,7 @@ static jrd_nod* parse(thread_db* tdbb, CompilerScratch* csb, USHORT expected,
 				n = BLR_WORD;
 				temp->nod_arg[e_arg_number] = (jrd_nod*) (IPTR) n;
 				if (n >= format->fmt_count)
-					error(csb, isc_badparnum, 0);
+					error(csb, isc_badparnum, isc_arg_end);
 			}
 			if (blr_operator == blr_parameter3) {
 				jrd_nod* temp = PAR_make_node(tdbb, e_arg_length);
@@ -3103,7 +3119,7 @@ static jrd_nod* parse(thread_db* tdbb, CompilerScratch* csb, USHORT expected,
 				n = BLR_WORD;
 				temp->nod_arg[e_arg_number] = (jrd_nod*) (IPTR) n;
 				if (n >= format->fmt_count)
-					error(csb, isc_badparnum, 0);
+					error(csb, isc_badparnum, isc_arg_end);
 			}
 		}
 		break;
@@ -3264,7 +3280,7 @@ static void syntax_error(CompilerScratch* csb, const TEXT* string)
 	error(csb, isc_syntaxerr,
 		  isc_arg_string, string,
 		  isc_arg_number, (SLONG) (csb->csb_running - csb->csb_blr - 1),
-		  isc_arg_number, (SLONG) csb->csb_running[-1], 0);
+		  isc_arg_number, (SLONG) csb->csb_running[-1], isc_arg_end);
 }
 
 
