@@ -24,6 +24,8 @@
 #ifndef JRD_PCMET_PROTO_H
 #define JRD_PCMET_PROTO_H
 
+#include "../common/classes/fb_string.h"
+
 namespace Jrd {
 	class DeferredWork;
 	class jrd_tra;
@@ -31,7 +33,7 @@ namespace Jrd {
 	struct index_desc;
 }
 
-bool PCMET_expression_index(Jrd::thread_db*, SSHORT, Jrd::DeferredWork*, Jrd::jrd_tra*);
+void PCMET_expression_index(Jrd::thread_db*, const Firebird::string&, USHORT, Jrd::jrd_tra*);
 void PCMET_lookup_index(Jrd::thread_db*, Jrd::jrd_rel*, Jrd::index_desc*);
 
 #endif // JRD_PCMET_PROTO_H
