@@ -54,6 +54,7 @@ class CompilerScratch;
 class jrd_prc;
 class Format;
 class VaryingStr;
+class BtrPageGCLock;
 
 // Record source block (RSB) types
 
@@ -76,7 +77,7 @@ enum rsb_t
 	rsb_left_cross,						// left outer join as a nested loop
 	rsb_procedure,						// stored procedure
 	rsb_virt_sequential,				// sequential access to a virtual table
-	rsb_recurse							// Recursive union
+	rsb_recursive_union					// Recursive union
 };
 
 
@@ -122,6 +123,7 @@ const USHORT rsb_singular = 1;			// singleton select, expect 0 or 1 records
 const USHORT rsb_descending = 4;		// an ascending index is being used for a descending sort or vice versa
 const USHORT rsb_project = 8;			// projection on this stream is requested
 const USHORT rsb_writelock = 16;		// records should be locked for writing
+const USHORT rsb_recursive = 32;		// this rsb is a sub_rsb of recursive rsb
 
 // special argument positions within the RecordSource
 
@@ -172,6 +174,9 @@ public:
 	};
 
 	static const USHORT MAX_RECURSE_LEVEL;
+
+private:
+	static void cleanup_level(jrd_req*, RecordSource*, irsb_recurse*);
 };
 
 // Impure area formats for the various RecordSource types
@@ -266,6 +271,7 @@ struct irsb_nav {
 	ULONG irsb_nav_count;					// record count of last record returned
 	RecordBitmap**	irsb_nav_bitmap;			// bitmap for inversion tree
 	RecordBitmap*	irsb_nav_records_visited;	// bitmap of records already retrieved
+	BtrPageGCLock*	irsb_nav_btr_gc_lock;		// lock to prevent removal of currently walked index page
 	USHORT irsb_nav_offset;					// page offset of current index node
 	USHORT irsb_nav_lower_length;			// length of lower key value
 	USHORT irsb_nav_upper_length;			// length of upper key value
