@@ -66,6 +66,7 @@
 #include "../jrd/ibase.h"
 #include "../common/utils_proto.h"
 #include "../jrd/scl.h"
+#include "../common/config/config.h"
 
 #ifdef SERVER_SHUTDOWN
 #include "../jrd/jrd_proto.h"
@@ -126,8 +127,6 @@ const int GET_LINE		= 1;
 const int GET_EOF		= 2;
 const int GET_BINARY	= 4;
 
-const TEXT SVC_TRMNTR	= '\377';
-
 namespace Jrd {
 	Service::Service(serv_entry *se, Firebird::MemoryPool& p) :
 		svc_parsed_sw(p), 
@@ -261,7 +260,7 @@ static inline void is_service_running(const Service* service)
 	if (!(service->svc_flags & SVC_forked)) {
 		THREAD_ENTER();
 		ERR_post (isc_svcnoexe, isc_arg_string,
-			service->svc_service->serv_name, 0); 
+			service->svc_service->serv_name, isc_arg_end); 
 	}
 }
 
@@ -541,11 +540,11 @@ Service* SVC_attach(USHORT	service_length,
 	if (!serv->serv_name)
 #ifdef NOT_USED_OR_REPLACED
 		ERR_post(isc_service_att_err, isc_arg_gds, isc_service_not_supported,
-				 0);
+				 isc_arg_end);
 #else
 		ERR_post(isc_service_att_err, isc_arg_gds, isc_svcnotdef,
 				 isc_arg_string, error_string(misc_buf),
-				 0);
+				 isc_arg_end);
 #endif
 
 	JRD_get_thread_data();
@@ -589,7 +588,15 @@ Service* SVC_attach(USHORT	service_length,
 	}
 	else {
 #ifdef TRUSTED_SERVICES
-		if (options.spb_trusted_login.hasData())
+		static AmCache useTrusted = AM_UNKNOWN;
+		if (useTrusted == AM_UNKNOWN)
+		{
+			Firebird::PathName authMethod(Config::getAuthMethod());
+			useTrusted = (authMethod == AmTrusted || authMethod == AmMixed) ? 
+				AM_ENABLED : AM_DISABLED;
+		}
+
+		if (options.spb_trusted_login.hasData() && (useTrusted == AM_ENABLED))
 		{
 			options.spb_user_name = options.spb_trusted_login;
 		}
@@ -600,7 +607,7 @@ Service* SVC_attach(USHORT	service_length,
 			{
 				// user name and password are required while
 				// attaching to the services manager
-				ERR_post(isc_service_att_err, isc_arg_gds, isc_svcnouser, 0);
+				ERR_post(isc_service_att_err, isc_arg_gds, isc_svcnouser, isc_arg_end);
 			}
 			else 
 			{
@@ -625,7 +632,7 @@ Service* SVC_attach(USHORT	service_length,
 		{
 			Firebird::status_exception::raise(isc_long_login, 
 											  isc_arg_number, options.spb_user_name.length(), 
-											  isc_arg_number, USERNAME_LENGTH, 0);
+											  isc_arg_number, USERNAME_LENGTH, isc_arg_end);
 		}
 
 /* Check that the validated user has the authority to access this service */
@@ -709,7 +716,7 @@ Service* SVC_attach(USHORT	service_length,
 		}
 		else {
 			ERR_post(isc_service_att_err, isc_arg_gds,
-					 isc_svc_in_use, isc_arg_string, serv->serv_name, 0);
+					 isc_svc_in_use, isc_arg_string, serv->serv_name, isc_arg_end);
 		}
 #endif
 	}
@@ -986,7 +993,7 @@ ISC_STATUS SVC_query2(Service* service,
 //			case isc_info_svc_user_dbpath:
 				break;
 			default:
-				ERR_post(isc_bad_spb_form, 0);
+				ERR_post(isc_bad_spb_form, isc_arg_end);
 				break;
 			}
 		}
@@ -1061,7 +1068,7 @@ ISC_STATUS SVC_query2(Service* service,
 			*info++ = item;
 			if (service->svc_user_flag & SVC_user_dba) {
 				service->svc_do_shutdown = false;
-				WHY_set_shutdown(FALSE);
+				WHY_set_shutdown(SHUTDOWN_NONE);
 			}
 			else
 				need_admin_privs(&status, "isc_info_svc_svr_online");
@@ -1071,7 +1078,7 @@ ISC_STATUS SVC_query2(Service* service,
 			*info++ = item;
 			if (service->svc_user_flag & SVC_user_dba) {
 				service->svc_do_shutdown = true;
-				WHY_set_shutdown(TRUE);
+				WHY_set_shutdown(SHUTDOWN_ATTACH);
 			}
 			else
 				need_admin_privs(&status, "isc_info_svc_svr_offline");
@@ -1086,25 +1093,33 @@ ISC_STATUS SVC_query2(Service* service,
 		case isc_info_svc_get_env:
 		case isc_info_svc_get_env_lock:
 		case isc_info_svc_get_env_msg:
-			switch (item) {
-			case isc_info_svc_get_env:
-				gds__prefix(buffer, "");
-				break;
-			case isc_info_svc_get_env_lock:
-				gds__prefix_lock(buffer, "");
-				break;
-			case isc_info_svc_get_env_msg:
-				gds__prefix_msg(buffer, "");
-			}
+			if (service->svc_user_flag & SVC_user_dba)
+			{
+				switch (item) 
+				{
+				case isc_info_svc_get_env:
+					gds__prefix(buffer, "");
+					break;
+				case isc_info_svc_get_env_lock:
+					gds__prefix_lock(buffer, "");
+					break;
+				case isc_info_svc_get_env_msg:
+					gds__prefix_msg(buffer, "");
+				}
 
-			/* Note: it is safe to use strlen to get a length of "buffer"
-			   because gds_prefix[_lock|_msg] return a zero-terminated
-			   string
-			 */
-			info = INF_put_item(item, strlen(buffer), buffer, info, end);
-			if (!info) {
-				THREAD_ENTER();
-				return 0;
+				/* Note: it is safe to use strlen to get a length of "buffer"
+				   because gds_prefix[_lock|_msg] return a zero-terminated
+				   string
+				 */
+				info = INF_put_item(item, strlen(buffer), buffer, info, end);
+				if (!info) {
+					THREAD_ENTER();
+					return 0;
+				}
+			}
+			else
+			{
+				need_admin_privs(&status, "isc_info_svc_get_env");
 			}
 			break;
 
@@ -1351,14 +1366,14 @@ ISC_STATUS SVC_query2(Service* service,
 	}
 
 	if (info < end)
-		*info = isc_info_end;
+		*info++ = isc_info_end;
 
 	if (start_info && (end - info >= 7))
 	{
-		SLONG number = 1 + (info - start_info);
+		SLONG number = info - start_info;
 		memmove(start_info + 7, start_info, number);
 		USHORT length2 = INF_convert(number, buffer);
-		INF_put_item(isc_info_length, length2, buffer, start_info, end);
+		INF_put_item(isc_info_length, length2, buffer, start_info, end, true);
 	}
 
 	if (!(service->svc_flags & SVC_thd_running))
@@ -1492,7 +1507,7 @@ void SVC_query(Service*		service,
 			*info++ = item;
 			if (service->svc_user_flag & SVC_user_dba) {
 				service->svc_do_shutdown = false;
-				WHY_set_shutdown(FALSE);
+				WHY_set_shutdown(SHUTDOWN_NONE);
 				*info++ = 0;	/* Success */
 			}
 			else
@@ -1503,7 +1518,7 @@ void SVC_query(Service*		service,
 			*info++ = item;
 			if (service->svc_user_flag & SVC_user_dba) {
 				service->svc_do_shutdown = true;
-				WHY_set_shutdown(TRUE);
+				WHY_set_shutdown(SHUTDOWN_ATTACH);
 				*info++ = 0;	/* Success */
 			}
 			else
@@ -1519,27 +1534,37 @@ void SVC_query(Service*		service,
 		case isc_info_svc_get_env:
 		case isc_info_svc_get_env_lock:
 		case isc_info_svc_get_env_msg:
-			switch (item) {
-			case isc_info_svc_get_env:
-				gds__prefix(PathBuffer, "");
-				break;
-			case isc_info_svc_get_env_lock:
-				gds__prefix_lock(PathBuffer, "");
-				break;
-			case isc_info_svc_get_env_msg:
-				gds__prefix_msg(PathBuffer, "");
-			}
-
-			/* Note: it is safe to use strlen to get a length of "buffer"
-			   because gds_prefix[_lock|_msg] return a zero-terminated
-			   string
-			 */
-			if (!(info = INF_put_item(item, strlen(PathBuffer),
-									  PathBuffer, info, end)))
+            if (service->svc_user_flag & SVC_user_dba)
 			{
-				THREAD_ENTER();
-				return;
+				switch (item) {
+				case isc_info_svc_get_env:
+					gds__prefix(PathBuffer, "");
+					break;
+				case isc_info_svc_get_env_lock:
+					gds__prefix_lock(PathBuffer, "");
+					break;
+				case isc_info_svc_get_env_msg:
+					gds__prefix_msg(PathBuffer, "");
+				}
+
+				/* Note: it is safe to use strlen to get a length of "buffer"
+				   because gds_prefix[_lock|_msg] return a zero-terminated
+				   string
+				 */
+				if (!(info = INF_put_item(item, strlen(PathBuffer),
+										  PathBuffer, info, end)))
+				{
+					THREAD_ENTER();
+					return;
+				}
 			}
+			/*
+			 * * Can not return error for service v.1 => simply ignore request
+			else
+			{
+				need_admin_privs(&status, "isc_info_svc_get_env");
+			}
+			 */
 			break;
 
 #ifdef SUPERSERVER
@@ -1825,12 +1850,12 @@ void* SVC_start(Service* service, USHORT spb_length, const SCHAR* spb_data)
 			break;
 
 	if (!serv->serv_name)
-		ERR_post(isc_service_att_err, isc_arg_gds, isc_service_not_supported, 0);
+		ERR_post(isc_service_att_err, isc_arg_gds, isc_service_not_supported, isc_arg_end);
 
 /* currently we do not use "anonymous" service for any purposes but
    isc_service_query() */
 	if (service->svc_user_flag == SVC_user_none) {
-		ERR_post(isc_bad_spb_form, 0);
+		ERR_post(isc_bad_spb_form, isc_arg_end);
 	}
 
 	thd_mutex.enter();
@@ -1838,7 +1863,7 @@ void* SVC_start(Service* service, USHORT spb_length, const SCHAR* spb_data)
 		thd_mutex.leave();
 		ERR_post(isc_svc_in_use, isc_arg_string,
 				 error_string(serv->serv_name, strlen(serv->serv_name)),
-				 0);
+				 isc_arg_end);
 	}
 	/* Another service may have been started with this service block.
 	 * If so, we must reset the service flags.
@@ -1917,13 +1942,13 @@ void* SVC_start(Service* service, USHORT spb_length, const SCHAR* spb_data)
 	spb.rewind();
 	if ((!service->svc_switches.hasData()) && svc_id != isc_action_svc_get_fb_log) 
 	{
-		ERR_post(isc_bad_spb_form, 0);
+		ERR_post(isc_bad_spb_form, isc_arg_end);
 	}
 
 // Do not let everyone look at server log
 	if (svc_id == isc_action_svc_get_fb_log && !(service->svc_user_flag & SVC_user_dba))
     {
-       	ERR_post(isc_adm_task_denied, 0);
+       	ERR_post(isc_adm_task_denied, isc_arg_end);
     }
 
 #ifndef SERVICE_THREAD
@@ -1955,10 +1980,14 @@ void* SVC_start(Service* service, USHORT spb_length, const SCHAR* spb_data)
 /* FREE: at SVC_detach() */
 	if (!service->svc_stdout)	/* NOMEM: */
 	{
-		ERR_post(isc_virmemexh, 0);
+		ERR_post(isc_virmemexh, isc_arg_end);
 	}
 
 	if (serv->serv_thd) {
+		// reset stdout buffer - fix for CORE-3220
+		service->svc_stdout_head = 1;
+		service->svc_stdout_tail = SVC_STDOUT_BUFFER_SIZE;
+
 		service->svc_flags &= ~SVC_evnt_fired;
 		THREAD_EXIT();
 
@@ -1989,7 +2018,7 @@ void* SVC_start(Service* service, USHORT spb_length, const SCHAR* spb_data)
 		ERR_post(isc_svcnotdef,
 				isc_arg_string,
 				error_string(serv->serv_name, strlen(serv->serv_name)),
-				0);
+				isc_arg_end);
 	}
 
 #endif /* SERVICE_THREAD */
@@ -2085,7 +2114,7 @@ static void get_options(Firebird::ClumpletReader&	spb,
  **************************************/
 	const UCHAR p = spb.getBufferTag();
 	if (p != isc_spb_version1 && p != isc_spb_current_version) {
-		ERR_post(isc_bad_spb_form, isc_arg_gds, isc_wrospbver, 0);
+		ERR_post(isc_bad_spb_form, isc_arg_gds, isc_wrospbver, isc_arg_end);
 	}
 	options->spb_version = p;
 
@@ -2170,7 +2199,7 @@ static void io_error(const TEXT* string,
  **************************************/
 
 	ERR_post(isc_io_error, isc_arg_string, string, isc_arg_string, filename,
-			 isc_arg_gds, operation, SYS_ERR, status, 0);
+			 isc_arg_gds, operation, SYS_ERR, status, isc_arg_end);
 }
 #endif
 
@@ -2336,7 +2365,7 @@ static void service_get(Service*		service,
 		const time_t elapsed_time = end_time - start_time;
 #endif
 		if ((timeout) && (elapsed_time >= timeout)) {
-			service->svc_flags &= SVC_timeout;
+			service->svc_flags |= SVC_timeout;
 			return;
 		}
 
@@ -2397,7 +2426,7 @@ static void service_fork(TEXT* service_path, Service* service)
  *	Startup a service. Just a stub.
  *
  **************************************/
-	ERR_post(isc_service_att_err, isc_arg_gds, isc_service_not_supported, 0);
+	ERR_post(isc_service_att_err, isc_arg_gds, isc_service_not_supported, isc_arg_end);
 }
 
 static void service_get(
@@ -2436,6 +2465,10 @@ static void service_put(Service* service, const SCHAR* buffer, USHORT length)
 
 #else
 
+#ifdef DARWIN
+#define NO_DOUBLE_VFORK
+#endif
+
 static void service_fork(TEXT* service_path, Service* service)
 {
 /**************************************
@@ -2474,11 +2507,11 @@ static void service_fork(TEXT* service_path, Service* service)
 	switch (pid = vfork()) {
 	case -1:
 		THREAD_ENTER();
-		ERR_post(isc_sys_request, isc_arg_string, "vfork", SYS_ERR, errno, 0);
+		ERR_post(isc_sys_request, isc_arg_string, "vfork", SYS_ERR, errno, isc_arg_end);
 		break;
 
 	case 0:
-#ifndef DARWIN
+#ifndef NO_DOUBLE_VFORK
 		if (vfork() > 0)
 			_exit(FINI_OK);
 #endif
@@ -2518,7 +2551,9 @@ static void service_fork(TEXT* service_path, Service* service)
 	close(pair1[1]);
 	close(pair2[0]);
 
+#ifndef NO_DOUBLE_VFORK
 	waitpid(pid, NULL, 0);
+#endif
 
 	THREAD_ENTER();
 
@@ -2846,7 +2881,7 @@ static bool process_switches(Firebird::ClumpletReader&	spb,
 						ERR_post(isc_unexp_spb_form, isc_arg_string,
 								 error_string(SPB_SEC_USERNAME,
 												strlen(SPB_SEC_USERNAME)),
-								 0);
+								 isc_arg_end);
 					}
 					found = true;
 				}
@@ -2884,7 +2919,7 @@ static bool process_switches(Firebird::ClumpletReader&	spb,
 						ERR_post(isc_unexp_spb_form, isc_arg_string,
 								 error_string(SPB_SEC_USERNAME,
 												strlen(SPB_SEC_USERNAME)),
-								 0);
+								 isc_arg_end);
 					}
 					found = true;
 				}
@@ -2937,7 +2972,12 @@ static bool process_switches(Firebird::ClumpletReader&	spb,
 				break;
 
 			case isc_spb_command_line: 
-				get_action_svc_string(spb, switches);
+				{
+					Firebird::string s;
+					spb.getString(s);
+					switches += s;
+					switches += ' ';
+				}
 				break;
 
 			default:
@@ -3124,17 +3164,7 @@ static void get_action_svc_string(const Firebird::ClumpletReader& spb,
  **************************************/
 	Firebird::string s;
 	spb.getString(s);
-	for (size_t i = 0; i < s.length(); ++i)
-	{
-		if (s[i] == SVC_TRMNTR)
-		{
-			s.insert(i, 1, SVC_TRMNTR);
-			++i;
-		}
-	}
-	switches += SVC_TRMNTR;
-	switches += s;
-	switches += SVC_TRMNTR;
+	switches.appendQuoted(s, SVC_TRMNTR);
 	switches += ' ';
 }
 
