@@ -40,9 +40,9 @@
 #include "../jrd/req.h"
 #include "../jrd/exe.h"
 #include "../jrd/rse.h"
+#include "../jrd/intl_classes.h"
 #include "../jrd/jrd_pwd.h"
 #include "../jrd/thd.h"
-#include "../jrd/all_proto.h"
 #include "../jrd/blb_proto.h"
 #include "../jrd/cch_proto.h"
 #include "../jrd/cmp_proto.h"
@@ -65,6 +65,7 @@
 #include "../jrd/enc_proto.h"
 #include "../jrd/jrd_proto.h"
 #include "../common/classes/ClumpletWriter.h"
+#include "../common/classes/TriState.h"
 
 #ifndef VMS
 #include "../lock/lock_proto.h"
@@ -91,9 +92,6 @@ static const SCHAR lock_types[] =
 };
 #endif /* VMS */
 
-const int DEFAULT_LOCK_TIMEOUT = -1; // infinite
-static const SLONG MAX_TRA_NUMBER = MAX_SLONG;
-
 using namespace Jrd;
 using namespace Ods;
 
@@ -101,6 +99,7 @@ using namespace Ods;
 #include "../jrd/isc_s_proto.h"
 #endif
 
+static int blocking_ast_transaction(void*);
 #ifdef SUPERSERVER_V2
 static SLONG bump_transaction_id(thread_db*, WIN *);
 #else
@@ -114,6 +113,7 @@ static void expand_view_lock(jrd_tra*, jrd_rel*, SCHAR);
 static tx_inv_page* fetch_inventory_page(thread_db*, WIN *, SLONG, USHORT);
 static SLONG inventory_page(thread_db*, SLONG);
 static SSHORT limbo_transaction(thread_db*, SLONG);
+static void link_transaction(thread_db*, jrd_tra*);
 static void restart_requests(thread_db*, jrd_tra*);
 #ifdef SWEEP_THREAD
 static void start_sweeper(thread_db*, Database*);
@@ -131,8 +131,8 @@ static const UCHAR sweep_tpb[] =
 };
 
 
-void TRA_attach_request(Jrd::jrd_tra* transaction, Jrd::jrd_req* request) {
-
+void TRA_attach_request(Jrd::jrd_tra* transaction, Jrd::jrd_req* request)
+{
 	// When request finishes normally transaction reference is not cleared.
 	// Then if afterwards request is restarted TRA_attach_request is called again.
 	if (request->req_transaction) {
@@ -144,7 +144,7 @@ void TRA_attach_request(Jrd::jrd_tra* transaction, Jrd::jrd_req* request) {
 	fb_assert(request->req_transaction == NULL);
 	fb_assert(request->req_tra_next == NULL);
 	fb_assert(request->req_tra_prev == NULL);
-	
+
 	// Assign transaction reference
 	request->req_transaction = transaction;
 
@@ -157,8 +157,10 @@ void TRA_attach_request(Jrd::jrd_tra* transaction, Jrd::jrd_req* request) {
 	transaction->tra_requests = request;
 }
 
-void TRA_detach_request(Jrd::jrd_req* request) {
-	if (!request->req_transaction) return;
+void TRA_detach_request(Jrd::jrd_req* request)
+{
+	if (!request->req_transaction)
+		return;
 
 	// Remove request from the doubly linked list
 	if (request->req_tra_next) {
@@ -214,7 +216,7 @@ bool TRA_active_transactions(thread_db* tdbb, Database* dbb)
 		active = MAX(dbb->dbb_oldest_active, dbb->dbb_oldest_transaction);
 	}
 	else {
-		WIN window(HEADER_PAGE);
+		WIN window(HEADER_PAGE_NUMBER);
 		const header_page* header = (header_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_header);
 		number = header->hdr_next_transaction;
 		oldest = header->hdr_oldest_transaction;
@@ -278,7 +280,7 @@ void TRA_cleanup(thread_db* tdbb)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* Return without cleaning up the TIP's for a ReadOnly database */
@@ -294,13 +296,13 @@ void TRA_cleanup(thread_db* tdbb)
 			return;
 	}
 
-	const SLONG trans_per_tip = dbb->dbb_pcontrol->pgc_tpt;
+	const SLONG trans_per_tip = dbb->dbb_page_manager.transPerTIP;
 
 /* Read header page and allocate transaction number.  Since
    the transaction inventory page was initialized to zero, it
    transaction is automatically marked active. */
 
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 	const header_page* header = (header_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_header);
 	const SLONG ceiling = header->hdr_next_transaction;
 	const SLONG active = header->hdr_oldest_active;
@@ -412,7 +414,7 @@ void TRA_commit(thread_db* tdbb, jrd_tra* transaction, const bool retaining_flag
 
 	if (retaining_flag
 		&& !(transaction->tra_flags & TRA_write
-			 || transaction->tra_deferred_work))
+			 || transaction->tra_deferred_job))
 	{
 		transaction->tra_flags &= ~TRA_prepared;
 		// Get rid of all user savepoints
@@ -500,12 +502,12 @@ void TRA_extend_tip(thread_db* tdbb, ULONG sequence, WIN * precedence_window)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* Start by fetching prior transaction page, if any */
 	tx_inv_page* prior_tip;
-	WIN prior_window(-1);
+	WIN prior_window(DB_PAGE_SPACE, -1);
 	if (sequence) {
 		prior_tip =
 			fetch_inventory_page(tdbb, &prior_window, (SLONG) (sequence - 1),
@@ -513,7 +515,7 @@ void TRA_extend_tip(thread_db* tdbb, ULONG sequence, WIN * precedence_window)
 	}
 
 /* Allocate and format new page */
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	tx_inv_page* tip = (tx_inv_page*) DPM_allocate(tdbb, &window);
 	tip->tip_header.pag_type = pag_transactions;
 
@@ -524,7 +526,7 @@ void TRA_extend_tip(thread_db* tdbb, ULONG sequence, WIN * precedence_window)
 
 	if (sequence) {
 		CCH_MARK_MUST_WRITE(tdbb, &prior_window);
-		prior_tip->tip_next = window.win_page;
+		prior_tip->tip_next = window.win_page.getPageNum();
 		CCH_RELEASE(tdbb, &prior_window);
 	}
 
@@ -532,11 +534,11 @@ void TRA_extend_tip(thread_db* tdbb, ULONG sequence, WIN * precedence_window)
 
 	vcl* vector = dbb->dbb_t_pages =
 		vcl::newVector(*dbb->dbb_permanent, dbb->dbb_t_pages, sequence + 1);
-	(*vector)[sequence] = window.win_page;
+	(*vector)[sequence] = window.win_page.getPageNum();
 
 /* Write into pages relation */
 
-	DPM_pages(tdbb, 0, pag_transactions, sequence, window.win_page);
+	DPM_pages(tdbb, 0, pag_transactions, sequence, window.win_page.getPageNum());
 }
 
 
@@ -555,15 +557,15 @@ int TRA_fetch_state(thread_db* tdbb, SLONG number)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* locate and fetch the proper TIP page */
 
     const ULONG tip_number = (ULONG) number;
-	const SLONG trans_per_tip = dbb->dbb_pcontrol->pgc_tpt;
+	const SLONG trans_per_tip = dbb->dbb_page_manager.transPerTIP;
 	const ULONG tip_seq = tip_number / trans_per_tip;
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	const tx_inv_page* tip = fetch_inventory_page(tdbb, &window, tip_seq, LCK_read);
 
 /* calculate the state of the desired transaction */
@@ -595,16 +597,16 @@ void TRA_get_inventory(thread_db* tdbb, UCHAR* bit_vector, ULONG base, ULONG top
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
-	const ULONG trans_per_tip = dbb->dbb_pcontrol->pgc_tpt;
+	const ULONG trans_per_tip = dbb->dbb_page_manager.transPerTIP;
 	ULONG sequence = base / trans_per_tip;
 	const ULONG last = top / trans_per_tip;
 
 /* fetch the first inventory page */
 
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	const tx_inv_page* tip =
 		fetch_inventory_page(tdbb, &window, (SLONG) sequence++, LCK_read);
 
@@ -657,7 +659,7 @@ int TRA_get_state(thread_db* tdbb, SLONG number)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 	if (dbb->dbb_tip_cache)
@@ -696,7 +698,7 @@ void TRA_header_write(thread_db* tdbb, Database* dbb, SLONG number)
 /* If transaction number is already on disk just return. */
 
 	if (!number || dbb->dbb_last_header_write < number) {
-		WIN window(HEADER_PAGE);
+		WIN window(HEADER_PAGE_NUMBER);
 		header_page* header = (header_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 
 		if (header->hdr_next_transaction) {
@@ -745,14 +747,12 @@ void TRA_init(thread_db* tdbb)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
-	jrd_tra* trans = FB_NEW_RPT(*dbb->dbb_permanent, 0) jrd_tra(*dbb->dbb_permanent);
+	jrd_tra* trans = FB_NEW_RPT(*dbb->dbb_permanent, 0) jrd_tra(dbb->dbb_permanent);
 	dbb->dbb_sys_trans = trans;
-	trans->tra_lock_timeout = DEFAULT_LOCK_TIMEOUT;
 	trans->tra_flags |= TRA_system | TRA_ignore_limbo;
-	trans->tra_pool = dbb->dbb_permanent;
 }
 
 
@@ -785,27 +785,6 @@ void TRA_invalidate(Database* database, ULONG mask)
 }
 
 
-void TRA_link_transaction(thread_db* tdbb, jrd_tra* transaction)
-{
-/**************************************
- *
- *	T R A _ l i n k _ t r a n s a c t i o n
- *
- **************************************
- *
- * Functional description
- *	Link transaction block into database attachment.
- *
- **************************************/
-	SET_TDBB(tdbb);
-
-	Attachment* attachment  = tdbb->tdbb_attachment;
-	transaction->tra_attachment = attachment;
-	transaction->tra_next = attachment->att_transactions;
-	attachment->att_transactions = transaction;
-}
-
-
 void TRA_post_resources(thread_db* tdbb, jrd_tra* transaction, ResourceList& resources)
 {
 /**************************************
@@ -815,8 +794,8 @@ void TRA_post_resources(thread_db* tdbb, jrd_tra* transaction, ResourceList& res
  **************************************
  *
  * Functional description
- *	Post interest in relation/procedure existence to transaction.
- *	This guarantees that the relation/procedure won't be dropped
+ *	Post interest in relation/procedure/collation existence to transaction.
+ *	This guarantees that the relation/procedure/collation won't be dropped
  *	out from under the transaction.
  *
  **************************************/
@@ -827,7 +806,8 @@ void TRA_post_resources(thread_db* tdbb, jrd_tra* transaction, ResourceList& res
 	for (Resource* rsc = resources.begin(); rsc < resources.end(); rsc++) 
 	{
 		if (rsc->rsc_type == Resource::rsc_relation ||
-			rsc->rsc_type == Resource::rsc_procedure)
+			rsc->rsc_type == Resource::rsc_procedure ||
+			rsc->rsc_type == Resource::rsc_collation)
 		{
 			size_t i;
 			if (!transaction->tra_resources.find(*rsc, i)) 
@@ -836,6 +816,9 @@ void TRA_post_resources(thread_db* tdbb, jrd_tra* transaction, ResourceList& res
 				switch (rsc->rsc_type) {
 				case Resource::rsc_relation:
 					MET_post_existence(tdbb, rsc->rsc_rel);
+					if (rsc->rsc_rel->rel_file) {
+						EXT_tra_attach(rsc->rsc_rel->rel_file, transaction);
+					}
 					break;
 				case Resource::rsc_procedure:
 					rsc->rsc_prc->prc_use_count++;
@@ -849,6 +832,9 @@ void TRA_post_resources(thread_db* tdbb, jrd_tra* transaction, ResourceList& res
 					}
 #endif
 					break;
+				case Resource::rsc_collation:
+					rsc->rsc_coll->incUseCount(tdbb);
+					break;
 				default:   // shut up compiler warning
 					break;
 				}
@@ -876,7 +862,7 @@ bool TRA_precommited(thread_db* tdbb, SLONG old_number, SLONG new_number)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 	vcl* vector = dbb->dbb_pc_transactions;
@@ -992,7 +978,7 @@ jrd_tra* TRA_reconnect(thread_db* tdbb, const UCHAR* id, USHORT length)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* Cannot work on limbo transactions for ReadOnly database */
@@ -1001,10 +987,8 @@ jrd_tra* TRA_reconnect(thread_db* tdbb, const UCHAR* id, USHORT length)
 
 
 	Jrd::ContextPoolHolder context(tdbb, JrdMemoryPool::createPool());
-	jrd_tra* trans = FB_NEW_RPT(*tdbb->getDefaultPool(), 0) jrd_tra(*tdbb->getDefaultPool());
-	trans->tra_pool = tdbb->getDefaultPool();
+	jrd_tra* trans = FB_NEW_RPT(*tdbb->getDefaultPool(), 0) jrd_tra(tdbb->getDefaultPool());
 	trans->tra_number = gds__vax_integer(id, length);
-	trans->tra_lock_timeout = DEFAULT_LOCK_TIMEOUT;
 	trans->tra_flags |= TRA_prepared | TRA_reconnected | TRA_write;
 
 	const UCHAR state = limbo_transaction(tdbb, trans->tra_number);
@@ -1039,7 +1023,7 @@ jrd_tra* TRA_reconnect(thread_db* tdbb, const UCHAR* id, USHORT length)
 				 isc_arg_string, ERR_cstring(text), isc_arg_end);
 	}
 
-	TRA_link_transaction(tdbb, trans);
+	link_transaction(tdbb, trans);
 
 	return trans;
 }
@@ -1060,6 +1044,7 @@ void TRA_release_transaction(thread_db* tdbb, jrd_tra* transaction)
  *
  **************************************/
 	SET_TDBB(tdbb);
+	Database* dbb = tdbb->getDatabase();
 
 	if (transaction->tra_blobs.getFirst()) 
 		while (true) 
@@ -1086,22 +1071,46 @@ void TRA_release_transaction(thread_db* tdbb, jrd_tra* transaction)
 			TRA_detach_request(transaction->tra_requests);
 	}
 
-/* Release interest in relation/procedure existence for transaction */
+	// Release interest in relation/procedure existence for transaction
 
 	for (Resource* rsc = transaction->tra_resources.begin();
 		rsc < transaction->tra_resources.end(); rsc++)
 	{
 		switch (rsc->rsc_type) {
+		case Resource::rsc_relation:
+			MET_release_existence(rsc->rsc_rel);
+			if (rsc->rsc_rel->rel_file) {
+				EXT_tra_detach(rsc->rsc_rel->rel_file, transaction);
+			}
+			break;
 		case Resource::rsc_procedure:
 			CMP_decrement_prc_use_count(tdbb, rsc->rsc_prc);
 			break;
-		default:
-			MET_release_existence(rsc->rsc_rel);
+		case Resource::rsc_collation:
+			rsc->rsc_coll->decUseCount(tdbb);
 			break;
+		default:
+			fb_assert(false);
 		}
 	}
 
-/* Release the locks associated with the transaction */
+	{
+		vec<jrd_rel*>& rels = *tdbb->getDatabase()->dbb_relations;
+		for (size_t i = 0; i < rels.count(); i++)
+		{
+			jrd_rel* relation = rels[i];
+			if (relation && (relation->rel_flags & REL_temp_tran))
+			{
+				relation->delPages(tdbb, transaction->tra_number);
+			}
+		}
+
+	}
+
+	// Release the locks associated with the transaction
+
+	if (transaction->tra_cancel_lock)
+		LCK_release(tdbb, transaction->tra_cancel_lock);
 
 	vec<Lock*>* vector = transaction->tra_relation_locks;
 	if (vector) {
@@ -1118,16 +1127,16 @@ void TRA_release_transaction(thread_db* tdbb, jrd_tra* transaction)
 		LCK_release(tdbb, transaction->tra_lock);
 	--transaction->tra_use_count;
 
-/* release the sparse bit map used for commit retain transaction */
+	// release the sparse bit map used for commit retain transaction
 
 	delete transaction->tra_commit_sub_trans;
 
 	if (transaction->tra_flags & TRA_precommitted)
 		TRA_precommited(tdbb, transaction->tra_number, 0);
 
-/* Unlink the transaction from the database block */
+	// Unlink the transaction from the database block
 
-	for (jrd_tra** ptr = &tdbb->tdbb_attachment->att_transactions;
+	for (jrd_tra** ptr = &tdbb->getAttachment()->att_transactions;
 							*ptr; ptr = &(*ptr)->tra_next) 
 	{ 
 		if (*ptr == transaction) {
@@ -1136,15 +1145,15 @@ void TRA_release_transaction(thread_db* tdbb, jrd_tra* transaction)
 		}
 	}
 
-/* Release transaction's under-modification-rpb list. */
+	// Release transaction's under-modification-rpb list.
 
 	delete transaction->tra_rpblist;
 
-/* Release the transaction pool. */
+	// Release the database snapshot, if any
 
-	JrdMemoryPool* tra_pool = transaction->tra_pool;
-	if (tra_pool)
-		JrdMemoryPool::deletePool(tra_pool);
+	delete transaction->tra_db_snapshot;
+
+	jrd_tra::destroy(dbb, transaction);
 }
 
 
@@ -1267,7 +1276,7 @@ void TRA_rollback(thread_db* tdbb, jrd_tra* transaction, const bool retaining_fl
 			// as committed
 			state = tra_committed;
 		}
-		catch (const std::exception&) {
+		catch (const Firebird::Exception&) {
 			/* Prevent a bugcheck in TRA_set_state to cause a loop */
 			/* Clear the error because the rollback will succeed. */
 			tdbb->tdbb_status_vector[0] = isc_arg_gds;
@@ -1308,7 +1317,7 @@ void TRA_set_state(thread_db* tdbb, jrd_tra* transaction, SLONG number, SSHORT s
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* If we're terminating ourselves and we've
@@ -1327,13 +1336,13 @@ void TRA_set_state(thread_db* tdbb, jrd_tra* transaction, SLONG number, SSHORT s
 		return;
 	}
 
-	ULONG trans_per_tip = dbb->dbb_pcontrol->pgc_tpt;
+	const ULONG trans_per_tip = dbb->dbb_page_manager.transPerTIP;
 	const SLONG sequence = number / trans_per_tip;
-	trans_per_tip = dbb->dbb_pcontrol->pgc_tpt;
+	//trans_per_tip = dbb->dbb_page_manager.transPerTIP;
 	const ULONG byte = TRANS_OFFSET(number % trans_per_tip);
 	const SSHORT shift = TRANS_SHIFT(number);
 
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	tx_inv_page* tip = fetch_inventory_page(tdbb, &window, (SLONG) sequence, LCK_write);
 
 #ifdef SUPERSERVER_V2
@@ -1482,7 +1491,7 @@ int TRA_snapshot_state(thread_db* tdbb, const jrd_tra* trans, SLONG number)
 }
 
 
-jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
+jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const UCHAR* tpb)
 {
 /**************************************
  *
@@ -1495,14 +1504,13 @@ jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
-	Attachment* attachment = tdbb->tdbb_attachment;
-	WIN window(-1);
+	Database* dbb = tdbb->getDatabase();
+	Attachment* attachment = tdbb->getAttachment();
+	WIN window(DB_PAGE_SPACE, -1);
 
 	if (dbb->dbb_ast_flags & DBB_shut_tran) {
-		ERR_post(isc_shutinprog, isc_arg_cstring,
-				 tdbb->tdbb_attachment->att_filename.length(),
-				 tdbb->tdbb_attachment->att_filename.c_str(),
+		ERR_post(isc_shutinprog, isc_arg_string,
+				 ERR_string(tdbb->getAttachment()->att_filename),
 				 isc_arg_end);
 	}
 
@@ -1511,11 +1519,8 @@ jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
    make up the real transaction block. */
 
 	Jrd::ContextPoolHolder context(tdbb, JrdMemoryPool::createPool());
-	jrd_tra* temp = FB_NEW_RPT(*tdbb->getDefaultPool(), 0) jrd_tra(*tdbb->getDefaultPool());
-	temp->tra_pool = tdbb->getDefaultPool();
-	temp->tra_lock_timeout = DEFAULT_LOCK_TIMEOUT;
-	transaction_options(tdbb, temp, reinterpret_cast<const UCHAR*>(tpb),
-						tpb_length);
+	jrd_tra* temp = FB_NEW_RPT(*tdbb->getDefaultPool(), 0) jrd_tra(tdbb->getDefaultPool());
+	transaction_options(tdbb, temp, tpb, tpb_length);
 
 	Lock* lock = TRA_transaction_lock(tdbb, temp);
 
@@ -1563,12 +1568,12 @@ jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
 
 	jrd_tra* trans;
 	if (temp->tra_flags & TRA_read_committed)
-		trans = FB_NEW_RPT(*tdbb->getDefaultPool(), 0) jrd_tra(*tdbb->getDefaultPool());
+		trans = FB_NEW_RPT(*tdbb->getDefaultPool(), 0) jrd_tra(tdbb->getDefaultPool());
 	else {
-		trans = FB_NEW_RPT(*tdbb->getDefaultPool(), (number - base + TRA_MASK) / 4) jrd_tra(*tdbb->getDefaultPool());
+		trans = FB_NEW_RPT(*tdbb->getDefaultPool(), (number - base + TRA_MASK) / 4) jrd_tra(tdbb->getDefaultPool());
 	}
 
-	trans->tra_pool = temp->tra_pool;
+	fb_assert(trans->tra_pool == temp->tra_pool);
 	trans->tra_relation_locks = temp->tra_relation_locks;
 	trans->tra_lock_timeout = temp->tra_lock_timeout;
 	trans->tra_flags = temp->tra_flags;
@@ -1603,7 +1608,7 @@ jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
 /* Link the transaction to the attachment block before releasing
    header page for handling signals. */
 
-	TRA_link_transaction(tdbb, trans);
+	link_transaction(tdbb, trans);
 
 #ifndef SUPERSERVER_V2
 	if (!(dbb->dbb_flags & DBB_read_only))
@@ -1764,7 +1769,7 @@ jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
 /* If the transaction block is getting out of hand, force a sweep */
 
 	if (dbb->dbb_sweep_interval &&
-		!(tdbb->tdbb_attachment->att_flags & ATT_no_cleanup) &&
+		!(tdbb->getAttachment()->att_flags & ATT_no_cleanup) &&
 		(trans->tra_oldest_active - trans->tra_oldest >
 		 dbb->dbb_sweep_interval) && oldest_state != tra_limbo)
 	{
@@ -1787,11 +1792,25 @@ jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
    undo the transaction if it rolls back. */
 
 	if ((trans != dbb->dbb_sys_trans) &&
-		!(trans->tra_flags & TRA_no_auto_undo)) {
+		!(trans->tra_flags & TRA_no_auto_undo))
+	{
 		VIO_start_save_point(tdbb, trans);
 		trans->tra_save_point->sav_flags |= SAV_trans_level;
 	}
 
+/* Allocate the cancellation lock */
+
+	lock = FB_NEW_RPT(*trans->tra_pool, sizeof(SLONG)) Lock();
+	trans->tra_cancel_lock = lock;
+	lock->lck_type = LCK_cancel;
+	lock->lck_owner_handle = LCK_get_owner_handle(tdbb, lock->lck_type);
+	lock->lck_parent = dbb->dbb_lock;
+	lock->lck_length = sizeof(SLONG);
+	lock->lck_key.lck_long = trans->tra_number;
+	lock->lck_dbb = dbb;
+	lock->lck_ast = blocking_ast_transaction;
+	lock->lck_object = trans;
+
 /* if the user asked us to restart all requests in this attachment,
    do so now using the new transaction */
 
@@ -1800,11 +1819,9 @@ jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
 
 /* If the transaction is read-only and read committed, it can be
    precommitted because it can't modify any records and doesn't
-   need a snapshot preserved. This is being enabled for internal
-   transactions used by the sweeper threads. This transaction type
-   can run forever without impacting garbage collection or causing
-   transaction bitmap growth. It can be turned on for external use
-   by removing the test for TDBB_sweeper. */
+   need a snapshot preserved. This transaction type can run
+   forever without impacting garbage collection or causing
+   transaction bitmap growth. */
 
 	if (trans->tra_flags & TRA_readonly &&
 		trans->tra_flags & TRA_read_committed)
@@ -1812,7 +1829,7 @@ jrd_tra* TRA_start(thread_db* tdbb, int tpb_length, const SCHAR* tpb)
 		TRA_set_state(tdbb, trans, trans->tra_number, tra_committed);
 		LCK_release(tdbb, trans->tra_lock);
 		delete trans->tra_lock;
-		trans->tra_lock = 0;
+		trans->tra_lock = NULL;
 		trans->tra_flags |= TRA_precommitted;
 	}
 
@@ -1860,7 +1877,7 @@ bool TRA_sweep(thread_db* tdbb, jrd_tra* trans)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* No point trying to sweep a ReadOnly database */
@@ -1889,7 +1906,7 @@ bool TRA_sweep(thread_db* tdbb, jrd_tra* trans)
 
 	dbb->dbb_flags |= DBB_sweep_in_progress;
 
-	jrd_tra* const tdbb_old_trans = tdbb->tdbb_transaction;
+	jrd_tra* const tdbb_old_trans = tdbb->getTransaction();
 	jrd_tra* transaction = 0;
 /* The following line seems to fix a bug that appears on VMS and AIX
    (and perhaps MPE XL).  It probably has to do with the fact that
@@ -1916,12 +1933,10 @@ bool TRA_sweep(thread_db* tdbb, jrd_tra* trans)
    below to advance the OIT we must save it before it changes. */
 
 	if (!(transaction = trans))
-		transaction = TRA_start(tdbb,
-								sizeof(sweep_tpb),
-								reinterpret_cast<const char*>(sweep_tpb));
+		transaction = TRA_start(tdbb, sizeof(sweep_tpb), sweep_tpb);
 
 	SLONG transaction_oldest_active = transaction->tra_oldest_active;
-	tdbb->tdbb_transaction = transaction;
+	tdbb->setTransaction(transaction);
 
 
 #ifdef GARBAGE_THREAD
@@ -1966,7 +1981,7 @@ bool TRA_sweep(thread_db* tdbb, jrd_tra* trans)
 
 		CCH_flush(tdbb, FLUSH_SWEEP, 0);
 
-		WIN window(HEADER_PAGE);
+		WIN window(HEADER_PAGE_NUMBER);
 		header_page* header =
 			(header_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 
@@ -1986,9 +2001,9 @@ bool TRA_sweep(thread_db* tdbb, jrd_tra* trans)
 	dbb->dbb_flags &= ~DBB_sweep_in_progress;
 
 	tdbb->tdbb_flags &= ~TDBB_sweeper;
-	tdbb->tdbb_transaction = tdbb_old_trans;
+	tdbb->setTransaction(tdbb_old_trans);
 	}	// try
-	catch (const std::exception& ex) {
+	catch (const Firebird::Exception& ex) {
 		Firebird::stuff_exception(tdbb->tdbb_status_vector, ex);
 		try {
 			if (!trans)
@@ -1997,14 +2012,14 @@ bool TRA_sweep(thread_db* tdbb, jrd_tra* trans)
 			LCK_release(tdbb, &temp_lock);
 			dbb->dbb_flags &= ~DBB_sweep_in_progress;
 			tdbb->tdbb_flags &= ~TDBB_sweeper;
-			tdbb->tdbb_transaction = tdbb_old_trans;
+			tdbb->setTransaction(tdbb_old_trans);
 		}
-		catch (const std::exception& ex2) {
+		catch (const Firebird::Exception& ex2) {
 			Firebird::stuff_exception(tdbb->tdbb_status_vector, ex2);
 			LCK_release(tdbb, &temp_lock);
 			dbb->dbb_flags &= ~DBB_sweep_in_progress;
 			tdbb->tdbb_flags &= ~TDBB_sweeper;
-			tdbb->tdbb_transaction = tdbb_old_trans;
+			tdbb->setTransaction(tdbb_old_trans);
 			return false;
 		}
 	}
@@ -2026,7 +2041,7 @@ Lock* TRA_transaction_lock(thread_db* tdbb, BLK object)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 
 	Lock* lock = FB_NEW_RPT(*tdbb->getDefaultPool(), sizeof(SLONG)) Lock();
 	lock->lck_type = LCK_tra;
@@ -2061,7 +2076,7 @@ int TRA_wait(thread_db* tdbb, jrd_tra* trans, SLONG number, jrd_tra::wait_t wait
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* Create, wait on, and release lock on target transaction.  If
@@ -2124,6 +2139,41 @@ int TRA_wait(thread_db* tdbb, jrd_tra* trans, SLONG number, jrd_tra::wait_t wait
 }
 
 
+static int blocking_ast_transaction(void* ast_object)
+{
+/**************************************
+ *
+ *	b l o c k i n g _ a s t _ t r a n s a c t i o n
+ *
+ **************************************
+ *
+ * Functional description
+ *	Mark the transaction to cancel its active requests.
+ *
+ **************************************/
+	jrd_tra* transaction = static_cast<jrd_tra*>(ast_object);
+	fb_assert(transaction);
+
+	thread_db thd_context, *tdbb;
+	JRD_set_thread_data(tdbb, thd_context);
+
+	tdbb->setDatabase(transaction->tra_cancel_lock->lck_dbb);
+	tdbb->setAttachment(transaction->tra_cancel_lock->lck_attachment);
+	tdbb->tdbb_quantum = QUANTUM;
+	tdbb->setRequest(NULL);
+	tdbb->setTransaction(NULL);
+	Jrd::ContextPoolHolder context(tdbb, 0);
+
+	if (transaction->tra_cancel_lock)
+		LCK_release(tdbb, transaction->tra_cancel_lock);
+
+	transaction->tra_flags |= TRA_cancel_request;
+
+	JRD_restore_thread_data();
+	return 0;
+}
+
+
 #ifdef SUPERSERVER_V2
 static SLONG bump_transaction_id(thread_db* tdbb, WIN * window)
 {
@@ -2139,7 +2189,7 @@ static SLONG bump_transaction_id(thread_db* tdbb, WIN * window)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 	if (dbb->dbb_next_transaction >= MAX_TRA_NUMBER - 1) 
@@ -2156,10 +2206,10 @@ static SLONG bump_transaction_id(thread_db* tdbb, WIN * window)
 /* If this is the first transaction on a TIP, allocate the TIP now. */
 
 	const bool new_tip = 
-		(number == 1 || (number % dbb->dbb_pcontrol->pgc_tpt) == 0);
+		(number == 1 || (number % dbb->dbb_page_manager.transPerTIP) == 0);
 	if (new_tip) {
 		TRA_extend_tip(tdbb,
-					   (ULONG) (number / dbb->dbb_pcontrol->pgc_tpt), window);
+					   (ULONG) (number / dbb->dbb_page_manager.transPerTIP), window);
 	}
 
 	return number;
@@ -2181,10 +2231,10 @@ static header_page* bump_transaction_id(thread_db* tdbb, WIN * window)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
-	window->win_page = HEADER_PAGE;
+	window->win_page = HEADER_PAGE_NUMBER;
 	header_page* header = (header_page*) CCH_FETCH(tdbb, window, LCK_write, pag_header);
 
 /* Before incrementing the next transaction Id, make sure the current one is valid */
@@ -2206,10 +2256,10 @@ static header_page* bump_transaction_id(thread_db* tdbb, WIN * window)
 /* If this is the first transaction on a TIP, allocate the TIP now. */
 
 	const bool new_tip = 
-		(number == 1 || (number % dbb->dbb_pcontrol->pgc_tpt) == 0);
+		(number == 1 || (number % dbb->dbb_page_manager.transPerTIP) == 0);
 	if (new_tip) {
 		TRA_extend_tip(tdbb,
-					   (ULONG) (number / dbb->dbb_pcontrol->pgc_tpt), window);
+					   (ULONG) (number / dbb->dbb_page_manager.transPerTIP), window);
 	}
 
 /* Extend, if necessary, has apparently succeeded.  Next, update header
@@ -2254,7 +2304,7 @@ static void compute_oldest_retaining(
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* Get a commit retaining lock, if not present. */
@@ -2399,7 +2449,7 @@ static tx_inv_page* fetch_inventory_page(
  **************************************/
 	SET_TDBB(tdbb);
 
-	window->win_page = inventory_page(tdbb, (int) sequence);
+	window->win_page = inventory_page(tdbb, sequence);
 	tx_inv_page* tip =
 		(tx_inv_page*) CCH_FETCH(tdbb, window, lock_level, pag_transactions);
 
@@ -2424,10 +2474,10 @@ static SLONG inventory_page(thread_db* tdbb, SLONG sequence)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	vcl* vector = dbb->dbb_t_pages;
 	while (!vector || sequence >= (SLONG) vector->count()) {
 		DPM_scan_pages(tdbb);
@@ -2446,7 +2496,7 @@ static SLONG inventory_page(thread_db* tdbb, SLONG sequence)
 		tip = (tx_inv_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_transactions);
 		CCH_RELEASE(tdbb, &window);
 		DPM_pages(tdbb, 0, pag_transactions, vector->count(),
-				  window.win_page);
+				  window.win_page.getPageNum());
 	}
 
 	return (*vector)[sequence];
@@ -2470,15 +2520,15 @@ static SSHORT limbo_transaction(thread_db* tdbb, SLONG id)
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
-	const SLONG trans_per_tip = dbb->dbb_pcontrol->pgc_tpt;
+	const SLONG trans_per_tip = dbb->dbb_page_manager.transPerTIP;
 
 	const SLONG page = id / trans_per_tip;
 	const SLONG number = id % trans_per_tip;
 
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	const tx_inv_page* tip = fetch_inventory_page(tdbb, &window, page, LCK_write);
 
 	const SLONG trans_offset = TRANS_OFFSET(number);
@@ -2491,6 +2541,27 @@ static SSHORT limbo_transaction(thread_db* tdbb, SLONG id)
 }
 
 
+static void link_transaction(thread_db* tdbb, jrd_tra* transaction)
+{
+/**************************************
+ *
+ *	l i n k _ t r a n s a c t i o n
+ *
+ **************************************
+ *
+ * Functional description
+ *	Link transaction block into database attachment.
+ *
+ **************************************/
+	SET_TDBB(tdbb);
+
+	Attachment* attachment  = tdbb->getAttachment();
+	transaction->tra_attachment = attachment;
+	transaction->tra_next = attachment->att_transactions;
+	attachment->att_transactions = transaction;
+}
+
+
 static void restart_requests(thread_db* tdbb, jrd_tra* trans)
 {
 /**************************************
@@ -2552,7 +2623,7 @@ static void retain_context(thread_db* tdbb, jrd_tra* transaction,
  *
  **************************************/
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* The new transaction needs to remember the 'commit-retained' transaction
@@ -2567,7 +2638,7 @@ static void retain_context(thread_db* tdbb, jrd_tra* transaction,
 /* Create a new transaction lock, inheriting oldest active
    from transaction being committed. */
 
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	SLONG new_number;
 #ifdef SUPERSERVER_V2
 	new_number = bump_transaction_id(tdbb, &window);
@@ -2714,7 +2785,7 @@ static void start_sweeper(thread_db* tdbb, Database* dbb)
 	LCK_release(tdbb, &temp_lock);
 
 	/* allocate space for the string and a null at the end */
-	const char* pszFilename = tdbb->tdbb_attachment->att_filename.c_str();
+	const char* pszFilename = tdbb->getAttachment()->att_filename.c_str();
 
 	char* database = (char*)gds__alloc(strlen(pszFilename) + 1);
 
@@ -2748,7 +2819,6 @@ static THREAD_ENTRY_DECLARE sweep_database(THREAD_ENTRY_PARAM database)
  *	Sweep database.
  *
  **************************************/
-	isc_db_handle db_handle = 0;
 	Firebird::ClumpletWriter dpb(Firebird::ClumpletReader::Tagged, MAX_DPB_SIZE, isc_dpb_version1);
 
 	const char* szAuthenticator = "sweeper";
@@ -2758,14 +2828,15 @@ static THREAD_ENTRY_DECLARE sweep_database(THREAD_ENTRY_PARAM database)
 	dpb.insertString(isc_dpb_password, 
 		szPassword, strlen(szPassword));
 	dpb.insertByte(isc_dpb_sweep, isc_dpb_records);
-	// sometimes security database is also to be sweeped
+	// sometimes security database is also to be swept
 	dpb.insertByte(isc_dpb_gsec_attach, 1);
 
-	ISC_STATUS_ARRAY status_vector;
-
 	// Temporary disable security for this thread to proceed with internal attachment
 	JRD_thread_security_disable(true);
 
+	ISC_STATUS_ARRAY status_vector = {0};
+	isc_db_handle db_handle = 0;
+
 	isc_attach_database(status_vector, 0, (char*)database,
 						&db_handle, dpb.getBufferLength(),
 						reinterpret_cast<const char*>(dpb.getBuffer()));
@@ -2808,24 +2879,36 @@ static void transaction_options(
 	if (*tpb != isc_tpb_version3 && *tpb != isc_tpb_version1)
 		ERR_post(isc_bad_tpb_form, isc_arg_gds, isc_wrotpbver, isc_arg_end);
 
-	bool wait = true, lock_timeout = false;
+	TriState wait, lock_timeout;
+	TriState isolation, read_only, rec_version;
 
 	++tpb;
 
-	while (tpb < end) {
+	while (tpb < end)
+	{
 		const USHORT op = *tpb++;
-		switch (op) {
+		switch (op)
+		{
 		case isc_tpb_consistency:
+			if (!isolation.assignOnce(true))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+				
 			transaction->tra_flags |= TRA_degree3;
 			transaction->tra_flags &= ~TRA_read_committed;
 			break;
 
 		case isc_tpb_concurrency:
+			if (!isolation.assignOnce(true))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+
 			transaction->tra_flags &= ~TRA_degree3;
 			transaction->tra_flags &= ~TRA_read_committed;
 			break;
 
 		case isc_tpb_read_committed:
+			if (!isolation.assignOnce(true))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+
 			transaction->tra_flags &= ~TRA_degree3;
 			transaction->tra_flags |= TRA_read_committed;
 			break;
@@ -2833,33 +2916,53 @@ static void transaction_options(
 		case isc_tpb_shared:
 		case isc_tpb_protected:
 		case isc_tpb_exclusive:
+			ERR_post(isc_bad_tpb_content, isc_arg_end);
 			break;
 
 		case isc_tpb_wait:
+			if (!wait.assignOnce(true))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
 			break;
 
 		case isc_tpb_rec_version:
+			if (isolation.isAssigned() && !(transaction->tra_flags & TRA_read_committed))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+				
+			if (!rec_version.assignOnce(true))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+
 			transaction->tra_flags |= TRA_rec_version;
 			break;
 
 		case isc_tpb_no_rec_version:
+			if (isolation.isAssigned() && !(transaction->tra_flags & TRA_read_committed))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+
+			if (!rec_version.assignOnce(false))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+
 			transaction->tra_flags &= ~TRA_rec_version;
 			break;
 
 		case isc_tpb_nowait:
-			if (lock_timeout)
+			if (lock_timeout.asBool() || !wait.assignOnce(false))
 			{
 				ERR_post(isc_bad_tpb_content, isc_arg_end);
 			}
 			transaction->tra_lock_timeout = 0;
-			wait = false;
 			break;
 
 		case isc_tpb_read:
+			if (!read_only.assignOnce(true))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+
 			transaction->tra_flags |= TRA_readonly;
 			break;
 
 		case isc_tpb_write:
+			if (!read_only.assignOnce(false))
+				ERR_post(isc_bad_tpb_content, isc_arg_end);
+
 			transaction->tra_flags &= ~TRA_readonly;
 			break;
 
@@ -2874,8 +2977,12 @@ static void transaction_options(
 		case isc_tpb_lock_write:
 		case isc_tpb_lock_read:
 			{
-				USHORT l = *tpb++;
-				if (l > MAX_SQL_IDENTIFIER_LEN) {
+				// Do we have space for the identifier length?
+				if (tpb >= end)
+					ERR_post(isc_bad_tpb_form, isc_arg_end);
+					
+				USHORT len = *tpb++;
+				if (len > MAX_SQL_IDENTIFIER_LEN) {
 					TEXT text[BUFFER_TINY];
 					USHORT flags = 0;
 					gds__msg_lookup(0, DYN_MSG_FAC, 159, sizeof(text),
@@ -2884,41 +2991,53 @@ static void transaction_options(
 					ERR_post(isc_bad_tpb_content, isc_arg_gds, isc_random,
 							 isc_arg_string, ERR_cstring(text), isc_arg_end);
 				}
-				Firebird::MetaName name(reinterpret_cast<const char*>(tpb), l);
-				tpb += l;
+				// Does the identifier length surpasses the remaining of the TPB?
+				if (tpb >= end || end - tpb < len)
+					ERR_post(isc_bad_tpb_form, isc_arg_end);
+				
+				const Firebird::MetaName name(reinterpret_cast<const char*>(tpb), len);
+				tpb += len;
 				jrd_rel* relation = MET_lookup_relation(tdbb, name);
 				if (!relation) {
 					ERR_post(isc_bad_tpb_content,
 						 isc_arg_gds, isc_relnotdef, isc_arg_string,
 						 ERR_cstring(name), isc_arg_end);
-			}
-
-			/* force a scan to read view information */
-			MET_scan_relation(tdbb, relation);
-
-			SCHAR lock_type = (op == isc_tpb_lock_read) ? LCK_none : LCK_SW;
-			if (tpb < end) {
-				if (*tpb == isc_tpb_shared)
-					tpb++;
-				else if (*tpb == isc_tpb_protected
-						 || *tpb == isc_tpb_exclusive) 
-				{
-					tpb++;
-					lock_type = (lock_type == LCK_SW) ? LCK_EX : LCK_PR;
 				}
-			}
 
-			expand_view_lock(transaction, relation, lock_type);
-			break;
+				/* force a scan to read view information */
+				MET_scan_relation(tdbb, relation);
+
+				SCHAR lock_type = (op == isc_tpb_lock_read) ? LCK_none : LCK_SW;
+				if (tpb < end)
+				{
+					// Things we don't check yet: that the same table is reserved
+					// for read/write shared and (protected or exclusive).
+					if (*tpb == isc_tpb_shared)
+						tpb++;
+					else if (*tpb == isc_tpb_protected ||
+							 *tpb == isc_tpb_exclusive)
+					{
+						tpb++;
+						lock_type = (lock_type == LCK_SW) ? LCK_EX : LCK_PR;
+					}
+				}
+
+				expand_view_lock(transaction, relation, lock_type);
 			}
+			break;
 
 		case isc_tpb_verb_time:
 		case isc_tpb_commit_time:
 			{
-				const USHORT l = *tpb++;
-				tpb += l;
-				break;
+				// Will be caught at the top of the loop if it goes beyond "end".
+				// Harmless for now even if formally invalid.
+				//if (tpb >= end)
+				//	ERR_post(isc_bad_tpb_form, isc_arg_end);
+
+				const USHORT len = *tpb++;
+				tpb += len;
 			}
+			break;
 
 		case isc_tpb_autocommit:
 			transaction->tra_flags |= TRA_autocommit;
@@ -2930,21 +3049,35 @@ static void transaction_options(
 
 		case isc_tpb_lock_timeout:
 			{
-				if (!wait)
+				if (wait.isAssigned() && !wait.asBool() || !lock_timeout.assignOnce(true))
 				{
 					ERR_post(isc_bad_tpb_content, isc_arg_end);
 				}
-				const USHORT l = *tpb++;
-				transaction->tra_lock_timeout = gds__vax_integer(tpb, l);
-				tpb += l;
-				lock_timeout = true;
-				break;
+				// Do we have space for the identifier length?
+				if (tpb >= end)
+					ERR_post(isc_bad_tpb_form, isc_arg_end);
+
+				const USHORT len = *tpb++;
+				// Does the encoded number's length surpasses the remaining of the TPB?
+				if (tpb >= end || end - tpb < len)
+					ERR_post(isc_bad_tpb_form, isc_arg_end);
+
+				transaction->tra_lock_timeout = gds__vax_integer(tpb, len);
+				tpb += len;
 			}
+			break;
 
 		default:
 			ERR_post(isc_bad_tpb_form, isc_arg_end);
 		}
 	}
+	
+	if (rec_version.isAssigned() &&
+		!(transaction->tra_flags & TRA_read_committed))
+	{
+		ERR_post(isc_bad_tpb_content, isc_arg_end);
+	}
+
 
 /* If there aren't any relation locks to seize, we're done. */
 
@@ -2966,7 +3099,7 @@ static void transaction_options(
 		{
 			continue;
 		}
-		for (USHORT l = 0; l < id; l++) {
+		for (ULONG l = 0; l < id; l++) {
 			if ( (lock = (*vector)[l]) ) {
 				level = lock->lck_logical;
 				LCK_release(tdbb, lock);
