@@ -198,7 +198,7 @@ static const TEXT* const impl_implementation[] = {
     "Firebird/sun",		/* 28 */
     NULL, // "Firebird/OS2",        /* 29 */
     "Firebird/sun4",		/* 30 */
-    NULL, // "Firebird/hpux800",	/* 31 */
+    "Firebird/hpux",		/* 31 */
     "Firebird/sun386",		/* 32 */
     "Firebird:ORACLE/vms",	/* 33 */
     NULL, // "Firebird/mac/aux",    /* 34 */
@@ -240,7 +240,15 @@ static const TEXT* const impl_implementation[] = {
 	"Firebird/Darwin/Intel",	/* 70 */
     "Firebird/linux MIPSEL",	/* 71 */
     "Firebird/linux MIPS",	/* 72 */
-	"Firebird/Darwin/Intel64"	/* 73 */
+	"Firebird/Darwin/Intel64",	/* 73 */
+    "Firebird/sun/amd64",	/* 74 */	
+    "Firebird/linux ARM",	/* 75 */
+    "Firebird/linux IA64",	/* 76 */
+    "Firebird/Darwin/PowerPC64",	/* 77 */
+    "Firebird/linux s390x",             // 78
+    "Firebird/linux s390",              // 79
+    "Firebird/linux SH",                // 80
+    "Firebird/linux SHEB"               // 81
 };
 
 
@@ -1746,10 +1754,10 @@ int API_ROUTINE BLOB_load(ISC_QUAD* blob_id,
 }
 
 
-BSTREAM* API_ROUTINE Bopen(ISC_QUAD* blob_id,
-						   FB_API_HANDLE database, 
-						   FB_API_HANDLE transaction,
-						   const SCHAR* mode)
+StreamPtr API_ROUTINE Bopen(ISC_QUAD* blob_id,
+							FB_API_HANDLE database, 
+							FB_API_HANDLE transaction,
+							const SCHAR* mode)
 {
 /**************************************
  *
@@ -1804,7 +1812,7 @@ BSTREAM* API_ROUTINE Bopen(ISC_QUAD* blob_id,
 
 
 // CVC: This routine doesn't open a blob really!
-BSTREAM* API_ROUTINE BLOB_open(FB_API_HANDLE blob, SCHAR* buffer, int length)
+StreamPtr API_ROUTINE BLOB_open(FB_API_HANDLE blob, SCHAR* buffer, int length)
 {
 /**************************************
  *
