@@ -264,7 +264,7 @@ static int		accept_connection(rem_port*, P_CNCT *);
 #ifdef HAVE_SETITIMER
 static void		alarm_handler(int);
 #endif
-static rem_port*		alloc_port(rem_port*);
+static rem_port*		alloc_port(rem_port*, const USHORT = 0);
 static rem_port*		aux_connect(rem_port*, PACKET*, t_event_ast);
 static rem_port*		aux_request(rem_port*, PACKET*);
 #if !defined(WIN_NT)
@@ -285,10 +285,20 @@ static int fork(void);
 
 #ifdef WIN_NT
 static int fork(SOCKET, USHORT);
+
+static THREAD_ENTRY_DECLARE forkThread(THREAD_ENTRY_PARAM);
+
+static Firebird::Mutex forkMutex;
+static HANDLE forkEvent = INVALID_HANDLE_VALUE;
+static bool forkThreadStarted = false;
+
+typedef Firebird::Array<SOCKET> SocketsArray;
+static SocketsArray* forkSockets;
+
 #endif
 
 static in_addr get_bind_address();
-static in_addr get_host_address(const Firebird::string&);
+static bool get_host_address(const Firebird::string&, in_addr*);
 
 static void copy_p_cnct_repeat_array(	p_cnct::p_cnct_repeat*			pDest,
 										const p_cnct::p_cnct_repeat*	pSource,
@@ -335,7 +345,7 @@ static rem_port*		receive(rem_port*, PACKET *);
 static rem_port*		select_accept(rem_port*);
 
 static rem_port*		select_port(rem_port*, SLCT *);
-static rem_port*        select_multi(rem_port*, UCHAR* buffer, SSHORT bufsize, SSHORT* length);
+static bool		select_multi(rem_port*, UCHAR* buffer, SSHORT bufsize, SSHORT* length, rem_port*& port);
 static int		select_wait(rem_port*, SLCT *);
 static int		send_full(rem_port*, PACKET *);
 static int		send_partial(rem_port*, PACKET *);
@@ -713,8 +723,6 @@ rem_port* INET_connect(const TEXT* name,
 	}
 #else
 
-	TEXT msg[BUFFER_SMALL];
-
 /* U N I X style sockets */
 
 	address.sin_family = AF_INET;
@@ -723,20 +731,16 @@ rem_port* INET_connect(const TEXT* name,
 
 	if (packet) {
 		// client connection
-		host_addr = get_host_address(host);
-
-		if (host_addr.s_addr == INADDR_NONE)
+		if (!get_host_address(host, &host_addr))
 		{
-			SNPRINTF(msg, FB_NELEM(msg),
-					"INET/INET_connect: gethostbyname (%s) failed, error code = %d",
-					host.c_str(), H_ERRNO);
-			gds__log(msg, 0);
+			gds__log("INET/INET_connect: gethostbyname (%s) failed, error code = %d",
+					 host.c_str(), H_ERRNO);
 			inet_gen_error(port,
 						   isc_network_error,
 						   isc_arg_string,
 						   port->port_connection->str_data,
 						   isc_arg_gds,
-						   isc_net_lookup_err, isc_arg_gds, isc_host_unknown, 0);
+						   isc_net_lookup_err, isc_arg_gds, isc_host_unknown, isc_arg_end);
 
 			disconnect(port);
 			return NULL;
@@ -799,10 +803,8 @@ rem_port* INET_connect(const TEXT* name,
 		{
 			/* end of modification by FSG */
 			/* this is the original code */
-			SNPRINTF(msg, FB_NELEM(msg),
-					"INET/INET_connect: getservbyname failed, error code = %d",
-					H_ERRNO);
-			gds__log(msg, 0);
+			gds__log("INET/INET_connect: getservbyname failed, error code = %d",
+					 H_ERRNO);
 			inet_gen_error(port,
 						   isc_network_error,
 						   isc_arg_string,
@@ -813,7 +815,7 @@ rem_port* INET_connect(const TEXT* name,
 						   isc_service_unknown,
 						   isc_arg_string,
 						   protocol.c_str(),
-						   isc_arg_string, "tcp", 0);
+						   isc_arg_string, "tcp", isc_arg_end);
 			return NULL;
 		}						/* else / not hardwired gds_db translation */
 	}
@@ -865,6 +867,12 @@ rem_port* INET_connect(const TEXT* name,
 		lingerInfo.l_onoff = 0;
 		lingerInfo.l_linger = 0;
 
+#ifndef WIN_NT
+		// dimitr:	on Windows, lack of SO_REUSEADDR works the same way as it was specified on POSIX,
+		//			i.e. it allows binding to a port in a TIME_WAIT/FIN_WAIT state. If this option
+		//			is turned on explicitly, then a port can be re-bound regardless of its state,
+		//			e.g. while it's listening. This is surely not what we want.
+
 		int optval = TRUE;
 		n = setsockopt((SOCKET) port->port_handle, SOL_SOCKET, SO_REUSEADDR,
 					   (SCHAR*) &optval, sizeof(optval));
@@ -874,6 +882,7 @@ rem_port* INET_connect(const TEXT* name,
 			disconnect(port);
 			return NULL;
 		}
+#endif
 
 		/* Get any values for SO_LINGER so that they can be reset during
 		 * disconnect.  SO_LINGER should be set by default on the socket
@@ -914,15 +923,16 @@ rem_port* INET_connect(const TEXT* name,
 				sleep(10);
 				n = bind((SOCKET) port->port_handle,
 						 (struct sockaddr *) &address, sizeof(address));
-				if (n == 0)
+				if (n == 0 || INET_ERRNO != INET_ADDR_IN_USE)
 					break;
 			}
 		}
-		else {
-			inet_error(port, "bind", isc_net_connect_listen_err, INET_ERRNO);
-			disconnect(port);
-			return NULL;
-		}
+	}
+
+	if (n == -1) {
+		inet_error(port, "bind", isc_net_connect_listen_err, INET_ERRNO);
+		disconnect(port);
+		return NULL;
 	}
 
 	n = listen((SOCKET) port->port_handle, 5);
@@ -956,7 +966,7 @@ rem_port* INET_connect(const TEXT* name,
 			return NULL;
 		}
 #ifdef WIN_NT
-		if ((flag & SRVR_debug) || !fork(s, flag))
+		if ((flag & SRVR_debug))
 #else
 		if ((flag & SRVR_debug) || !fork())
 #endif
@@ -967,9 +977,39 @@ rem_port* INET_connect(const TEXT* name,
 			port->port_server_flags |= SRVR_server;
 			return port;
 		}
+
+#ifdef WIN_NT
+		Firebird::MutexLockGuard forkGuard(forkMutex);
+		THREAD_ENTER();
+		if (!forkThreadStarted)
+		{
+			forkThreadStarted = true;
+			forkEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
+			forkSockets = new SocketsArray(*getDefaultMemoryPool());
+
+			gds__thread_start(forkThread, (void*) flag, THREAD_medium, 0, 0);
+		}
+		forkSockets->add(s);
+		SetEvent(forkEvent);
+#else
 		THREAD_ENTER();
 		SOCLOSE(s);
+#endif
 	}
+
+#ifdef WIN_NT
+	THREAD_EXIT();
+	Firebird::MutexLockGuard forkGuard(forkMutex);
+	THREAD_ENTER();
+	if (forkThreadStarted)
+	{
+		SetEvent(forkEvent);
+		CloseHandle(forkEvent);
+
+		delete forkSockets;
+		forkSockets = NULL;
+	}
+#endif
 }
 
 rem_port* INET_reconnect(HANDLE handle, ISC_STATUS* status_vector)
@@ -1271,7 +1311,7 @@ static int accept_connection(rem_port* port,
 }
 
 
-static rem_port* alloc_port( rem_port* parent)
+static rem_port* alloc_port( rem_port* parent, const USHORT flags)
 {
 /**************************************
  *
@@ -1290,15 +1330,12 @@ static rem_port* alloc_port( rem_port* parent)
 #ifdef WIN_NT
 	if (!INET_initialized) {
 	    const WORD version = MAKEWORD(2, 0);
-		if (WSAStartup(version, &INET_wsadata)) {
+		const int wsaError = WSAStartup(version, &INET_wsadata);
+		if (wsaError) {
 			if (parent)
-				inet_error(parent, "WSAStartup", isc_net_init_error, INET_ERRNO);
-			else {
-				SNPRINTF(buffer, FB_NELEM(buffer),
-						 "INET/alloc_port: WSAStartup failed, error code = %d",
-						 INET_ERRNO);
-				gds__log(buffer, 0);
-			}
+				inet_error(parent, "WSAStartup", isc_net_init_error, wsaError);
+			else
+				gds__log("INET/alloc_port: WSAStartup failed, error code = %d", wsaError);
 			return NULL;
 		}
 		gds__register_cleanup(exit_handler, 0);
@@ -1316,18 +1353,16 @@ static rem_port* alloc_port( rem_port* parent)
 		}
 		INET_max_data = INET_remote_buffer;
 #ifdef DEBUG
-		{
-			char msg[BUFFER_SMALL];
-			SNPRINTF(msg, FB_NELEM(msg), " Info: Remote Buffer Size set to %ld",
-					 INET_remote_buffer);
-			gds__log(msg, 0);
-		}
+		gds__log(" Info: Remote Buffer Size set to %ld", INET_remote_buffer);
 #endif
 		first_time = false;
 	}
 	rem_port* port = (rem_port*) ALLR_block(type_port, INET_remote_buffer * 2);
 	port->port_type = port_inet;
 	port->port_state = state_pending;
+	port->port_flags = flags;
+	port->port_handle = (HANDLE) INVALID_SOCKET;
+	port->port_channel = INVALID_SOCKET;
 	REMOTE_get_timeout_params(port, 0, 0);
 
 	gethostname(buffer, sizeof(buffer));
@@ -1341,7 +1376,6 @@ static rem_port* alloc_port( rem_port* parent)
 		port->port_parent = parent;
 		port->port_next = parent->port_clients;
 		parent->port_clients = parent->port_next = port;
-		port->port_handle = parent->port_handle;
 		port->port_server = parent->port_server;
 		port->port_server_flags = parent->port_server_flags;
 	}
@@ -1397,9 +1431,51 @@ static rem_port* aux_connect(rem_port* port, PACKET* packet, t_event_ast ast)
 
 	if (port->port_server_flags) {
 	
+		struct timeval timeout;
+		timeout.tv_sec = port->port_connect_timeout;
+		timeout.tv_usec = 0;
+
+		fd_set slct_fdset;
+		FD_ZERO(&slct_fdset);
+		FD_SET(port->port_channel, &slct_fdset);
+
+		int inetErrNo = 0;
+
+		while (true)
+		{
+			THREAD_EXIT();
+			const int count =
+#ifdef WIN_NT
+				select(FD_SETSIZE, &slct_fdset, NULL, NULL, &timeout);
+#else
+				select(port->port_channel + 1, &slct_fdset, NULL, NULL, &timeout);
+#endif
+			inetErrNo = INET_ERRNO;
+			THREAD_ENTER();
+
+			if (count != -1 || !INTERRUPT_ERROR(inetErrNo))
+			{
+				if (count == 1)
+				{
+					break;
+				}
+				else
+				{
+					if (count == 0) 
+						inet_gen_error(port, 
+							isc_random, isc_arg_string, "Timeout occurred while waiting for a secondary connection for event processing", 
+							isc_arg_end);
+					else
+						inet_error(port, "select", isc_net_event_connect_err, inetErrNo);
+					SOCLOSE(port->port_channel);
+					return NULL;
+				}
+			}
+		}
+
 		THREAD_EXIT();
 		SOCKET n = accept(port->port_channel, (struct sockaddr *) &address, &l);
-		const int inetErrNo = INET_ERRNO;
+		inetErrNo = INET_ERRNO;
 		THREAD_ENTER();
 		
 		if (n == INVALID_SOCKET) {
@@ -1517,6 +1593,12 @@ static rem_port* aux_request( rem_port* port, PACKET* packet)
 		return NULL;
 	}
 
+#ifndef WIN_NT
+	// dimitr:	on Windows, lack of SO_REUSEADDR works the same way as it was specified on POSIX,
+	//			i.e. it allows binding to a port in a TIME_WAIT/FIN_WAIT state. If this option
+	//			is turned on explicitly, then a port can be re-bound regardless of its state,
+	//			e.g. while it's listening. This is surely not what we want.
+
 	int optval = TRUE;
 	int ret = setsockopt(n, SOL_SOCKET, SO_REUSEADDR,
 						 (SCHAR *) &optval, sizeof(optval));
@@ -1525,6 +1607,7 @@ static rem_port* aux_request( rem_port* port, PACKET* packet)
 		inet_error(port, "setsockopt REUSE", isc_net_event_listen_err, INET_ERRNO);
 		return NULL;
 	}
+#endif
 
 	if (bind(n, (struct sockaddr *) &address, sizeof(address)) < 0) {
 		inet_error(port, "bind", isc_net_event_listen_err, INET_ERRNO);
@@ -1543,14 +1626,14 @@ static rem_port* aux_request( rem_port* port, PACKET* packet)
 		return NULL;
 	}
 
-    rem_port* new_port = alloc_port(port->port_parent);
+    rem_port* new_port = alloc_port(port->port_parent, PORT_async);
 	port->port_async = new_port;
 	new_port->port_dummy_packet_interval = port->port_dummy_packet_interval;
 	new_port->port_dummy_timeout = new_port->port_dummy_packet_interval;
 
 	new_port->port_server_flags = port->port_server_flags;
 	new_port->port_channel = (int) n;
-	new_port->port_flags = port->port_flags & PORT_no_oob;
+	new_port->port_flags |= port->port_flags & PORT_no_oob;
 
 	P_RESP* response = &packet->p_resp;
 
@@ -1971,12 +2054,17 @@ static int fork( SOCKET old_handle, USHORT flag)
 	GetModuleFileName(NULL, name, sizeof(name));
 
 	HANDLE new_handle;
-	DuplicateHandle(GetCurrentProcess(), (HANDLE) old_handle,
+	if (!DuplicateHandle(GetCurrentProcess(), (HANDLE) old_handle,
 					GetCurrentProcess(), &new_handle, 0, TRUE,
-					DUPLICATE_SAME_ACCESS);
+					DUPLICATE_SAME_ACCESS))
+	{
+		gds__log("INET/inet_error: fork/DuplicateHandle errno = %d", GetLastError());
+		return 0;
+	}
 
 	Firebird::string cmdLine;
-	cmdLine.printf("%s -i -h %"SLONGFORMAT, name, (SLONG) new_handle);
+	cmdLine.printf("%s -i -h %"SLONGFORMAT"@%"SLONGFORMAT, name, (SLONG) new_handle, 
+		GetCurrentProcessId());
 
 	STARTUPINFO start_crud;
 	start_crud.cb = sizeof(STARTUPINFO);
@@ -1988,21 +2076,61 @@ static int fork( SOCKET old_handle, USHORT flag)
 	start_crud.dwFlags = STARTF_FORCEOFFFEEDBACK;
 	
 	PROCESS_INFORMATION pi;
-	if (CreateProcess(NULL, cmdLine.begin(), NULL, NULL, TRUE,
+	if (CreateProcess(NULL, cmdLine.begin(), NULL, NULL, FALSE,
 					  (flag & SRVR_high_priority ?
 						 HIGH_PRIORITY_CLASS | DETACHED_PROCESS :
 						 NORMAL_PRIORITY_CLASS | DETACHED_PROCESS),
 					  NULL, NULL, &start_crud, &pi))
 	{
+		// hvlad: child process will close our handle of just accepted socket 
 		CloseHandle(pi.hThread);
 		CloseHandle(pi.hProcess);
+		return 1;
 	}
-	CloseHandle(new_handle);
 
-	return 1;
+	gds__log("INET/inet_error: fork/CreateProcess errno = %d", GetLastError());
+	CloseHandle(new_handle);
+	return 0;
+}
+
+
+THREAD_ENTRY_DECLARE forkThread(THREAD_ENTRY_PARAM arg)
+{
+	const USHORT flag = (USHORT) arg;
+
+	while (WaitForSingleObject(forkEvent, INFINITE) == WAIT_OBJECT_0)
+	//while (!INET_shutting_down)
+	{
+		while (true) //(!INET_shutting_down)
+		{
+			SOCKET s = 0;
+			{	// scope
+				Firebird::MutexLockGuard forkGuard(forkMutex);
+
+				if (!forkSockets || forkSockets->getCount() == 0)
+					break;
+
+				s = (*forkSockets)[0];
+				forkSockets->remove((size_t) 0);
+			}
+			fork(s, flag);
+			SOCLOSE(s);
+		}
+	}
+
+	return 0;
 }
 #endif
 
+#ifdef WIN_NT
+bool inet_aton(const char* name, in_addr* address)
+{
+	address->s_addr = inet_addr(name);
+	return address->s_addr != INADDR_NONE;
+}
+#endif
+
+
 static in_addr get_bind_address()
 {
 /**************************************
@@ -2018,15 +2146,23 @@ static in_addr get_bind_address()
 	in_addr config_address;
 
 	const char* config_option = Config::getRemoteBindAddress();
-	config_address.s_addr =
-		(config_option) ? inet_addr(config_option) : INADDR_NONE;
-	if (config_address.s_addr == INADDR_NONE) {
+	if (config_option)
+	{
+		if (!inet_aton(config_option, &config_address))
+		{
+			gds__log("Wrong RemoteBindAddress '%s' in firebird.conf - "
+					 "binding to loopback interface", config_option);
+			config_address.s_addr = htonl(INADDR_LOOPBACK);
+		}
+	}
+	else
+	{
 		config_address.s_addr = INADDR_ANY;
 	}
 	return config_address;
 }
 
-static in_addr get_host_address(const Firebird::string& name)
+static bool get_host_address(const Firebird::string& name, in_addr* address)
 {
 /**************************************
  *
@@ -2038,14 +2174,12 @@ static in_addr get_host_address(const Firebird::string& name)
  *	Return host address.
  *
  **************************************/
-	in_addr address;
+	bool rc = true;
 
 	THREAD_EXIT();
 
-	address.s_addr = inet_addr(name.c_str());
-
-	if (address.s_addr == INADDR_NONE) {
-
+	if (!inet_aton(name.c_str(), address)) {
+		rc = false;
 		const hostent* host = gethostbyname(name.c_str());
 
 		/* On Windows NT/9x, gethostbyname can only accomodate
@@ -2066,13 +2200,14 @@ static in_addr get_host_address(const Firebird::string& name)
 		}
 
 		if (host) {
-			inet_copy(host->h_addr, (UCHAR*) &address, sizeof(address));
+			rc = true;
+			inet_copy(host->h_addr, (UCHAR*) address, sizeof(*address));
 		}
 	}
 
 	THREAD_ENTER();
 
-	return address;
+	return rc;
 }
 
 //____________________________________________________________
@@ -2374,7 +2509,7 @@ static rem_port* receive( rem_port* main_port, PACKET * packet)
 #endif //SUPERSERVER
 }
 
-static rem_port* select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSHORT* length)
+static bool select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSHORT* length, rem_port*& port)
 {
 /**************************************
  *
@@ -2390,21 +2525,21 @@ static rem_port* select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize
  *
  **************************************/
 	fb_assert(main_port->port_server_flags & SRVR_multi_client);
-	
+
 	for (;;) 
 	{
-		rem_port* port = select_port(main_port, &INET_select);
+		port = select_port(main_port, &INET_select);
 		if (port == main_port) 
 		{
-			if (port = select_accept(main_port)) 
+			if ( (port = select_accept(main_port)) )
 			{
 				if (!packet_receive(port, buffer, bufsize, length))
 				{
 					*length = 0;
 				}
-				return port;
+				return (*length) ? true : false;
 			}
-			
+
 			continue;
 		}
 		if (port) 
@@ -2413,20 +2548,24 @@ static rem_port* select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize
 			{
 				port->port_dummy_timeout = port->port_dummy_packet_interval;
 				if (port->port_flags & PORT_async ||
-					port->port_protocol < PROTOCOL_VERSION8) continue;
+					port->port_protocol < PROTOCOL_VERSION8)
+				{
+					continue;
+				}
 				*length = 0;
-				return port;
+				return true;
 			}
-			
+
 			if (!packet_receive(port, buffer, bufsize, length))
 			{
 				*length = 0;
 			}
-			return port;
+			return (*length) ? true : false;
 		}
 		if (!select_wait(main_port, &INET_select))
 		{
-			return NULL;
+			port = NULL;
+			return false;
 		}
 	}
 }
@@ -2470,10 +2609,7 @@ static rem_port* select_accept( rem_port* main_port)
 	if (n >= INET_max_clients) {
 		main_port->port_state = state_closed;
 		SOCLOSE((int) main_port->port_handle);
-		TEXT msg[BUFFER_SMALL];
-		SNPRINTF(msg, FB_NELEM(msg),
-				"INET/select_accept: exec new server at client limit: %d", n);
-		gds__log(msg, 0);
+		gds__log("INET/select_accept: exec new server at client limit: %d", n);
 
 		setreuid(0, 0);
 		kill(getppid(), SIGUSR1);
@@ -2541,6 +2677,9 @@ static rem_port* select_port( rem_port* main_port, SLCT * selct)
 	for (rem_port* port = main_port; port; port = port->port_next) {
 		const int n = (int) port->port_handle;
 		if (n < 0 || n >= FD_SETSIZE) {
+			if (port->port_flags & PORT_disconnect) {
+				continue;
+			}
 			STOP_PORT_CRITICAL();
 			return port;
 		}		
@@ -2576,7 +2715,6 @@ static int select_wait( rem_port* main_port, SLCT * selct)
  *	to read from them.
  *
  **************************************/
-	TEXT msg[BUFFER_SMALL];
 	struct timeval timeout;
 	bool checkPorts = false;
 
@@ -2607,8 +2745,9 @@ static int select_wait( rem_port* main_port, SLCT * selct)
 #endif
 		for (rem_port* port = main_port; port; port = port->port_next)
 		{
-			if ((port->port_state == state_active) ||
-				(port->port_state == state_pending))
+			if (((port->port_state == state_active) || (port->port_state == state_pending)) &&
+				// don't wait on still listening (not connected) async port
+				!((SOCKET)port->port_handle == INVALID_SOCKET && port->port_flags & PORT_async) )
 			{
 				/* Adjust down the port's keepalive timer. */
 
@@ -2665,7 +2804,7 @@ static int select_wait( rem_port* main_port, SLCT * selct)
 				++selct->slct_width;
 #else
 				selct->slct_width =
-					MAX(selct->slct_width, (int) port->port_handle);
+					MAX(selct->slct_width, (int) port->port_handle + 1);
 #endif
 				found = true;
 			}
@@ -2675,13 +2814,11 @@ static int select_wait( rem_port* main_port, SLCT * selct)
 
 		if (!found)
 		{
-			gds__log("INET/select_wait: client rundown complete, server exiting",
-				 0);
+			gds__log("INET/select_wait: client rundown complete, server exiting");
 			return FALSE;
 		}
 
 		THREAD_EXIT();
-		++selct->slct_width;
 
 		for (;;)
 		{
@@ -2700,6 +2837,7 @@ static int select_wait( rem_port* main_port, SLCT * selct)
 			const int inetErrNo = INET_ERRNO;
 			if (selct->slct_count != -1)
 			{
+				THREAD_ENTER();
 				/* if selct->slct_count is zero it means that we timed out of
 				   select with nothing to read or accept, so clear the fd_set
 				   bit as this value is undefined on some platforms (eg. HP-UX),
@@ -2707,16 +2845,20 @@ static int select_wait( rem_port* main_port, SLCT * selct)
 				   they can be used in select_port() */
 				if (selct->slct_count == 0)
 				{
+					START_PORT_CRITICAL();
 					for (rem_port* port = main_port; port; port = port->port_next)
 					{
+						if (!( (SOCKET)port->port_handle == INVALID_SOCKET && port->port_flags & PORT_async))
+						{
 #ifdef WIN_NT
-						FD_CLR((SOCKET)port->port_handle, &selct->slct_fdset);
+							FD_CLR((SOCKET)port->port_handle, &selct->slct_fdset);
 #else
-						FD_CLR(port->port_handle, &selct->slct_fdset);
+							FD_CLR(port->port_handle, &selct->slct_fdset);
 #endif
+						}
 					}
+					STOP_PORT_CRITICAL();
 				}
-				THREAD_ENTER();
 				return TRUE;
 			}
 			else if (INTERRUPT_ERROR(inetErrNo))
@@ -2728,10 +2870,7 @@ static int select_wait( rem_port* main_port, SLCT * selct)
 			}
 			else {
 				THREAD_ENTER();
-				SNPRINTF(msg, FB_NELEM(msg),
-						 "INET/select_wait: select failed, errno = %d",
-						 inetErrNo);
-				gds__log(msg, 0);
+				gds__log("INET/select_wait: select failed, errno = %d", inetErrNo);
 				return FALSE;
 			}
 		}	// for (;;)
@@ -3055,7 +3194,6 @@ static void inet_error(
  *	to format the status vector if any.
  *
  **************************************/
-	TEXT msg[BUFFER_SMALL];
 
 	if (status) {
 #ifdef VMS
@@ -3066,18 +3204,16 @@ static void inet_error(
 						   isc_arg_string,
 						   (ISC_STATUS) port->port_connection->str_data,
 						   isc_arg_gds, operation, isc_arg_string,
-						   (ISC_STATUS) win_errlist[status], 0);
+						   (ISC_STATUS) win_errlist[status], isc_arg_end);
 		}
 		else
 #endif
 			inet_gen_error(port, isc_network_error,
 						   isc_arg_string,
 						   (ISC_STATUS) port->port_connection->str_data,
-						   isc_arg_gds, operation, SYS_ERR, status, 0);
+						   isc_arg_gds, operation, SYS_ERR, status, isc_arg_end);
 
-		SNPRINTF(msg, FB_NELEM(msg),
-				 "INET/inet_error: %s errno = %d", function, status);
-		gds__log(msg, 0);
+		gds__log("INET/inet_error: %s errno = %d", function, status);
 	}
 	else {
 		/* No status value, just format the basic arguments. */
@@ -3085,7 +3221,7 @@ static void inet_error(
 		inet_gen_error(port, isc_network_error,
 					   isc_arg_string,
 					   (ISC_STATUS) port->port_connection->str_data, isc_arg_gds,
-					   operation, 0);
+					   operation, isc_arg_end);
 	}
 }
 
@@ -3588,7 +3724,7 @@ static int packet_receive(
 	}
 
 	if (!n) {
-		inet_error(port, "read end_of_file", isc_net_read_err, 0);
+		inet_error(port, "read end_of_file", isc_net_read_err, isc_arg_end);
 		return FALSE;
 	}
 
@@ -3603,7 +3739,7 @@ static int packet_receive(
 		if (INET_force_error == 0) {
 			INET_force_error = 1;
 			inet_error(port, "simulated error - read",
-					   isc_net_read_err, 0);
+					   isc_net_read_err, isc_arg_end);
 			return FALSE;
 		}
 	}
@@ -3748,7 +3884,7 @@ static bool_t packet_send( rem_port* port, const SCHAR* buffer, SSHORT buffer_le
 		if (INET_force_error == 0) {
 			INET_force_error = 1;
 			inet_error(port, "simulated error - send",
-					   isc_net_write_err, 0);
+					   isc_net_write_err, isc_arg_end);
 			return FALSE;
 		}
 	}
