@@ -104,8 +104,6 @@ static xdr_t::xdr_ops wnet_ops =
 	wnet_destroy
 };
 
-const USHORT MAX_PTYPE	= ptype_out_of_band;
-
 
 rem_port* WNET_analyze(Firebird::PathName& file_name,
 					ISC_STATUS*	status_vector,
@@ -136,7 +134,6 @@ rem_port* WNET_analyze(Firebird::PathName& file_name,
 
 /* Pick up some user identification information */
 	Firebird::string buffer;
-	TEXT *p;
 	Firebird::ClumpletWriter user_id(Firebird::ClumpletReader::UnTagged, MAX_DPB_SIZE);
 
 	ISC_get_user(&buffer, 0, 0, 0);
@@ -173,12 +170,13 @@ rem_port* WNET_analyze(Firebird::PathName& file_name,
 
 	static const p_cnct::p_cnct_repeat protocols_to_try1[] =
 	{
-		REMOTE_PROTOCOL(PROTOCOL_VERSION7, ptype_rpc, MAX_PTYPE, 1),
-		REMOTE_PROTOCOL(PROTOCOL_VERSION8, ptype_rpc, MAX_PTYPE, 2),
-		REMOTE_PROTOCOL(PROTOCOL_VERSION10, ptype_rpc, MAX_PTYPE, 3)
+		REMOTE_PROTOCOL(PROTOCOL_VERSION7, ptype_rpc, ptype_batch_send, 1),
+		REMOTE_PROTOCOL(PROTOCOL_VERSION8, ptype_rpc, ptype_batch_send, 2),
+		REMOTE_PROTOCOL(PROTOCOL_VERSION10, ptype_rpc, ptype_batch_send, 3),
+		REMOTE_PROTOCOL(PROTOCOL_VERSION11, ptype_rpc, ptype_batch_send, 4)
 #ifdef SCROLLABLE_CURSORS
 		,
-		REMOTE_PROTOCOL(PROTOCOL_SCROLLABLE_CURSORS, ptype_rpc, MAX_PTYPE, 4)
+		REMOTE_PROTOCOL(PROTOCOL_SCROLLABLE_CURSORS, ptype_rpc, ptype_batch_send, 99)
 #endif
 	};
 	cnct->p_cnct_count = FB_NELEM(protocols_to_try1);
@@ -290,7 +288,8 @@ rem_port* WNET_analyze(Firebird::PathName& file_name,
    string to reflect it...  */
 
 	Firebird::string temp;
-	temp.printf("%s/P%d", port->port_version->str_data, port->port_protocol);
+	temp.printf("%s/P%d", port->port_version->str_data, 
+						  port->port_protocol & FB_PROTOCOL_MASK);
 	ALLR_free(port->port_version);
 	port->port_version = REMOTE_make_string(temp.c_str());
 
@@ -417,10 +416,11 @@ rem_port* WNET_connect(const TEXT*		name,
 		GetModuleFileName(NULL, name, sizeof(name));
 
 		Firebird::string cmdLine;
-		cmdLine.printf("%s -s -w -h %"SLONGFORMAT, name, (SLONG) port->port_handle);
+		cmdLine.printf("%s -w -h %"SLONGFORMAT"@%"SLONGFORMAT, name, (SLONG) port->port_handle, 
+			GetCurrentProcessId());
 
-		STARTUPINFO           start_crud;
-		PROCESS_INFORMATION   pi;
+		STARTUPINFO start_crud;
+		PROCESS_INFORMATION pi;
 		start_crud.cb = sizeof(STARTUPINFO);
 		start_crud.lpReserved = NULL;
 		start_crud.lpReserved2 = NULL;
@@ -428,20 +428,22 @@ rem_port* WNET_connect(const TEXT*		name,
 		start_crud.lpDesktop = NULL;
 		start_crud.lpTitle = NULL;
 		start_crud.dwFlags = STARTF_FORCEOFFFEEDBACK;
-		const USHORT ret = CreateProcess(NULL,
-							cmdLine.begin(),
-							NULL,
-							NULL,
-							TRUE,
-							(flag & SRVR_high_priority ?
+
+		if (CreateProcess(NULL, cmdLine.begin(), NULL, NULL, FALSE,
+						  (flag & SRVR_high_priority ?
 							 HIGH_PRIORITY_CLASS | DETACHED_PROCESS :
 							 NORMAL_PRIORITY_CLASS | DETACHED_PROCESS),
-							NULL, NULL, &start_crud, &pi);
-		if (ret) {
+						  NULL, NULL, &start_crud, &pi))
+		{
+			// hvlad: child process will close our handle of client pipe
 			CloseHandle(pi.hThread);
 			CloseHandle(pi.hProcess);
 		}
-		CloseHandle(port->port_handle);
+		else
+		{
+			gds__log("WNET/inet_error: fork/CreateProcess errno = %d", GetLastError());
+			CloseHandle(port->port_handle);
+		}
 	}
 #endif /* REQUESTER */
 }
